#ifndef ADC_DRV_H
#define ADC_DRV_H

//_____ I N C L U D E S ____________________________________________________

#ifndef  ADHSM
#define  ADHSM   7
#endif
//_____ A D C _ M A C R O S ________________________________________________
#define Enable_adc()                         (ADCSRA |= (1<<ADEN))
#define Disable_adc()                        (ADCSRA &= ~(1<<ADEN))

// Configure the Result alignement
#define Right_adjust_adc_result()            (ADMUX  &= ~(1<<ADLAR))
#define Left_adjust_adc_result()             (ADMUX  |=  (1<<ADLAR))

//Set the high spped mode in case ADC frequency is higher than 200KHz
#define Enable_adc_high_speed_mode()         (ADCSRB |=  (1<<ADHSM))
#define Disable_adc_high_speed_mode()        (ADCSRB &= ~(1<<ADHSM))

//Configure the Vref
#define Enable_internal_vref()               (ADMUX  |=  ((1<<REFS1)|(1<<REFS0)) )
#define Enable_external_vref()               (ADMUX  &= ~((1<<REFS1)|(1<<REFS0)) )
#define Enable_vcc_vref()                    (ADMUX  &= ~(1<<REFS1),          \
                                              ADMUX  |=  (1<<REFS0) )
//! Configure the ADC interrupts
#define Enable_all_int()                      (SREG   |=  (0x80) )
#define Disable_all_int()                     (SREG   &= ~(0x80) )
#define Enable_adc_int()                      (ADCSRA |=  (1<<ADIE) )
#define Disable_adc_int()                     (ADCSRA &= ~(1<<ADIE) )
#define Clear_adc_flag()                     (ADCSRA &=  (1<<ADIF) )

//Configure the ADC prescaler
#define Set_prescaler(prescaler)             (ADCSRA &= ~((1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)),\
                                              ADCSRA |=  (prescaler) )
//Select the ADC channel to be converted
#define Clear_adc_mux()                      (ADMUX  &= ~((1<<MUX3)|(1<<MUX2)|(1<<MUX1)|(1<<MUX0)) )
#define Select_adc_channel(channel)          (Clear_adc_mux(), ADMUX |= (channel) )

//Start the conversion in normal mode
#define Start_conv()                          (ADCSRA |= (1<<ADSC) )
#define Start_conv_channel(channel)           (Select_adc_channel(channel),\
											   Start_conv())
#define Start_amplified_conv()                (ADCSRB |= (1<<ADASCR) )
#define Stop_amplified_conv()                 (ADCSRB &= ~(1<<ADASCR) )
#define Start_amplified_conv_channel(channel) (Select_adc_channel(channel), Start_amplified_conv() )

//Start the Analog to Digital Conversion in noise reduction mode
#define Start_conv_idle()                    (SMCR   |=  (1<<SM0)|(1<<SE) )
#define Start_conv_idle_channel(channel)     (Select_adc_channel(channel), Start_conv_idle() )
#define Clear_sleep_mode()                   (SMCR   &= ~(1<<SM0)|(1<<SE) )

//ADC Get x Bits Result
#define Adc_8bit()               			((u08)(ADCH))
#define Adc_10bit()              			((u16)(ADCL+((u16)(ADCH<<8))))

//Check conversion status
#define Is_adc_conv_finished()               (ADCSRA &  (1<<ADIF)
#define WaitADC()         					 !(ADCSRA &  (1<<ADIF))

#endif  // ADC_DRV_H
