//-----------------------------------------------------------------------------
//   FTDI EEPROM emulation
//   Note the FTDI table is offset from address zero to allow for
//   USB16 flags in the first eeprom entries. The FTDI table is 
//	 128 entries long.
//-----------------------------------------------------------------------------

#include "eeprom.h"
#include "../Gecko_library/include/usb_descriptors.h"
#include <avr/eeprom.h>
unsigned char eeprom[128] EEMEM;

unsigned char ee_ptr;
unsigned short ee_cksum;

void eeprom_append(unsigned char nb)
{
unsigned char pree_ptr;
unsigned int address;
	pree_ptr = ee_ptr & ~1;
	if(pree_ptr != ee_ptr){
		ee_cksum = ee_cksum ^((unsigned short)nb << 8);
		ee_cksum = ee_cksum ^ eeprom_read_byte(&pree_ptr+eeOffset);
		ee_cksum = (ee_cksum << 1) | (ee_cksum >> 15);
	};
	address = ee_ptr+eeOffset;
	eeprom_write_byte((unsigned char*)address,nb);
	ee_ptr++;
	eeprom_busy_wait();
}

void eeprom_init(void)
{
	char StartofString1,StartofString2,StartofString3;
	u08 j;
	ee_ptr = 0x00;
	ee_cksum = 0xAAAA;
	eeprom_append(0x00);//keep these two entries
	eeprom_append(0x00);
	eeprom_append((u08)(Vid&0xFF));
	eeprom_append((u08)(Vid>>8 &0xFF));
	eeprom_append((u08)(Pid&0xFF));
	eeprom_append((u08)(Pid>>8 &0xFF));
	eeprom_append((pgm_read_byte(&usb_dev_desc.bcdDevice))>>8);
	eeprom_append((pgm_read_byte(&usb_dev_desc.bcdDevice))&0xff);

	eeprom_append(pgm_read_byte(conf_desc[0].conf_desc_ptr+7));	//bmAttibutes
	eeprom_append(pgm_read_byte(conf_desc[0].conf_desc_ptr+8));	//MaxPower

	eeprom_append(0x1C);
	eeprom_append(0x00);

	eeprom_append((pgm_read_byte(&usb_dev_desc.bscUSB))>>8);
	eeprom_append((pgm_read_byte(&usb_dev_desc.bscUSB))&0xff);

//	StartofString = 0x80 + ee_ptr + 6;
	StartofString1 = ee_ptr + 6;		//create a list of 6 string and pointer lengths
	eeprom_append(StartofString1);
	eeprom_append(LSB(string_desc[0].size)*2+2);
	StartofString2 = StartofString1+LSB(string_desc[0].size)*2+2;
	eeprom_append(StartofString2);
	eeprom_append(LSB(string_desc[1].size)*2+2);
	StartofString3 = StartofString2+LSB(string_desc[1].size)*2+2;
	eeprom_append(StartofString3);	
	eeprom_append(LSB(string_desc[2].size)*2+2);

	eeprom_append(StartofString1);
	eeprom_append(0x03);	
	for(j=0;j<string_desc[0].size;j++){
	 	eeprom_append(pgm_read_word(&string_desc[0].string[j]));
		eeprom_append(0x00);
	}
	eeprom_append(StartofString2);
	eeprom_append(0x03);
	for(j=0;j<string_desc[1].size;j++){
	 	eeprom_append(pgm_read_word(&string_desc[1].string[j]));
		eeprom_append(0x00);
	}
	eeprom_append(StartofString3);
	eeprom_append(0x03);
	for(j=0;j<string_desc[2].size;j++){
	 	eeprom_append(pgm_read_word(&string_desc[2].string[j]));
		eeprom_append(0x00);
	}
	eeprom_append(pgm_read_word(&usb_dev_desc.iManufacturer));
	eeprom_append(pgm_read_word(&usb_dev_desc.iProduct));
	eeprom_append(pgm_read_word(&usb_dev_desc.iSerialNumber));
	eeprom_append(pgm_read_word(&usb_dev_desc.bNumConfigurations));

	while(ee_ptr < 126) eeprom_append(0);
	eeprom_write_byte((unsigned char*)126+eeOffset,ee_cksum&0xFF);
	eeprom_write_byte((unsigned char*)127+eeOffset,(ee_cksum>>8)&0xFF);
}

