//-----------------------------------------------------------------------------
// Hardware-dependent code for ChameleonV1.5
//-----------------------------------------------------------------------------
#include "../Gecko_library/include/AVR_Chameleon_V1.5.h"//hardware description
#include "../Gecko_library/utility/jtag.h"				//jtag debug description
#include "hw_basic.h"
#include "jtag_map.h"
#include <avr/pgmspace.h>
#include "../Gecko_library/include/stdio_driver.h"		//console i/o

unsigned char WrPtr=0;

//Add byte to OutBuffer modulo 128
void OutputByte(unsigned char data)
{
	JTAG_data++;
   	OutBuffer[WrPtr] = data;
   	WrPtr = (WrPtr+1) & (OUTBUFFER_LEN-1);
   	Pending++;
}

void ProgIO_Init(void)
{
	TDO_input;				//This is the tap input
	TDO_high;				//with pullups
	TDI_low;				//TDI,TMS and TCK are
	TCK_low;				//outputs
	TMS_low;				//
	TDI_output;
	TCK_output;
	TMS_output;
	PgmInit();
}

#ifndef FAST_IO
void ProgIO_Set_State(unsigned char d)
{
// Set state of output pins:
// d.0 => TCK
// d.1 => TMS
// d.2 => nCE not supported
// d.3 => nCS not supported
// d.4 => TDI
// d.5 => LED / Output Enable
//
JTAGclocks++;
	if(d & 1<<0){		//TCK
#ifdef DEBUG
		PrintTDO();
		PrintTDI();
		PrintTMS();
		PrintTCK();
		PrintCR();
		PrintState(TMS_read>>TMS);//print JTAG state on clock high transition
#endif
		TCK_high;
		asm volatile ("nop");
		asm volatile ("nop");
	}else{
		TCK_low;
		asm volatile ("nop");
		asm volatile ("nop");
	};

	if(d & 1<<1){		//TMS
		TMS_high;
	}else{
		TMS_low;
	};

	if(d & 1<<4){		//TDI
		TDI_high;
	}else{
		TDI_low;
	};

	if(d & 1<<5){		//LED
		PgmOn();
	}else{
		PgmOff();
	};
}
#endif					//FAST_IO

unsigned char ProgIO_Set_Get_State(unsigned char d)
{
	ProgIO_Set_State(d);
	return TDO_read>>TDO;	//TDO => d.0 
}




//-----------------------------------------------------------------------------
//Two options exist for shift mode clocking. First there is SPI clocking 
//which is very fast (4 or 8MHz) but it does require the USB16 I/O to be 
//re-mapped for bit-bang mode and the connector. The second mode is a software
//shift routine that has not been optimized. It works with any I/O mapping
//but suffers from being alot slower. Use the following define to select the 
//shift mode clocking.
#ifdef SPITX
void ProgIO_ShiftOut(unsigned char TDI_byte)
{
//	asm volatile ("cli");					//disable interrupts 
   	SPCR |= (1<<SPE);             			//Enable the SPI 
	SPDR = TDI_byte;
	while (!(SPSR & (1<<SPIF)));      		// wait for complete transfer
	SPCR &= ~(1<<SPE);             			//Disable the SPI 
//	asm volatile ("sei");					//enable interrupts
}
#else
void ProgIO_ShiftOut(unsigned char TDI_byte)
{
unsigned char i;				//Shift out TDI_byte: 
	for(i=0;i<8;i++){			//8x{
		JTAGclocks++;
		if(TDI_byte & 0x01){	//Output least significant bit on TDI
			TDI_high;
		}else{
			TDI_low;
		}
#ifdef DEBUG
		PrintTDI();
#endif
		TCK_high;				//Raise TCK
		asm volatile ("nop");
		asm volatile ("nop");
#ifdef DEBUG
		PrintState(TMS_read>>TMS);//print JTAG state on clock high transition
#endif
		TDI_byte = TDI_byte>>1;	//Shift TDI_byte right
		TCK_low;				//Lower TCK
		asm volatile ("nop");
		asm volatile ("nop");
	}							//}
}
#endif 

#ifdef SPIRX
unsigned char ProgIO_ShiftInOut(unsigned char TDI_byte)
{
unsigned char temp;
   	SPCR |= (1<<SPE);             			//Enable the SPI 
	SPDR = TDI_byte;
   	while (!(SPSR & (1<<SPIF)));      		// wait for complete transfer
	temp = SPDR;
	SPCR &= ~(1<<SPE);             			//Disable the SPI 
	return temp;
}

#else
//
//We must set TMS before running this to stay in shift mode

unsigned char ProgIO_ShiftInOut(unsigned char TDI_byte)
{
unsigned char i,TDO_state;		//Shift out TDI_byte, shift in from TDO:
	for(i=0;i<8;i++){			//8x {
		TDO_state = (TDO_read>>TDO);//  Read carry from TDO
#ifdef DEBUG
		PrintTDO();
#endif
		if(TDI_byte &0x01){		//  Output least significant bit on TDI
			TDI_high;
		}else{
			TDI_low;
		}
#ifdef DEBUG
		PrintTDI();
#endif
		TCK_high;				//  Raise TCK
		asm volatile ("nop");
		asm volatile ("nop");
#ifdef DEBUG
		PrintState(TMS_read>>TMS);//print JTAG state on clock high transition
#endif
		TDI_byte =TDI_byte>>1;	//  Shift TDI_byte right, append TDO at left
		if(TDO_state){
			TDI_byte |= 0x80;	//  set MSB
		}else{
			TDI_byte &= 0x7f;	//  clear MSB
		}
		TCK_low;				//  Lower TCK
		asm volatile ("nop");
		asm volatile ("nop");
	}							//
  	return TDI_byte;
}
#endif		//SPI selection

