#!/usr/bin/perl
#This program creates a file called "jtag_map.hex" that maps the UrJTAG
#control word to the USB16 jtag port I/O bits.
#The actual AVR port bits must be defined for TDI,LED,TMS,CS,CE and TCK
#The output file is compatible with the AVR GCC compiler

open(OUTPUT, "> jtag_map.c");

print(OUTPUT "//Create a C source file called [jtag_map.c] that maps the UrJTAG\n");
print(OUTPUT "//control byte to the USB16 jtag port I/O bits.\n");
print(OUTPUT "//The TDO input is set high to enable I/O pullups.\n");
print(OUTPUT "//Don't edit this file since it is created from a perl script.\n");
print(OUTPUT "//Changes will be lost with each new execution of the script.\n");
print(OUTPUT "#include <avr/pgmspace.h>\n\n");

print(OUTPUT "char jtag_map[] PROGMEM = {\n");  


#This is the AVR port I/O definition
$TCK = 1;
$TMS = 0;
$CE  = 6;
$CS  = 7;
$TDI = 2;
$LED = 5;
$TDO = 3;

#This is the UrJTAG command byte definition
$UrTCK = 0;
$UrTMS = 1;
$UrCE  = 2;
$UrCS  = 3;
$UrTDI = 4;
$UrLED = 5;
    
for($i=0; $i<64; $i++){	
	if($UrTCK > $TCK){
		$mapTCK = ((1<<$UrTCK)&$i)>>($UrTCK-$TCK);
	}elsif($UrTCK < $TCK){
		$mapTCK = ((1<<$UrTCK)&$i)<<($TCK-$UrTCK);
	}else{
		$mapTCK = ((1<<$UrTCK)&$i);
	}
	if($UrTMS > $TMS){
		$mapTMS = ((1<<$UrTMS)&$i)>>($UrTMS-$TMS);
	}elsif($UrTMS < $TMS){
		$mapTMS = ((1<<$UrTMS)&$i)<<($TMS-$UrTMS);
	}else{
		$mapTMS = ((1<<$UrTMS)&$i);
	}
	if($UrCS > $CS){
		$mapCS = ((1<<$UrCS)&$i)>>($UrCS-$CS);
	}elsif($UrCS < $CS){
		$mapCS = ((1<<$UrCS)&$i)<<($CS-$UrCS);
	}else{
		$mapCS = ((1<<$UrCS)&$i);
	}	
	if($UrCE > $CE){
		$mapCE = ((1<<$UrCE)&$i)>>($UrCE-$CE);
	}elsif($UrCE < $CE){
		$mapCE = ((1<<$UrCE)&$i)<<($CE-$UrCE);
	}else{
		$mapCE = ((1<<$UrCE)&$i);
	}
	if($UrTDI > $TDI){
		$mapTDI = ((1<<$UrTDI)&$i)>>($UrTDI-$TDI);
	}elsif($UrTDI < $TDI){
		$mapTDI = ((1<<$UrTDI)&$i)<<($TDI-$UrTDI);
	}else{
		$mapTDI = ((1<<$UrTDI)&$i);
	}
	if($UrLED > $LED){
		$mapLED = ((1<<$UrLED)&$i)>>($UrLED-$LED);
	}elsif($UrLED < $LED){
		$mapLED = ((1<<$UrLED)&$i)<<($TCK-$UrLED);
	}else{
		$mapLED = ((1<<$UrLED)&$i);
	}


	$map = $mapTCK | $mapTMS | $mapCS | $mapCE | $mapTDI | $mapLED | 1<<$TDO;
	if($map == 0){
		print(OUTPUT "\t0x00");
	}else{
		printf(OUTPUT "\t%#2.2x",$map);
	}
	if($i < 63){
		print(OUTPUT ",");
	}
	
	if((($i+1 & 0x7)==0)){
		print(OUTPUT "\n");
	}
}
print(OUTPUT " };\n\n");
close(FILE);




