//usb_descriptors.c
//This file contains the usb parameters that uniquely identify the
//UrJTAG application through descriptor tables.
//_____ I N C L U D E S ____________________________________________________

#include "../Gecko_library/include/usb_api.h"
#include "../Gecko_library/include/usb_drv.h"
#include "../Gecko_library/include/usb_descriptors.h"
#include "../Gecko_library/include/usb_standard_request.h"
#include <avr/pgmspace.h>

#undef  Manufacturer
#undef  Product
#undef  Serial
#define Manufacturer 	"Altera"
#define Product			"USB-Blaster"
#define Serial 			"GPL/GPL2"

//_____ U S B    D E S C R I P T O R    T A B L E S ________________________
//#define NB_ENDPOINTS		3  			//number of endpoints in the 
//										//application including control 
#define TX_EP 0x01		//endpoint 
#define RX_EP 0x02		//


// usb device descriptor
S_usb_device_descriptor PROGMEM usb_dev_desc =
{
    sizeof(usb_dev_desc),					// Size of descriptor in bytes
    DEVICE_DESCRIPTOR,				     	// DEVICE descriptor type
    0x0200,									// BCD - USB Spec Release Number 
    0,  	           						// Class Code
    0,                   					// Subclass code
    0,                   					// Protocol code
    EP_CONTROL_LENGTH,	          			// Max size EP0,usb_descriptors.h
  	0x16C0,     							// Vendor ID
	0x06AD,     							// Product ID 
    0x0200,  								// BCD - Device release number
    MAN_INDEX,                   			// Manufacturer string index
    PROD_INDEX,                   			// Product string index
    SN_INDEX,                   			// Serial number string index
    1                    					// Number of configurations
};

// usb configuration descriptor 			
S_usb_user_configuration_descriptor PROGMEM usb_conf_desc = {
 {  sizeof(S_usb_configuration_descriptor),	// Size of descriptor in bytes
    CONFIGURATION_DESCRIPTOR, 			    // CONFIGURATION descriptor type
  	sizeof(usb_conf_desc),      			// Total length of data for cfg
    0x01,                      				// Number of interfaces in cfg
    0x01,                      				// Index value of  configuration
    0,                      				// Configuration string index
    0xA0,        			       			// Attributes, usb_descriptors.h
    100,                     				// Max power consumption (2X mA)
 },
 //Interface 0 Descriptor
 { 	sizeof(S_usb_interface_descriptor), 	// Size of descriptor in bytes
    INTERFACE_DESCRIPTOR,     	        	// INTERFACE descriptor type
    0,                      				// Interface Number
    0,                      				// Alternate Setting Number
    2,                      				// Number of endpoints in intf
    0x00,              						// Class code where CDC ACM=0x02
    0x00, 									// Subclass code 
    0x00, 	                				// Protocol code 
    0,    			                  		// Interface string index
 },
//Usb Endpoint 1 Descriptor IN
 { 	sizeof(S_usb_endpoint_descriptor),		//Size of endpoint in bytes
  	ENDPOINT_DESCRIPTOR,					//
  	(0x80|TX_EP),							//Endpoint number
  	BULK,									//EP transfer type attribute
  	64,										//EP fifo size in bytes
  	0,										//Endpoint interval
 },
 //Usb Endpoint 2 Descriptor OUT
 { 	sizeof(S_usb_endpoint_descriptor),		//Size of endpoint in bytes
  	ENDPOINT_DESCRIPTOR,					//
  	RX_EP,									//Endpoint number
  	BULK,									//EP transfer type attribute
	64,										//EP fifo size in bytes
	0										//Endpoint interval
 }
};

// usb_user_language_id
S_usb_language_descriptor PROGMEM usb_user_language_id = {
  sizeof(usb_user_language_id)
, STRING_DESCRIPTOR
, Language_ID								//LANGUAGE_ID = English
};

S_conf_desc	conf_desc[]={
	{sizeof(usb_conf_desc),
	(u08*)&usb_conf_desc}
};

u08 Manufacturer_string[] PROGMEM = {Manufacturer};
u08 Product_string[] PROGMEM = {Product};
u08 Serial_string[] PROGMEM = {Serial};

S_string_desc 	string_desc[] = {
	{Manufacturer_string,sizeof(Manufacturer)-1},
	{Product_string,sizeof(Product)-1},
	{Serial_string,sizeof(Serial)-1}
};

S_EP_definition EP_definition[] = {
	{NO_INT,BANK_1,0},
//	{NO_INT,BANK_1,TX_EP},
//	{NO_INT,BANK_1,RX_EP}
	{TX_INT,BANK_1,TX_EP},
	{RX_INT,BANK_1,RX_EP}
};

