// CDC.c
//
// This file contains the USB control endpoint functions corresponding
// to the CDC enumeration process. The descriptor tables are contained
// in the usb_descriptors.c file.
#include "include/CDC_api.h"
#include <avr/io.h>
#include <stdio.h>
#include <avr/pgmspace.h>
#include "include/usb_api.h"
#include "include/usb_drv.h"
#include "include/stdio_driver.h"
#include "include/usb_debug.h"
//__________________________________________________________________________
//			CDC Request Processing
//__________________________________________________________________________

u08 usb_CDC_request(u08* type, u08* request)
{	
u08  descriptor_type, string_type;
	debug_ep("usb_CDC_request%c\n",' ');
	debug_ep("(u08)*type 0x%x  ",(u08)*type);	
	debug_ep("(u08)*request 0x%x\n",(u08)*request);	
  	string_type     = LSB(wValue);
	descriptor_type = MSB(wValue);
	
	switch(*request)
	{
		case GET_LINE_CODING:
			cdc_get_line_coding();
			debug_enum("     GET_LINE_CODING%c",' ');
			return TRUE;
  		case SET_LINE_CODING:
			cdc_set_line_coding();
			debug_enum("     SET_LINE_CODING%c",' ');
			return TRUE;
		case SET_CONTROL_LINE_STATE:
			cdc_set_control_line_state();
			debug_enum("     SET_CONTROL_LINE_STATE%c",' ');
      		return TRUE;
		default:
			return FALSE;
	}
}

void CDC_init(void)
{	
	usb_user_request_callback(usb_CDC_request);
	Enable_CDC_flush = 1;
}
//________________________ USB CDC class requests  ________________________
// This function manages reception of line coding parameters 
// (baudrate, bits, stop, parity).
//
void cdc_get_line_coding(void)
{
  	Endpoint_ack_setup_receive();
 	Usb_write_byte(LSB0(line_coding.dwDTERate));
  	Usb_write_byte(LSB1(line_coding.dwDTERate));
  	Usb_write_byte(LSB2(line_coding.dwDTERate));
 	Usb_write_byte(LSB3(line_coding.dwDTERate));
//	Usb_write_u32(line_coding.dwDTERate);
  	Usb_write_byte(line_coding.bCharFormat);
  	Usb_write_byte(line_coding.bParityType);
  	Usb_write_byte(line_coding.bDataBits);

  	Usb_send_control_in();
	debug_ep("cdc_get_line_coding: ep#%d\n",UENUM);
  	while(!(Is_usb_read_control_enabled()));

   	while(!Endpoint_OUT_received_int());
   	Usb_ack_receive_out();
	debug_ep("cdc_get_line_coding ep#%d\n",UENUM);//ep send
#ifdef DEBUG_CDC
	print_line_code();
#endif
}

// This function configures the endpoint line coding
void cdc_set_line_coding (void)
{
   	Endpoint_ack_setup_receive();
   	while (!(Endpoint_OUT_received_int()));
	LSB0(line_coding.dwDTERate) = Usb_read_u08();
   	LSB1(line_coding.dwDTERate) = Usb_read_u08();
   	LSB2(line_coding.dwDTERate) = Usb_read_u08();
   	LSB3(line_coding.dwDTERate) = Usb_read_u08();
//	line_coding.dwDTERate = USBread32;
   	line_coding.bCharFormat = Usb_read_u08();
   	line_coding.bParityType = Usb_read_u08();
   	line_coding.bDataBits = Usb_read_u08();
  	Usb_ack_receive_out();
	debug_ep("cdc_set_line_coding ep#%d\n",UENUM);//ep send
  	Usb_send_control_in();                // send a ZLP for STATUS phase
	debug_ep("cdc_set_line_coding: ep#%d\n",UENUM);
  	while(!(Is_usb_read_control_enabled()));
#ifdef DEBUG_CDC
	print_line_code();
#endif
//	Uart_set_baudrate(line_coding.dwDTERate);
}

// This function manages the SET_CONTROL_LINE_LINE_STATE CDC request.
void cdc_set_control_line_state (void)
{
  	Endpoint_ack_setup_receive();
	Usb_send_control_in();				// send a ZLP for STATUS phase
	debug_ep("cdc_set_control_line_state: ep#%d\n",UENUM);
  	while(!(Is_usb_read_control_enabled()));
}
//__________________________________________________________________________
//                     CDC API
//__________________________________________________________________________
#define USB_STDIO		//<<<<<<<<<<<<<<<<<<<<<<<<<<<
//This function fills the USB transmit buffer with the new data. This buffer
//is sent if complete. To flush this buffer without waiting, use
//the usb_flush() function.
void usb_putchar(u08 data)
{
  	Endpoint_select_endpoint(TX_EP);
  	while(Usb_FIFO_full()){				//Wait for Endpoint ready
		Usb_send_in();
		debug_ep("usb_putchar1 ep#%d\n",UENUM);
	};
	Usb_write_byte(data);	
	if(Usb_FIFO_full()){ 					//If full -> purge fifo
   		Usb_send_in();
		timer2 = usb_flush_time;			//no need to purge for a while
		debug_ep("usb_putchar2 ep#%d\n",UENUM);
  	}
}

void usb_flush(void)
{
  	Endpoint_select_endpoint(TX_EP);
   	Usb_send_in();
//	debug_ep("usb_flush ep#%d\n",UENUM);//omit temporarily to reduce messages.
}

//This function checks if a character has been received on the USB bus.
unsigned char usb_rx_ready(void)
{
  	if (!rx_counter)							//test for rx data
  	{											//no data yet!
    	Endpoint_select_endpoint(RX_EP);		
    	if (Endpoint_OUT_received_int()){					//wait for host data
#if defined __AVR_AT90USB1287__ | __AVR_ATmega32U4__
      		rx_counter = Usb_byte_counter();	//determine packet size
#elif defined __AVR_AT90USB162__	
			rx_counter = Usb_byte_counter();	//determine packet size
#endif
      		if (!rx_counter){					//if zero length packet
        		Usb_ack_receive_out();			//acknowledge packet
				debug_ep("usb_rx_ready ep#%d\n",UENUM);
			}
    	}
  	}
  	return (rx_counter);						//true when data available
}

//This function reads one byte from the USB bus
//NOTE: If one byte is present in the USB fifo, this byte is returned. If 
//no data is present in the USB fifo, this function waits for USB data.
u08 usb_getchar(void)
{
	u08 data_rx;
  	Endpoint_select_endpoint(RX_EP);					//Select RX FIFO endpoint
  	if (!rx_counter) while (!usb_rx_ready());	//wait for Rx data
  	data_rx=Usb_read_u08();						//receive data
  	rx_counter--;								//pkt size - 1
  	if (!rx_counter){ 
		Usb_ack_receive_out();	//ACK packet read
		debug_ep("usb_getchar ep#%d\n",UENUM);
	}
  	return data_rx;
}
 
void usb_PrintfProgStr(const prog_char *pBuf)
{
        char c;
        while ((c=pgm_read_byte(pBuf++))) {
                usb_putchar(c);
        }
}

#ifdef USB_STDIO
//--------------The following two functions provide stdio-------------------
//
// Send usb character to host. Note that the data is only queued to be sent.
// The actual data transfer occurs when either the tx buffer is full,
// the flush timer times out, or a usb_flush() is requested. 
//
int usb_std_putchar(char c, FILE *stream)
{
	usb_putchar(c);
	if (c == '\n'){
    	usb_putchar('\r');}
  	return 0;
}

/*
 * Receive a character from the Usb port.
 *
 * This features a simple line-editor that allows for delete to
 * edit the characters entered, until either CR or NL is entered.
 * Printable characters entered will be echoed using usb_std_putchar().
 *
 * Editing characters:
 *
 * . \b (backspace) or \177 (delete) delete the previous character
 * . \t (tab) will be replaced by a single space
 *
 * All other control characters will be ignored.
 *
 * The internal line buffer is RX_BUFSIZE (20) characters long, which
 * includes the terminating \n (but no terminating \0).  If the buffer
 * is full (i. e., at RX_BUFSIZE-1 characters in order to keep space for
 * the trailing \n), any further input attempts will send a \a 
 * (BEL character), although line editing is still
 * allowed.
 *
 * Successive calls to usb_getchar() will be satisfied from the
 * internal buffer until that buffer is emptied again.
 */
#define RX_BUFSIZE 	20
 
u08 *rxp = 0;
int usb_std_getchar(FILE *stream)
{
	u08 RxChar, *CharPtr, TextBuffer[RX_BUFSIZE];

  	if (rxp == 0)
    	for (CharPtr = TextBuffer;;){
			RxChar = usb_getchar();
			if (RxChar == CR)// behaviour similar to Unix stty ICRNL 
	  			RxChar = new_line;
			if (RxChar == new_line){					//CR to newline
	    		*CharPtr = RxChar;
	    		usb_std_putchar(RxChar, stream);
	    		rxp = TextBuffer;
	    		break;
	  		}else if(RxChar == tab)					//tab to space
	  			RxChar = ' ';

			if ((RxChar >= (u08)' ' && RxChar <= (u08)'\x7e') ||
	    	RxChar >= (u08)'\xa0'){
	    		if (CharPtr == TextBuffer + RX_BUFSIZE - 1)
	      		usb_std_putchar(bell, stream);			//no room in buffer
	    	else{
				*CharPtr++ = RxChar;					//add ch to buffer
				usb_std_putchar(RxChar, stream);		//echo character
	      	}
	    	continue;
	  	}
		if((RxChar == '\b') | (RxChar == '\x7f')){	//backspace or del
	    	if (CharPtr > TextBuffer){
				usb_std_putchar(backspace, stream);
				usb_std_putchar(space, stream);
				usb_std_putchar(backspace, stream);
				CharPtr--;
	      	}
	    }
   	}

  	RxChar = *rxp++;									//last char in buffer
  	if (RxChar == new_line){
    	rxp = 0;
	}
  	return RxChar;
}
#endif
