//usb_composite_descriptors.c
//This file contains the usb parameters that uniquely identify the
//USBHID16 application through descriptor tables.

//_____ I N C L U D E S ____________________________________________________
#include "../include/usb_descriptors.h"
#include "../include/usb_standard_request.h"
#include "../include/HID_report.h"
#include <avr/pgmspace.h>

//Product Strings
//To custommize the product strings to your application, the strings 
//defined by the library must be undefined and replaced with your
//custom string as shown here. These strings are stored in Flash
//memory as bytes to conserve SRAM and Flash. The strings get converted
//to unicode by the "usb_handle_request()" code in the standard_library.
#undef  Manufacturer
#undef  Product
#undef  Serial
#define Manufacturer 	"USB16"
#define Product			"HID composite demo"
#define Serial 			"GPL/GPL2"

//_____ U S B    D E S C R I P T O R    T A B L E S ________________________

//__________ USB MOUSE REPORT ______________________________________________
//The mouse report consists of a four byte report defined as follows
//byte 1 	- bit 0 left click
//			  bit 1 middle click
//			  bit 2 right click
//			  bit 3-7 are zero
//byte 2	- mouse X co-ordinate defaults to 0
//byte 3	- mouse Y co-ordinate defaults to 0
//byte 4	- wheel value defaults to 0
//
u08 PROGMEM Usb_mouse_report[] = {
      Usage_page Generic_desktop,			// 
      Usage_Mouse,          				//
	  Collection Application,      			//
	 		Usage Pointer,          		//
      		Collection Physical,      		//
      			Usage_page (Button),    	//
      			Usage_minimum (1),      	//
      			Usage_maximum (3),      	//define 3 bit report
      			Logical_minimum (0),    	//define clicks as on or off
      			Logical_maximum (1),    	//
      			Report_size (1),        	//click size is one byte
      			Report_count (3),       	//3 bits used
      			Input 0x02,          		//(Data, variable, Absolute)
     			Report_size (5),        	//5 bits are zero
      			Report_count (1),       	//
      			Input (Constant),          	//Constant
      			Usage_page (Generic_desktop),// 
				Usage_X,					//X
				Usage_Y,					//Y
				Usage_Wheel,				//Wheel
				Logical_minimum (-127),		//X,Y,wheel results -127 to 127
				Logical_maximum (127),		//
				Report_size (8),			//X,Y,wheel result = 8 bits
				Report_count (3),			//3 byte report
				Input 0x06,					//(data, variable, relative)
      		End_collection,         		//
      End_collection                		//
};	
//__________ USB KEYBOARD REPORT ___________________________________________
//This is the keyboard HID with a 2 byte report
//The first byte defines the key modifier while the second byte defines the 
//actual key code. 
//The key modifiers include shift, alt and ctrl
//The key code itself has a value of 0-101 which defines the 101 keyboard
//Report: (modifier byte, keycode byte)
u08 PROGMEM Usb_keyboard_report[0x25] = {
	Usage_page (Generic_desktop),			//Start report
	Usage_Keyboard,          				// Usage (Keyboard)
	Collection Application,         		//
		Usage_page (Keyboard),      		//
		Usage_minimum (224),				//
		Usage_maximum (231),				//
		Logical_minimum (0),				//
		Logical_maximum (1),				//
		Report_size (1),					//1 byte report
		Report_count (8),					//
		Input 0x02,          				//Input(Data,Variable,Absolute)
		Usage_minimum (0),         			//
		Usage_maximum (101),        		//
		Logical_minimum (0),        		//
		Logical_maximum (101),      		//
		Report_size (8),          			//8 bit byte
		Report_count (1),					//one return code
		Input 0x00,          				// Input (Data, Array)
	End_collection              			// End Collection
};

// usb_user_device_descriptor
S_usb_device_descriptor PROGMEM usb_dev_desc =
{
    sizeof(usb_dev_desc),   				// Descriptor size in bytes
    DEVICE_DESCRIPTOR,      				// DEVICE descriptor type
    0x0200,									// USB BCD Release Number
    0,      	       						// Class Code
    0,                   					// Subclass code
    0,                   					// Protocol code
    64,          							// Max packet size for EP0
    0x03EB,       							// Vendor ID
	0x1020,      							// Product ID: HID Demo
    0x1000,  								// BCD Device release number
    MAN_INDEX,                   			// Manufacturer string index
    PROD_INDEX,                   			// Product string index
    SN_INDEX,                   			// Serial number string index
    1                    					// Number of configurations
};

// usb configuration descriptor 			// HID CONFIGURATION
S_usb_HID_composite_descriptor  PROGMEM usb_conf_desc = 
{
 {  sizeof(S_usb_configuration_descriptor),	// Descriptor size in bytes
    CONFIGURATION_DESCRIPTOR,          	 	// Descriptor type
  	sizeof(usb_conf_desc),				    // Length of data for this cfg
    2,                      				// Number of interfaces in cfg
    1,                      				// Index for this configuration
    0,                      				// Configuration string index
    CONF_ATTRIBUTES,               			// Attributes
    50,                     				// Max power consumption (2X mA)
 },
 //Interface 0 Descriptor Mouse
 { 	sizeof(S_usb_interface_descriptor), 	// Descriptor size in bytes
    INTERFACE_DESCRIPTOR,               	// Descriptor type
    0,                      				// Interface Number
    0,                      				// Alternate Setting Number
    1,                      				// Number of endpoints in intf
    0x03,              						// Class code where HID =0x03
    0x00, 									// Subclass 0=none 1=kbd 2=mouse
    0x02,                 					// Protocol 0=none 1=kbd 2=mouse
    0,                      				// Interface string index
 },
 //Usb HID Descriptor
 { 	sizeof(S_usb_hid_descriptor),			//
  	0x21,									//HID descriptor
  	0x1001,									//HID_BCD
 	0,										//Country code
 	1,										//Class descriptor number
  	0x22,									//Descriptor type
	sizeof(Usb_mouse_report)
 },

 //Usb Endpoint Descriptor
 { 	sizeof(S_usb_endpoint_descriptor),		//
  	ENDPOINT_DESCRIPTOR,					//
  	(EP1 | 0x80),							//ENDPOINT_NB
  	INTR,									//EP_ATTRIBUTES
 	8,										//EP size in bytes
  	0x02,									//EP_INTERVAL
 },

 //Interface 1 Descriptor Keyboard
 { 	sizeof(S_usb_interface_descriptor), 	// Descriptor size in bytes
    INTERFACE_DESCRIPTOR,               	// Descriptor type
    1,                      				// Interface Number
    0,                      				// Alternate Setting Number
    1,                      				// Number of endpoints in intf
    0x03,              						// Class code where HID =0x03
    0x00, 									// Subclass 0=none 1=kbd 2=mouse
    0x01,                 					// Protocol 0=none 1=kbd 2=mouse
    0,                      				// Interface string index
 },
 //Usb HID Descriptor
 { 	sizeof(S_usb_hid_descriptor),			//
  	0x21,									//HID descriptor
  	0x1001,									//HID_BCD
 	0,										//Country code
 	1,										//Class descriptor number
  	0x22,									//Descriptor type
	sizeof(Usb_keyboard_report)
 },

 //Usb Endpoint Descriptor
 { 	sizeof(S_usb_endpoint_descriptor),		//
  	ENDPOINT_DESCRIPTOR,					//
  	(EP2 | 0x80),							//ENDPOINT_NB
  	INTR,									//EP_ATTRIBUTES
 	8,										//EP size in bytes
  	0x02,									//EP_INTERVAL
 }
};

//This is a fixed size data structure that points to the actual data 
//structures used by the library. This allows the user application an
//easy way to define custom structures. Because this structure is a
//fixed size it can be statically compiled into the library even 
//though the actual configuration descriptor has a variable size. 
//The library defaults to using its internal "descriptor.c" file but
//this can be overwritten by compiling your own "descriptor.c" with your
//code. 
S_conf_desc	conf_desc[]={
	{sizeof(usb_conf_desc),
	(u08*)&usb_conf_desc}
};

// usb_user_language_id
S_usb_language_descriptor PROGMEM usb_user_language_id = {
	sizeof(usb_user_language_id),
 	STRING_DESCRIPTOR,
	0x0409
};

//This is a fixed size data structure that points to the actual data
//structures used by the library. Each HID report is described by this
//structure. First the size of the report is given, then a pointer to
//the actual structure is provided. Note that your "usb_descriptor.c"
//must define an entry for each HID report in your application.
S_hid_report HID_report[]= {
	{sizeof(Usb_mouse_report),Usb_mouse_report},
	{sizeof(Usb_keyboard_report),Usb_keyboard_report}
};

//Define the memory strings to be coded into Flash memory
u08 Manufacturer_string[] PROGMEM = {Manufacturer};
u08 Product_string[] PROGMEM = {Product};
u08 Serial_string[] PROGMEM = {Serial};

//This is a fixed size data structure that points to the actual data
//structures used by the library. Note that the order of this entry
//shouldn't be changes as it is hard coded into the libary. Again the
//entry consists of a size followed by a pointer to the actual string.
S_string_desc 	string_desc[] = {
	{Manufacturer_string,sizeof(Manufacturer)-1},
	{Product_string,sizeof(Product)-1},
	{Serial_string,sizeof(Serial)-1}
};

//For each endpoint defined as part of the configuration descriptor
//you must define the endpoints operation. Speciffically you can
//define whether the endpoint callback is enabled, how many hardware
//banks each endpoint has and whether Rx interrupts are used. This 
//is done for all enpoints. The order of definition should be from
//the lowest to highest endpoint number. 
S_EP_definition EP_definition[] = {
	{NO_INT,BANK_1,0},
	{NO_INT,BANK_1,EP1},
	{NO_INT,BANK_1,EP2}
};
