//Chameleon_V1.0a.h
#ifndef _Chameleon_V1_0a_H_
#define _Chameleon_V1_0a_H_
#include <avr/io.h> 

//______________________Crystal definition________________________________ 
#ifndef F_CPU
#define F_CPU 16000000
#endif

//_______________________SPI definition___________________________________

#define 	SPIDDR	DDRB
#define 	SPIPort	PORTB
#define 	SPIPin	PINB

#define 	RST		0
#define		SCK		1
#define 	MOSI	2
#define		MISO	3	

#define		RST_input	SPIDDR &= ~(1<<RST);
#define		RST_output	SPIDDR |= 1<<RST;
#define		RST_low		SPIPort &= ~(1<<RST);
#define 	RST_high	SPIPort |= 1<<RST;
#define		RST_read	(SPIPin & (1<<RST))

#define		MISO_input	SPIDDR &= ~(1<<MISO);
#define		MISO_output	SPIDDR |= 1<<MISO;
#define		MISO_low	SPIPort &= ~(1<<MISO);
#define 	MISO_high	SPIPort |= 1<<MISO;
#define		MISO_read	(SPIPin & (1<<MISO))

#define		MOSI_input	SPIDDR &= ~(1<<MOSI);
#define		MOSI_output	SPIDDR |= 1<<MOSI;
#define		MOSI_low	SPIPort &= ~(1<<MOSI);
#define 	MOSI_high	SPIPort |= 1<<MOSI;
#define		MOSI_read	(SPIPin & (1<<MOSI))

#define		SCK_input	SPIDDR &= ~(1<<SCK);
#define		SCK_output	SPIDDR |= 1<<SCK;
#define		SCK_low		SPIPort &= ~(1<<SCK);
#define 	SCK_high	SPIPort |= 1<<SCK;
#define		SCK_read	(SPIPin & (1<<SCK))

//______________________ Power LED definition_____________________________
//#define Pgm 			4					//port bit for anode
//#define PgmC 			6					//port bit for cathode
#define Pgm 			7					//port bit for anode
#define PgmC 			7					//port bit for cathode
#define PgmPort			PORTC				//actual port for Led 
#define PgmDDR			DDRC

#define PgmInit()		PgmDDR |= 1<<Pgm | 1<<PgmC;
#define PgmOn()			PgmPort |= 1<<Pgm;\
						PgmPort &= ~(1<<PgmC);
#define PgmOff()		PgmPort &= ~(1<<Pgm);
#define PgmToggle()		PgmPort ^= 1<<Pgm;

//______________________ Power LED definition_____________________________
#define Pwr 			5					//port bit for anode
#define PwrPort			PORTC				//actual port for Led 
#define PwrDDR			DDRC

#define PwrInit()		PwrDDR |= 1<<Pwr;
#define PwrOn()			PwrPort |= 1<<Pwr;
#define PwrOff()		PwrPort &= ~(1<<Pwr);
#define PwrToggle()		PwrPort ^= 1<<Pwr;

#define HeartBeatPwr()	PwrOn();\
						_delay_ms(150);\
						PwrOff();	\
						_delay_ms(150);\
						PwrOn();\
						_delay_ms(600);\
						PwrOff();\
						_delay_ms(150);
#define PulsePwr(On,Off)PwrOn();\
						_delay_ms(On);\
						PwrOff();	\
						_delay_ms(Off);	


#define DisplayPwrOn()  DisplayLED(0,100,100,'C',Pwr,0xff,0)
#define DisplayPwrOff() DisplayLED(0,  0,100,'C',Pwr,0xff,0)

//______________________ Photosensor definition___________________________ 
#define SensorDDR		DDRC
#define SensorPORT		PORTC
#define SensorPIN		PINC
#define sensor 			1<<PgmC

//______________________Boot Vector definition____________________________  
#define BOOT_START		FLASHEND/2-4095		//4K boot sector
#define BOOT_END		(FLASHEND/2-1)

#if(FLASH_END == 0x1FFFF)
	#define BOOT_END 	0xFFFE
#elif(FLASH_END == 0x3FFF)
	#define BOOT_END 	0x3FFE
#endif

//______________________Uart definition___________________________________ 
//Define a basic set of uart regiters and map them to the actual AT90USB16
#define SerialPort		PORTD
#define SerialDDR		DDRD
#define SerialPIN		PIND
#define RXD				2
#define TXD				3
#define UBRRH			UBRR1H
#define UBRRL			UBRR1H
#define RXC				RXC1
#define TXC				TXC1
#define TXEN			TXEN1
#define RXEN			RXEN1
#define RXCIE			RXCIE1
#define UCSRA			UCSR1A 
#define UCSRB			UCSR1B
#define UDRE			UDRE1 
#define UDR				UDR1

//___________________Timer 0 definition___________________________________ 
//Define a basic set of uart regiters and map them to the actual AT90USB16

#define TIFR		TIFR1
#define TIMSK		TIMSK1
#define	TICIE1		ICIE1

//___________________________JTAG definition_____________________________
#define 	TMS		5		
#define		TCK		6
#define 	TDI		4
#define		TDO		2
#define  	JPORT	PORTC
#define 	JDDR	DDRC
#define 	JPIN	PINC


#define 	TMS_0	0
#define 	TMS_1	1
#define		TMS_input	JDDR &= ~(1<<TMS);
#define		TMS_output	JDDR |= 1<<TMS;
#define		TMS_low		JPORT &= ~(1<<TMS);
#define 	TMS_high	JPORT |= 1<<TMS;
#define		TMS_read	(JPIN & (1<<TMS))

#define		TCK_input	JDDR &= ~(1<<TCK);
#define		TCK_output	JDDR |= 1<<TCK;
#define		TCK_low		JPORT &= ~(1<<TCK);
#define 	TCK_high	JPORT |= 1<<TCK;
#define		TCK_read	(JPIN & (1<<TCK))

#define		TDI_input	JDDR &= ~(1<<TDI);
#define		TDI_output	JDDR |= 1<<TDI;
#define		TDI_low		JPORT &= ~(1<<TDI);
#define 	TDI_high	JPORT |= 1<<TDI;
#define		TDI_read	(JPIN & (1<<TDI))

#define		TDO_input	JDDR &= ~(1<<TDO);
#define		TDO_output	JDDR |= 1<<TDO;
#define		TDO_low		JPORT &= ~(1<<TDO);
#define 	TDO_high	JPORT |= 1<<TDO;
#define		TDO_read	(JPIN & (1<<TDO))

//______________________Signature definition____________________________
#define 	SignatureDDR	DDRD
#define 	SignaturePORT	PORTD
#define 	SignaturePIN	PIND

#define 	EN				1
#define 	SO				0
#define 	SIGCK			4


#define 	InitSignature()	SignatureDDR |= 1<<EN | 1<<SIGCK;\
							SignatureDDR &= ~(1<<SO);\
							SignaturePORT &= ~(1<<SIGCK);
#define 	Enable_High()	SignaturePORT |= 1<<EN;
#define 	Enable_Low()	SignaturePORT &= ~(1<<EN);
#define		EN_read			(SignaturePIN & (1<<EN))

#define 	SIGCK_Low()		SignaturePORT &= ~(1<<SIGCK);
#define 	SIGCK_High() 	SignaturePORT |= 1<<SIGCK;
#define 	ResetSignature() SignaturePORT |= 1<<EN;\
							SignaturePORT &= ~(1<<EN);
#define 	SO_read()		(PIND >> SO)&0x01;

#endif //_Chameleon_V1_0a_H_
