// CDC_api.h
//
// This file contains the control endpoint functions corresponding to
// the standard CDC process. The enumeration parameters (descriptor tables) 
// are contained in the usb_descriptors.c file.

#ifndef _CDC_API_H_
#define _CDC_API_H_

//_____ I N C L U D E S ____________________________________________________
#include "usb_descriptors.h"
#include "usb_standard_request.h"

//CDC Requests
#define SEND_ENCAPSULATED_COMMAND 			0x00
#define GET_ENCAPSULATED_COMMAND  			0x01
#define SET_LINE_CODING           			0x20
#define GET_LINE_CODING           			0x21
#define SET_CONTROL_LINE_STATE    			0x22
#define SEND_BREAK                			0x23

//_____ D E C L A R A T I O N ______________________________________________
typedef struct {
	u32		dwDTERate;  	//data terminal rate, in bits per second
	u08    	bCharFormat;  	//num of stop bits (0=1, 1=1.5, 2=2)
	u08    	bParityType;  	//parity (0=none, 1=odd, 2=even, 3=mark, 4=space)
	u08    	bDataBits;    	//data bits (5,6,7,8 or 16)
}S_line_coding;
S_line_coding line_coding;
Bool usb_user_read_request(u08 type, u08 request);
u08 bmRequestType;
u16 wValue,wIndex,wLength;
u08	 bmRequest;


//CDC internal functions
void    usb_process_request( void);
void cdc_get_line_coding(void);
void cdc_set_line_coding (void);
void cdc_set_control_line_state (void);
void usb_user_request_callback(void * fct);

//CDC API functions
void CDC_init(void);
void usb_putchar(u08 data);
void usb_PrintfProgStr(const prog_char *pBuf);
unsigned char usb_rx_ready(void);
u08 usb_getchar(void);
int usb_std_putchar(char c, FILE *stream);
int usb_std_getchar(FILE *stream);
void usb_flush(void);
extern u08 Rx_buf[];//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#endif  // _CDC_API_H_

