//AVR_Chameleon_V1.1.h
// 		JTAG connections	
//	PB0	TCK		1	2	VCCB1
//	PB1	TDO		3	4	VCCB2
//	PB2	TMS		5	6	RX
//	PB3	JSEL	7	8	TX
//	PB4	TDI		9	10	GNDB1
//	 			11	12	
//		TX		13	14	GNDB1
//
//  PB5 TRST
//  PB6 SRST
// 
//  PF7 = power
//  PD1 = PGM

#include <avr/io.h> 

//___________________USB16 Crystal definition_____________________________ 
#ifndef F_CPU
#define F_CPU 16000000
#endif

//___________________USB16 SPI definition_________________________________
#define 	SPIDDR	DDRB
#define 	SPIPort	PORTB
#define 	SPIPin	PINB
#define 	RST		7
#define		SCK		1
#define 	MOSI	2
#define		MISO	3	

#define		RST_input	SPIDDR &= ~(1<<RST);
#define		RST_output	SPIDDR |= 1<<RST;
#define		RST_low		SPIPort &= ~(1<<RST);
#define 	RST_high	SPIPort |= 1<<RST;
#define		RST_read	(SPIPin & (1<<RST))

#define		MISO_input	SPIDDR &= ~(1<<MISO);
#define		MISO_output	SPIDDR |= 1<<MISO;
#define		MISO_low	SPIPort &= ~(1<<MISO);
#define 	MISO_high	SPIPort |= 1<<MISO;
#define		MISO_read	(SPIPin & (1<<MISO))

#define		MOSI_input	SPIDDR &= ~(1<<MOSI);
#define		MOSI_output	SPIDDR |= 1<<MOSI;
#define		MOSI_low	SPIPort &= ~(1<<MOSI);
#define 	MOSI_high	SPIPort |= 1<<MOSI;
#define		MOSI_read	(SPIPin & (1<<MOSI))

#define		SCK_input	SPIDDR &= ~(1<<SCK);
#define		SCK_output	SPIDDR |= 1<<SCK;
#define		SCK_low		SPIPort &= ~(1<<SCK);
#define 	SCK_high	SPIPort |= 1<<SCK;
#define		SCK_read	(SPIPin & (1<<SCK))

//___________________USB16 Power LED definition___________________________
#define Pgm 			1					//port bit for anode
#define PgmC 			0					//port bit for cathode
#define PgmPort			PORTD				//actual port for Led 
#define PgmDDR			DDRD
#define PgmPIN			PIND

#define PgmInit()		PgmDDR |= 1<<Pgm | 1 <<PgmC;
#define PgmOn()			PgmPort |= 1<<Pgm;\
						PgmPort &= ~(1<<PgmC);
#define PgmOff()		PgmPort &= ~(1<<Pgm);
#define Pgm_read		(PgmPIN & 1<<Pgm)
#define PgmToggle()		PgmPort ^= 1<<Pgm;

//___________________USB16 Power LED definition___________________________
#define Pwr 			7					//port bit for anode
#define PwrPort			PORTF				//actual port for Led 
#define PwrDDR			DDRF
#define PwrPIN			PINF

#define PwrInit()		PwrDDR |= 1<<Pwr;\
						MCUCR |= 1<<JTD;\
						MCUCR |= 1<<JTD;			//disable jtag on led bit
#define PwrOn()			PwrPort |= 1<<Pwr;
#define PwrOff()		PwrPort &= ~(1<<Pwr);
#define Pwr_read		(PwrPIN & 1<<Pgm)
#define PwrToggle()		PwrPort ^= 1<<Pwr;

#define HeartBeatPwr()	PwrOn();\
						_delay_ms(150);\
						PwrOff();	\
						_delay_ms(150);\
						PwrOn();\
						_delay_ms(600);\
						PwrOff();\
						_delay_ms(150);
#define PulsePwr(On,Off)PwrOn();\
						_delay_ms(On);\
						PwrOff();	\
						_delay_ms(Off);	


#define DisplayPwrOn()  DisplayLED(0,100,100,'C',Pwr,0xff,0)
#define DisplayPwrOff() DisplayLED(0,  0,100,'C',Pwr,0xff,0)

//___________________Target Boot & Reset definition_______________________
#define TargetRst 		5					//bit that is toggled for rst-
#define TargetRstPort	PORTC				//rst- output port
#define TargetRstDDR	DDRC
#define TargetHWB 		3					//bit that is toggled for HWB
#define TargetHWBPort	PORTB				//HWB output port
#define TargetHWBDDR	DDRB

#define	TargetRst_input		TargetRstDDR &= ~(1<<TargetRst);
#define	TargetRst_output	TargetRstDDR |= 1<<TargetRst;
#define TargetRst_high		TargetRstPort |= 1<<TargetRst;
#define TargetRst_low		TargetRstPort &= ~(1<<TargetRst);

#define	TargetHWB_input		TargetHWBDDR &= ~(1<<TargetHWB);
#define	TargetHWB_output	TargetHWBDDR |= 1<<TargetHWB;
#define TargetHWB_high		TargetHWBPort |= 1<<TargetHWB;
#define TargetHWB_low		TargetHWBPort &= ~(1<<TargetHWB);

//___________________USB16 Photosensor definition_________________________ 
#define SensorDDR		DDRD
#define SensorPORT		PORTD
#define SensorPIN		PIND
#define sensor 			1<<PGMC

//___________________USB16 Boot Vector definition_________________________  
#define BOOT_START		FLASHEND/2-4095		//4K boot sector
#define BOOT_END		(FLASHEND/2-1)

#if(FLASH_END == 0x1FFFF)
	#define BOOT_END 	0xFFFE
#elif(FLASH_END == 0x3FFF)
	#define BOOT_END 	0x3FFE
#endif

//___________________USB16 Uart definition________________________________ 
//Define a basic set of uart regiters and map them to the actual AT90USB16
/*#define SerialPort		PORTD
#define SerialDDR		DDRD
#define SerialPIN		PIND
#define RXD				2
#define TXD				3
#define UBRRH			UBRR1H
#define UBRRL			UBRR1L
#define RXC				RXC1
#define TXC				TXC1
#define TXEN			TXEN1
#define RXEN			RXEN1
#define RXCIE			RXCIE1
#define UCSRA			UCSR1A 
#define UCSRB			UCSR1B
#define UDRE			UDRE1 
#define UDR				UDR1*/

//___________________Timer 0 definition___________________________________ 
//Define a basic set of uart regiters and map them to the actual AT90USB16

#define TIFR		TIFR1
#define TIMSK		TIMSK1
#define	TICIE1		ICIE1

//___________________JTAG definition____________________
		
#define		TCK		0
#define 	TMS		2
#define		TDO		1
#define 	TDI		4
#define 	JSEL	3
#define 	TRST	5
#define 	SRST	6

#define  	JPORT	PORTB
#define 	JDDR	DDRB
#define 	JPIN	PINB


#define 	TMS_0	0
#define 	TMS_1	1
#define		TMS_input	JDDR &= ~(1<<TMS);
#define		TMS_output	JDDR |= 1<<TMS;
#define		TMS_low		JPORT &= ~(1<<TMS);
#define 	TMS_high	JPORT |= 1<<TMS;
#define		TMS_read	(JPIN & (1<<TMS))

#define		TCK_input	JDDR &= ~(1<<TCK);
#define		TCK_output	JDDR |= 1<<TCK;
#define		TCK_low		JPORT &= ~(1<<TCK);
#define 	TCK_high	JPORT |= 1<<TCK;
#define		TCK_read	(JPIN & (1<<TCK))

#define		TDI_input	JDDR &= ~(1<<TDI);
#define		TDI_output	JDDR |= 1<<TDI;
#define		TDI_low		JPORT &= ~(1<<TDI);
#define 	TDI_high	JPORT |= 1<<TDI;
#define		TDI_read	(JPIN & (1<<TDI))

#define		TDO_input	JDDR &= ~(1<<TDO);
#define		TDO_output	JDDR |= 1<<TDO;
#define		TDO_low		JPORT &= ~(1<<TDO);
#define 	TDO_high	JPORT |= 1<<TDO;
#define		TDO_read	(JPIN & (1<<TDO))

#define		JSEL_input	JDDR &= ~(1<<JSEL);
#define		JSEL_output	JDDR |= 1<<JSEL;
#define		JSEL_low		JPORT &= ~(1<<JSEL);
#define 	JSEL_high	JPORT |= 1<<JSEL;
#define		JSEL_read	(JPIN & (1<<JSEL))

#define		TRST_input	JDDR &= ~(1<<TRST);
#define		TRST_output	JDDR |= 1<<TRST;
#define		TRST_low	JPORT &= ~(1<<TRST);
#define 	TRST_high	JPORT |= 1<<TRST;
#define		TRST_read	(JPIN & (1<<TRST))

#define		SRST_input	JDDR &= ~(1<<SRST);
#define		SRST_output	JDDR |= 1<<SRST;
#define		SRST_low	JPORT &= ~(1<<SRST);
#define 	SRST_high	JPORT |= 1<<SRST;
#define		SRST_read	(JPIN & (1<<SRST))

//________________Signature definition__________________
#define 	SignatureDDR	DDRF
#define 	SignaturePORT	PORTF
#define 	SignaturePIN	PINF

#define 	EN				1
#define 	SO				0
#define 	SIGCK			4


#define 	InitSignature()	SignatureDDR |= 1<<EN | 1<<SIGCK;\
							SignatureDDR &= ~(1<<SO);\
							SignaturePORT &= ~(1<<SIGCK);
#define 	Enable_High()	SignaturePORT |= 1<<EN;
#define 	Enable_Low()	SignaturePORT &= ~(1<<EN);
#define		EN_read			(SignaturePIN & (1<<EN))

#define 	SIGCK_Low()		SignaturePORT &= ~(1<<SIGCK);
#define 	SIGCK_High() 	SignaturePORT |= 1<<SIGCK;
#define 	ResetSignature() SignaturePORT |= 1<<EN;\
							SignaturePORT &= ~(1<<EN);
#define 	SO_read()		(SignaturePIN >> SO)&0x01;

