//LA8_V1.0.h

#include <avr/io.h> 

//___________________USB16 Crystal definition_____________________________ 
#ifndef F_CPU
#define F_CPU 16000000
#endif

//___________________HWB definition_______________________________________
#define 	HWB			7
#define		F_RST		1

#define		FPGA_output		DDRD |= 1<<F_RST;
#define		FPGA_RST_low	PORTD &= ~(1<<F_RST);
#define 	FPGA_RST_high	PORTD |= 1<<F_RST;

#define		HWB_init	DDRD &= ~(1<<HWB);\
						PORTD |= 1<<HWB;
#define		HWB_input	DDRD &= ~(1<<HWB);
#define		HWB_output	DDRD |= 1<<HWB;
#define		HWB_low		PORTD &= ~(1<<HWB);
#define 	HWB_high	PORTD |= 1<<HWB;
#define		HWB_read	(PIND & 1<<HWB)

//___________________USB16 SPI definition_________________________________
#define 	RST		0
#define		SCK		1
#define 	MOSI	2
#define		MISO	3	

#define		RST_input	DDRB &= ~(1<<RST);
#define		RST_output	DDRB |= 1<<RST;
#define		RST_low		PORTB &= ~(1<<RST);
#define 	RST_high	PORTB |= 1<<RST;
#define		RST_read	(PINB & (1<<RST))

#define		MISO_input	DDRB &= ~(1<<MISO);
#define		MISO_output	DDRB |= 1<<MISO;
#define		MISO_low	PORTB &= ~(1<<MISO);
#define 	MISO_high	PORTB |= 1<<MISO;
#define		MISO_read	(PINB & (1<<MISO))

#define		MOSI_input	DDRB &= ~(1<<MOSI);
#define		MOSI_output	DDRB |= 1<<MOSI;
#define		MOSI_low	PORTB &= ~(1<<MOSI);
#define 	MOSI_high	PORTB |= 1<<MOSI;
#define		MOSI_read	(PINB & (1<<MOSI))

#define		SCK_input	DDRB &= ~(1<<SCK);
#define		SCK_output	DDRB |= 1<<SCK;
#define		SCK_low		PORTB &= ~(1<<SCK);
#define 	SCK_high	PORTB |= 1<<SCK;
#define		SCK_read	(PINB & (1<<SCK))

//___________________     PGM LED definition   ___________________________
#define Pgm 			4					//port bit for anode
#define PgmPort			PORTB				//actual port for Led 
#define PgmDDR			DDRB

#define PgmInit()		PgmDDR |= 1<<Pgm;
#define PgmOn()			PgmPort |= 1<<Pgm;
#define PgmOff()		PgmPort &= ~(1<<Pgm);
#define PgmToggle()		PgmPort ^= 1<<Pgm;

//___________________USB16 Boot Vector definition_________________________  
#define BOOT_START		FLASHEND/2-4095		//4K boot sector
#define BOOT_END		(FLASHEND/2-1)

#if(FLASH_END == 0x1FFFF)
	#define BOOT_END 	0xFFFE
#elif(FLASH_END == 0x3FFF)
	#define BOOT_END 	0x3FFE
#endif

//___________________USB16 Uart definition________________________________ 
//Define a basic set of uart regiters and map them to the actual AT90USB16
#define SerialPort		PORTD
#define SerialDDR		DDRD
#define SerialPIN		PIND
#define RXD				2
#define TXD				3
#define UBRRH			UBRR1H
#define UBRRL			UBRR1H
#define RXC				RXC1
#define TXC				TXC1
#define TXEN			TXEN1
#define RXEN			RXEN1
#define RXCIE			RXCIE1
#define UCSRA			UCSR1A 
#define UCSRB			UCSR1B
#define UDRE			UDRE1 
#define UDR				UDR1

//___________________Timer 0 definition___________________________________ 
//Define a basic set of uart regiters and map them to the actual AT90USB16

#define TIFR		TIFR1
#define TIMSK		TIMSK1
#define	TICIE1		ICIE1

//___________________JTAG definition____________________
#define 	TMS		0		
#define		TCK		1
#define 	TDI		2
#define		TDO		3
#define  	JPORT	PORTB
#define 	JDDR	DDRB
#define 	JPIN	PINB


#define 	TMS_0	0
#define 	TMS_1	1
#define		TMS_input	JDDR &= ~(1<<TMS);
#define		TMS_output	JDDR |= 1<<TMS;
#define		TMS_low		JPORT &= ~(1<<TMS);
#define 	TMS_high	JPORT |= 1<<TMS;
#define		TMS_read	(JPIN & (1<<TMS))

#define		TCK_input	JDDR &= ~(1<<TCK);
#define		TCK_output	JDDR |= 1<<TCK;
#define		TCK_low		JPORT &= ~(1<<TCK);
#define 	TCK_high	JPORT |= 1<<TCK;
#define		TCK_read	(JPIN & (1<<TCK))

#define		TDI_input	JDDR &= ~(1<<TDI);
#define		TDI_output	JDDR |= 1<<TDI;
#define		TDI_low		JPORT &= ~(1<<TDI);
#define 	TDI_high	JPORT |= 1<<TDI;
#define		TDI_read	(JPIN & (1<<TDI))

#define		TDO_input	JDDR &= ~(1<<TDO);
#define		TDO_output	JDDR |= 1<<TDO;
#define		TDO_low		JPORT &= ~(1<<TDO);
#define 	TDO_high	JPORT |= 1<<TDO;
#define		TDO_read	(JPIN & (1<<TDO))

//___________________FPGA definition____________________
#define 	M0			7
#define		M1			6
#define 	M2			5
#define		PROG_B		4
#define		INIT_B		2
#define		DONE		0
#define  	MPORT		PORTB
#define 	MDDR		DDRB
#define 	MPIN		PINB
#define  	PROG_PORT	PORTC
#define 	PROG_DDR	DDRC
#define 	PROG_PIN	PINC
#define  	DONE_PORT	PORTC
#define 	DONE_DDR	DDRC
#define 	DONE_PIN	PINC


#define		Mbits_Init	MDDR |= 1<<M0 | 1<<M1 | 1<<M2;
#define		M0_low		MPORT &= ~(1<<M0);
#define 	M0_high		MPORT |= 1<<M0;
#define		M1_low		MPORT &= ~(1<<M1);
#define 	M1_high		MPORT |= 1<<M1;
#define		M2_low		MPORT &= ~(1<<M2);
#define 	M2_high		MPORT |= 1<<M2;

#define 	Internal_SPI MPORT |= 1<<M0 | 1<<M1;\
						MPORT &= ~(1<<M2);
#define 	JTAG_mode	MPORT |= 1<<M0 | 1<<M2;\
						MPORT &= ~(1<<M1);

#define		FPGA_Init	MDDR |= 1<<M0 | 1<<M1 | 1<<M2;\
						PROG_DDR &= ~(1<<PROG_B | 1<<INIT_B);\
						PROG_PORT |= 1<<PROG_B | 1<<INIT_B;\
						DONE_DDR &= ~(1<<DONE);\
						DONE_PORT |= 1<<DONE;

#define		DONE_input	DONE_DDR &= ~(1<<DONE);
#define		DONE_output	DONE_DDR |= 1<<DONE;
#define		DONE_low	DONE_PORT &= ~(1<<DONE);
#define 	DONE_high	DONE_PORT |= 1<<DONE;
#define		DONE_read	(DONE_PIN & (1<<DONE))

#define		Prog_B_output	PROG_DDR |= 1<<PROG_B;
#define		Prog_B_low		PROG_PORT &= ~(1<<PROG_B);
#define		Prog_B_high		PROG_PORT |= 1<<PROG_B;

#define		Init_B_input	PROG_DDR &= ~(1<<INIT_B);
#define		Init_B_output	PROG_DDR |= 1<<INIT_B;
#define		Init_B_low		PROG_PORT &= ~(1<<INIT_B);
#define 	Init_B_high		PROG_PORT |= 1<<INIT_B;

#define		FPGA_RST	PROG_PORT &= ~(1<<PROG_B);\
						asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");\
						PROG_PORT |= 1<<PROG_B;
