//LA8_V1.1.h

#include <avr/io.h> 

//___________________USB16 Crystal definition_____________________________ 
#ifndef F_CPU
#define F_CPU 16000000
#endif

//___________________HWB definition_______________________________________
//
//HWB is always an input with a pullup
#define 	HWB			1
#define		HWB_init	DDRE &= ~(1<<HWB);\
						PORTE |= 1<<HWB;

//___________________SPI definition_______________________________________
#define		SCK		1
#define 	MOSI	2
#define		MISO	3	

#define		MISO_input	DDRB &= ~(1<<MISO);
#define		MISO_output	DDRB |= 1<<MISO;
#define		MISO_low	PORTB &= ~(1<<MISO);
#define 	MISO_high	PORTB |= 1<<MISO;
#define		MISO_read	(PINB & (1<<MISO))

#define		MOSI_input	DDRB &= ~(1<<MOSI);
#define		MOSI_output	DDRB |= 1<<MOSI;
#define		MOSI_low	PORTB &= ~(1<<MOSI);
#define 	MOSI_high	PORTB |= 1<<MOSI;
#define		MOSI_read	(PINB & (1<<MOSI))

#define		SCK_input	DDRB &= ~(1<<SCK);
#define		SCK_output	DDRB |= 1<<SCK;
#define		SCK_low		PORTB &= ~(1<<SCK);
#define 	SCK_high	PORTB |= 1<<SCK;
#define		SCK_read	(PINB & (1<<SCK))

//_______________________ LED definition _________________________________
#define Pgm1 			1					//port bit for anode (Blue)
#define Pgm2 			2					//port bit for anode (Yellow)
#define PgmPort			PORTB				//actual port for Led 
#define PgmDDR			DDRB

#define PgmInit()		PgmDDR |= 1<<Pgm1;
#define PgmOn()			PgmPort |= 1<<Pgm1;
#define PgmOff()		PgmPort &= ~(1<<Pgm1);
#define PgmToggle()		PgmPort ^= 1<<Pgm1;

#define Pgm1Init()		PgmDDR |= 1<<Pgm1;
#define Pgm1On()		PgmPort |= 1<<Pgm1;
#define Pgm1Off()		PgmPort &= ~(1<<Pgm1);
#define Pgm1Toggle()	PgmPort ^= 1<<Pgm1;

#define Pgm2Init()		PgmDDR |= 1<<Pgm2;
#define Pgm2On()		PgmPort |= 1<<Pgm2;
#define Pgm2Off()		PgmPort &= ~(1<<Pgm2);
#define Pgm2Toggle()	PgmPort ^= 1<<Pgm2;

//_________________________Boot Vector definition_________________________  

//????????????????????????
#define BOOT_START		FLASHEND/2-4095		//4K boot sector
#define BOOT_END		(FLASHEND/2-1)

#if(FLASH_END == 0x1FFFF)
	#define BOOT_END 	0xFFFE
#elif(FLASH_END == 0x3FFF)
	#define BOOT_END 	0x3FFE
#endif

//___________________Uart definition______________________________________ 
//Define a basic set of uart regiters and map them to the processor
#define SerialPort		PORTD
#define SerialDDR		DDRD
#define SerialPIN		PIND
#define RXD				2
#define TXD				3
#define UBRRH			UBRR1H
#define UBRRL			UBRR1H
#define RXC				RXC1
#define TXC				TXC1
#define TXEN			TXEN1
#define RXEN			RXEN1
#define RXCIE			RXCIE1
#define UCSRA			UCSR1A 
#define UCSRB			UCSR1B
#define UDRE			UDRE1 
#define UDR				UDR1

//___________________Timer 0 definition___________________________________ 
//Define a basic set of uart regiters and map them to the processor

#define TIFR		TIFR1
#define TIMSK		TIMSK1
#define	TICIE1		ICIE1

//___________________JTAG definition______________________________________
#define 	TMS		4		
#define		TCK		5
#define 	TDI		6
#define		TDO		7
#define  	JPORT	PORTD
#define 	JDDR	DDRD
#define 	JPIN	PIND


#define 	TMS_0	0
#define 	TMS_1	1
#define		TMS_input	JDDR &= ~(1<<TMS);
#define		TMS_output	JDDR |= 1<<TMS;
#define		TMS_low		JPORT &= ~(1<<TMS);
#define 	TMS_high	JPORT |= 1<<TMS;
#define		TMS_read	(JPIN & (1<<TMS))

#define		TCK_input	JDDR &= ~(1<<TCK);
#define		TCK_output	JDDR |= 1<<TCK;
#define		TCK_low		JPORT &= ~(1<<TCK);
#define 	TCK_high	JPORT |= 1<<TCK;
#define		TCK_read	(JPIN & (1<<TCK))

#define		TDI_input	JDDR &= ~(1<<TDI);
#define		TDI_output	JDDR |= 1<<TDI;
#define		TDI_low		JPORT &= ~(1<<TDI);
#define 	TDI_high	JPORT |= 1<<TDI;
#define		TDI_read	(JPIN & (1<<TDI))

#define		TDO_input	JDDR &= ~(1<<TDO);
#define		TDO_output	JDDR |= 1<<TDO;
#define		TDO_low		JPORT &= ~(1<<TDO);
#define 	TDO_high	JPORT |= 1<<TDO;
#define		TDO_read	(JPIN & (1<<TDO))





//___________________JTAG definition____________________
		
#define 	JSEL	3
#define 	TRST	5
#define 	SRST	6

#define  	JPORT	PORTD
#define 	JDDR	DDRD
#define 	JPIN	PIND


#define		JSEL_input	JDDR &= ~(1<<JSEL);
#define		JSEL_output	JDDR |= 1<<JSEL;
#define		JSEL_low		JPORT &= ~(1<<JSEL);
#define 	JSEL_high	JPORT |= 1<<JSEL;
#define		JSEL_read	(JPIN & (1<<JSEL))

#define		TRST_input	JDDR &= ~(1<<TRST);
#define		TRST_output	JDDR |= 1<<TRST;
#define		TRST_low	JPORT &= ~(1<<TRST);
#define 	TRST_high	JPORT |= 1<<TRST;
#define		TRST_read	(JPIN & (1<<TRST))

#define		SRST_input	JDDR &= ~(1<<SRST);
#define		SRST_output	JDDR |= 1<<SRST;
#define		SRST_low	JPORT &= ~(1<<SRST);
#define 	SRST_high	JPORT |= 1<<SRST;
#define		SRST_read	(JPIN & (1<<SRST))






//___________________FPGA definition______________________________________
#define		PROG_B		1
#define  	PROG_PORT	PORTF
#define 	PROG_DDR	DDRF
#define 	PROG_PIN	PINF	
#define		RST			0

#define 	DisableJTAG()	MCUCR = 1<<JTD;\
							MCUCR = 1<<JTD;


#define		RSTDDR			DDRF
#define		RSTPORT			PORTF

#define		Prog_B_output	DisableJTAG();\
							PROG_DDR |= 1<<PROG_B;
#define		Prog_B_low		DisableJTAG();\
							PROG_PORT &= ~(1<<PROG_B);
#define		Prog_B_high		DisableJTAG();\
							PROG_PORT |= 1<<PROG_B;

#define 	Load_FPGA()		Prog_B_output;\
							Prog_B_low;\
							asm volatile ("nop");\
							asm volatile ("nop");\
							asm volatile ("nop");\
							asm volatile ("nop");\
							asm volatile ("nop");\
							Prog_B_high;

#define 	FPGA_reset()	DisableJTAG();\
							DDRF |= (1<<FRST);\
							PORTF |= (1<<FRST);\
							asm volatile ("nop");\
							asm volatile ("nop");\
							PORTF &= ~(1<<FRST);\
							asm volatile ("nop");\
							asm volatile ("nop");\
							PORTF |= (1<<FRST);\
							asm volatile ("nop");\
							asm volatile ("nop");



#define SOP_INT	1		//PB1 SOP valid 
#define RDY		3		//PB3 FIFO has data
#define RD		6		//PF6
#define FRST 	0		//PF0
#define BIT754	(PINF&(1<<7 | 1<<5 | 1<<4))
#define BIT6	((PINB&(1<<2))<<4)	//PB2=data6
#define BIT3210	(PIND&0x0F)
#define DATA	BIT754 | BIT6 | BIT3210

unsigned volatile char rdy,sop;
unsigned int waiting;

#define FIFOEMPTY 		!(PINB & 1<<RDY)

/*#define Wait_for_SOP()	sop = (PINB & 1<<SOP_INT);\
						while(!sop){\
							answer[0] = ReadFIFO();\
							sop = PINB & 1<<SOP_INT;\
						};*/
#define Wait_for_SOP()	while((PINB & 1<<SOP_INT)==0){\
							answer[0] = ReadFIFO();\
						};


#define WaitforRdy()	rdy = PINB & 1<<RDY;\
						while(!rdy){\
							rdy = PINB & 1<<RDY;\
							waiting++;\
							if(waiting == 1000){\
								waiting = 0;\
								printf_P(PSTR("."));\
							}\
						};


#define USB_Analyzer_Init()\
						DDRB &= ~(1<<RDY | 1<<2 | 1<< SOP_INT);\
						DDRD &= ~(1<<3 | 1<<2 | 1<<1 | 1<<0);\
						DDRF &= ~(1<<7 | 1<<5 | 1<<4);\
						DDRF |= 1<<RD;

/*#define FIFO_Rd()		current = PIND & 0x03;\
						PORTD &= ~(1<<RD);\
						asm volatile ("nop");\
						PORTD |= (1<<RD);\
						asm volatile ("nop");\
						PORTD &= ~(1<<RD);*/

						
//___________________ I2C definition______________________________________		
#define		SCL		0
#define		SCA		1
#define		I2CDDR	DDRD
#define		I2CPORT	PORTD	

//________________Signature definition__________________
#define 	SignatureDDR	DDRB
#define 	SignaturePORT	PORTB
#define 	SignaturePIN	PINB

#define 	EN				2
#define 	SO				3
#define 	SIGCK			1


#define 	InitSignature()	SignatureDDR |= 1<<EN | 1<<SIGCK;\
							SignatureDDR &= ~(1<<SO);\
							SignaturePORT &= ~(1<<SIGCK);
#define 	Enable_High()	SignaturePORT |= 1<<EN;
#define 	Enable_Low()	SignaturePORT &= ~(1<<EN);
#define		EN_read			(SignaturePIN & (1<<EN))

#define 	SIGCK_Low()		SignaturePORT &= ~(1<<SIGCK);
#define 	SIGCK_High() 	SignaturePORT |= 1<<SIGCK;
#define 	ResetSignature() SignaturePORT &= ~(1<<SIGCK);\
							SignaturePORT |= 1<<EN;\
							asm volatile("nop");\
							SignaturePORT &= ~(1<<EN);\
							asm volatile("nop");
#define 	SO_read()		(SignaturePIN >> SO)&0x01;		

	
