//Newt_V1.1.h

#include <avr/io.h> 
#include <util/delay.h> 
//______________________Crystal definition________________________________ 
#ifndef F_CPU
#define F_CPU 16000000
#endif

//________________________SPI definition__________________________________
//Disable TDO when MISO is used
//Disable TCK when SCK is used
//Disable TDI when MOSI is used
//Disable TMS when SEL is used

#define 	SPIDDR	DDRB
#define 	SPIPort	PORTB
#define 	SPIPin	PINB
#define		SEL		0
#define		SCK		1
#define 	MOSI	2
#define		MISO	3	

#define		SEL_input	SPIDDR &= ~(1<<SEL);	JDDR &= ~(1<<TMS);
#define		SEL_output	SPIDDR |= 1<<SEL;		JDDR &= ~(1<<TMS);
#define		SEL_low		SPIPort &= ~(1<<SEL);	JPORT &= ~(1<<TMS);
#define 	SEL_high	SPIPort |= 1<<SEL;		JPORT &= ~(1<<TMS);
#define		SEL_read	(SPIPin & (1<<SEL))		

#define		MISO_input	SPIDDR &= ~(1<<MISO);	JDDR &= ~(1<<TDO);
#define		MISO_output	SPIDDR |= 1<<MISO;		JDDR &= ~(1<<TDO);
#define		MISO_low	SPIPort &= ~(1<<MISO);	JPORT &= ~(1<<TDO);
#define 	MISO_high	SPIPort |= 1<<MISO;		JPORT &= ~(1<<TDO);
#define		MISO_read	(SPIPin & (1<<MISO))

#define		MOSI_input	SPIDDR &= ~(1<<MOSI);	JDDR &= ~(1<<TDI);
#define		MOSI_output	SPIDDR |= 1<<MOSI;		JDDR &= ~(1<<TDI);
#define		MOSI_low	SPIPort &= ~(1<<MOSI);	JPORT &= ~(1<<TDI);
#define 	MOSI_high	SPIPort |= 1<<MOSI;		JPORT &= ~(1<<TDI);
#define		MOSI_read	(SPIPin & (1<<MOSI))

#define		SCK_input	SPIDDR &= ~(1<<SCK);	JDDR &= ~(1<<TCK);
#define		SCK_output	SPIDDR |= 1<<SCK;		JDDR &= ~(1<<TCK);
#define		SCK_low		SPIPort &= ~(1<<SCK);	JPORT &= ~(1<<TCK);
#define 	SCK_high	SPIPort |= 1<<SCK;		JPORT &= ~(1<<TCK);
#define		SCK_read	(SPIPin & (1<<SCK))

//_________________________LED definition_________________________________
#define USB 			6					//port bit for anode
#define USBPort			PORTC				//actual port for Led 
#define USBDDR			DDRC

#define USBOn()			USBDDR |= 1<<USB;	USBPort |= 1<<USB;
#define USBOff()		USBDDR |= 1<<USB;	USBPort &= ~(1<<USB);
#define USBToggle()		USBDDR |= 1<<USB;	USBPort ^= 1<<USB;

#define Mode 			6					//port bit for anode
#define ModePort		PORTE				//actual port for Led 
#define ModeDDR			DDRE

#define ModeOn()		ModeDDR |= 1<<Mode;	ModePort |= 1<<Mode;
#define ModeOff()		ModeDDR |= 1<<Mode;	ModePort &= ~(1<<Mode);
#define ModeToggle()	ModeDDR |= 1<<Mode;	ModePort ^= 1<<Mode;

#define Power 			7					//port bit for anode
#define PowerPort		PORTC				//actual port for Led 
#define PowerDDR		DDRC

#define PowerOn()		PowerDDR |= 1<<Power;	PowerPort |= 1<<Power;
#define PowerOff()		PowerDDR |= 1<<Power;	PowerPort &= ~(1<<Power);
#define PowerToggle()	PowerDDR |= 1<<Power;	PowerPort ^= 1<<Power;
#define Select_5_0()	PowerDDR |= 1<<Power;	PowerPort |= 1<<Power;
#define Select_3_3()	PowerDDR |= 1<<Power;	PowerPort &= ~(1<<Power);
#define ReadVoutState()	(PINC&(1<<Power))

//________________________ Boot Vector definition_________________________  
#define BOOT_START		FLASHEND/2-4095		//4K boot sector
#define BOOT_END		(FLASHEND/2-1)

#if(FLASH_END == 0x1FFFF)
	#define BOOT_END 	0xFFFE
#elif(FLASH_END == 0x3FFF)
	#define BOOT_END 	0x3FFE
#endif

//________________________ Uart definition________________________________ 
//Define a basic set of uart regiters and map them to the actual Mega32U4
#define SerialPort		PORTD
#define SerialDDR		DDRD
#define SerialPIN		PIND
#define RXD				2
#define TXD				3
#define UBRRH			UBRR1H
#define UBRRL			UBRR1L
#define RXC				RXC1
#define TXC				TXC1
#define TXEN			TXEN1
#define RXEN			RXEN1
#define RXCIE			RXCIE1
#define UCSRA			UCSR1A 
#define UCSRB			UCSR1B
#define UDRE			UDRE1 
#define UDR				UDR1

//Disable I2C when serial data is selected
#define UartInit()		SerialDDR |= 1<<TxD;\
						SerialDDR &= ~(1<<RxD);\
						I2CDDR &= ~(1<<SDA | SCL);

#define I2CPort			PORTD
#define I2CDDR			DDRD
#define I2CPIN			PIND
#define SDA				2
#define SCL				3
//Disable Rx/Tx when I2C is selected
#define I2CInit()		I2CDDR |= (1<<SDA | SCL);\
						SerialDDR &= ~(1<<TxD | 1<<RxD);

//___________________Timer 0 definition___________________________________ 
//Define a basic set of uart regiters and map them to the actual AT90USB16

#define TIFR		TIFR1
#define TIMSK		TIMSK1
#define	TICIE1		ICIE1

//_________________________JTAG definition________________________________
//Disable MISO when TDO is used
//Disable SCK when TCK is used
//Disable MOSI when TDI is used
//Disable SEL when TMS is used		
#define		TCK		4
#define 	TMS		5
#define		TDO		6
#define 	TDI		7

#define  	JPORT	PORTF
#define 	JDDR	DDRF
#define 	JPIN	PINF

#define 	DisableJTAG()		MCUCR |= 1<<JTD; MCUCR |= 1<<JTD;

#define 	TMS_0	0
#define 	TMS_1	1
#define		TMS_input	JDDR &= ~(1<<TMS);	SPIDDR &= ~(1<<SEL);
#define		TMS_output	JDDR |= 1<<TMS;		SPIDDR &= ~(1<<SEL);
#define		TMS_low		JPORT &= ~(1<<TMS);	SPIPort &= ~(1<SEL);
#define 	TMS_high	JPORT |= 1<<TMS;	SPIPort &= ~(1<<SEL);
#define		TMS_read	(JPIN & (1<<TMS))

#define		TCK_input	JDDR &= ~(1<<TCK);	SPIDDR &= ~(1<<SCK);
#define		TCK_output	JDDR |= 1<<TCK;		SPIDDR &= ~(1<<SCK);
#define		TCK_low		JPORT &= ~(1<<TCK);	SPIPort &= ~(1<<SCK);
#define 	TCK_high	JPORT |= 1<<TCK;	SPIPort &= ~(1<<SCK);
#define		TCK_read	(JPIN & (1<<TCK))

#define		TDI_input	JDDR &= ~(1<<TDI);	SPIDDR &= ~(1<<MOSI);
#define		TDI_output	JDDR |= 1<<TDI;		SPIDDR &= ~(1<<MOSI);
#define		TDI_low		JPORT &= ~(1<<TDI);	SPIPort &= ~(1<<MOSI);
#define 	TDI_high	JPORT |= 1<<TDI;	SPIPort &= ~(1<<MOSI);
#define		TDI_read	(JPIN & (1<<TDI))

#define		TDO_input	JDDR &= ~(1<<TDO);	SPIDDR &= ~(1<<MISO);
#define		TDO_output	JDDR |= 1<<TDO;		SPIDDR &= ~(1<<MISO);
#define		TDO_low		JPORT &= ~(1<<TDO);	SPIPort &= ~(1<<MISO);
#define 	TDO_high	JPORT |= 1<<TDO;	SPIPort &= ~(1<<MISO);
#define		TDO_read	(JPIN & (1<<TDO))

//_________________________I/O definition_________________________________

#define EnableVout()	DDRB |= 1<<6; PORTB |=1<<6;
#define DisableVout()	DDRB |= 1<<6; PORTB &=~(1<<6);
#define Read_VEN()		(PINB & (1<<6))

#define		AUX		7

//Disable PF1 when AUX is used
#define  	AUXPORT	PORTF
#define 	AUXDDR	DDRF
#define 	AUXPIN	PINF

#define		AUX_input	AUXDDR &= ~(1<<AUX); 	DDRF &= ~(1<<1);
#define		AUX_output	AUXDDR |= 1<<AUX; 		DDRF &= ~(1<<1);
#define		AUX_low		AUXPORT &= ~(1<<AUX); 	DDRF &= ~(1<<1);
#define 	AUX_high	AUXPORT |= 1<<AUX; 		DDRF &= ~(1<<1);
#define		AUX_read	(AUXPIN & (1<<AUX))
