//Scope_V1.0.h

#include <avr/io.h> 

#define FIFOA		0
#define FIFOB		1
#define address		2
#define control 	3
#define status 		4
#define signature 	5
#define LED 		0x00
#define ADCsel 		0x01
#define GainA 		0x02
#define GainB 		0x03
#define MasterCK 	0x04
#define Clock_div 	0x05
#define TriggerA	0x06
#define TriggerB	0x07
#define EdgeA		0x08
#define EdgeB 		0x09
#define PostAlo 	0x0a
#define PostAhi		0x0b
#define PostBlo		0x0c
#define PostBhi		0x0d
#define couple		0x0e
#define read		0x08
#define write		0xf7


#define DataBusOut()	DDRB = 0xFF;			
#define DataBusIn()		DDRB = 0x00;
#define FPGA_Reset()	DDRC |= 1<<6;\
						asm volatile ("nop");\
						PORTC &= ~(1<<6);\
						PORTC |= 1<<6;
#define Delay()			asm volatile ("nop");\
						asm volatile ("nop");
#define DelayLong()		asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");
#define WriteSelect()	PORTD &= ~(1 << 3);
#define ReadSelect()	PORTD |=  (1 << 3);

//___________________USB16 Crystal definition_____________________________ 
#ifndef F_CPU
#define F_CPU 16000000
#endif

//___________________HWB definition_______________________________________
//
//HWB is always an input with a pullup
#define 	HWB			1
#define		HWB_init	DDRE &= ~(1<<HWB);\
						PORTE |= 1<<HWB;

//___________________SPI definition_______________________________________
#define		SCK		1
#define 	MOSI	2
#define		MISO	3	

#define		MISO_input	DDRB &= ~(1<<MISO);
#define		MISO_output	DDRB |= 1<<MISO;
#define		MISO_low	PORTB &= ~(1<<MISO);
#define 	MISO_high	PORTB |= 1<<MISO;
#define		MISO_read	(PINB & (1<<MISO))

#define		MOSI_input	DDRB &= ~(1<<MOSI);
#define		MOSI_output	DDRB |= 1<<MOSI;
#define		MOSI_low	PORTB &= ~(1<<MOSI);
#define 	MOSI_high	PORTB |= 1<<MOSI;
#define		MOSI_read	(PINB & (1<<MOSI))

#define		SCK_input	DDRB &= ~(1<<SCK);
#define		SCK_output	DDRB |= 1<<SCK;
#define		SCK_low		PORTB &= ~(1<<SCK);
#define 	SCK_high	PORTB |= 1<<SCK;
#define		SCK_read	(PINB & (1<<SCK))

//_______________________ LED definition _________________________________
#define Pgm1 			1					//port bit for anode (Blue)
#define Pgm2 			2					//port bit for anode (Yellow)
#define PgmPort			PORTB				//actual port for Led 
#define PgmDDR			DDRB

#define PgmInit()		PgmDDR |= 1<<Pgm1;
#define PgmOn()			PgmPort |= 1<<Pgm1;
#define PgmOff()		PgmPort &= ~(1<<Pgm1);
#define PgmToggle()		PgmPort ^= 1<<Pgm1;

#define Pgm1Init()		PgmDDR |= 1<<Pgm1;
#define Pgm1On()		PgmPort |= 1<<Pgm1;
#define Pgm1Off()		PgmPort &= ~(1<<Pgm1);
#define Pgm1Toggle()	PgmPort ^= 1<<Pgm1;

#define Pgm2Init()		PgmDDR |= 1<<Pgm2;
#define Pgm2On()		PgmPort |= 1<<Pgm2;
#define Pgm2Off()		PgmPort &= ~(1<<Pgm2);
#define Pgm2Toggle()	PgmPort ^= 1<<Pgm2;

//_________________________Boot Vector definition_________________________  

//????????????????????????
#define BOOT_START		FLASHEND/2-4095		//4K boot sector
#define BOOT_END		(FLASHEND/2-1)

#if(FLASH_END == 0x1FFFF)
	#define BOOT_END 	0xFFFE
#elif(FLASH_END == 0x3FFF)
	#define BOOT_END 	0x3FFE
#endif

//___________________Uart definition______________________________________ 
//Define a basic set of uart regiters and map them to the processor
#define SerialPort		PORTD
#define SerialDDR		DDRD
#define SerialPIN		PIND
#define RXD				2
#define TXD				3
#define UBRRH			UBRR1H
#define UBRRL			UBRR1H
#define RXC				RXC1
#define TXC				TXC1
#define TXEN			TXEN1
#define RXEN			RXEN1
#define RXCIE			RXCIE1
#define UCSRA			UCSR1A 
#define UCSRB			UCSR1B
#define UDRE			UDRE1 
#define UDR				UDR1

//___________________Timer 0 definition___________________________________ 
//Define a basic set of uart regiters and map them to the processor

#define TIFR		TIFR1
#define TIMSK		TIMSK1
#define	TICIE1		ICIE1

//___________________JTAG definition______________________________________
#define 	TMS		6		
#define		TCK		4
#define 	TDI		7
#define		TDO		5
#define  	JPORT	PORTF
#define 	JDDR	DDRF
#define 	JPIN	PINF


#define 	TMS_0	0
#define 	TMS_1	1
#define		TMS_input	JDDR &= ~(1<<TMS);
#define		TMS_output	JDDR |= 1<<TMS;
#define		TMS_low		JPORT &= ~(1<<TMS);
#define 	TMS_high	JPORT |= 1<<TMS;
#define		TMS_read	(JPIN & (1<<TMS))

#define		TCK_input	JDDR &= ~(1<<TCK);
#define		TCK_output	JDDR |= 1<<TCK;
#define		TCK_low		JPORT &= ~(1<<TCK);
#define 	TCK_high	JPORT |= 1<<TCK;
#define		TCK_read	(JPIN & (1<<TCK))

#define		TDI_input	JDDR &= ~(1<<TDI);
#define		TDI_output	JDDR |= 1<<TDI;
#define		TDI_low		JPORT &= ~(1<<TDI);
#define 	TDI_high	JPORT |= 1<<TDI;
#define		TDI_read	(JPIN & (1<<TDI))

#define		TDO_input	JDDR &= ~(1<<TDO);
#define		TDO_output	JDDR |= 1<<TDO;
#define		TDO_low		JPORT &= ~(1<<TDO);
#define 	TDO_high	JPORT |= 1<<TDO;
#define		TDO_read	(JPIN & (1<<TDO))

//___________________FPGA definition______________________________________
#define		PROG_B		0
#define  	PROG_PORT	PORTF
#define 	PROG_DDR	DDRF
#define 	PROG_PIN	PINF	
#define		RST			0
//#define		SCA			1			//????????
#define		RSTDDR		DDRF
#define		RSTPORT		PORTF

#define		Prog_B_output	MCUCR |= 1<<JTD;\
							MCUCR |= 1<<JTD;\
							PROG_DDR |= 1<<PROG_B;
#define		Prog_B_low		MCUCR |= 1<<JTD;\
							MCUCR |= 1<<JTD;\
							PROG_PORT &= ~(1<<PROG_B);
#define		Prog_B_high		MCUCR |= 1<<JTD;\
							MCUCR |= 1<<JTD;\
							PROG_PORT |= 1<<PROG_B;

#define		FPGA_RST	MCUCR |= 1<<JTD;\
						MCUCR |= 1<<JTD;\
						PROG_PORT &= ~(1<<PROG_B);\
						asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");\
						asm volatile ("nop");\
						PROG_PORT |= 1<<PROG_B;
						
//___________________ I2C definition______________________________________		
#define		SCL		0
#define		SCA		1
#define		I2CDDR	DDRD
#define		I2CPORT	PORTD	

//________________Signature definition__________________
#define 	SignatureDDR	DDRB
#define 	SignaturePORT	PORTB
#define 	SignaturePIN	PINB

#define 	EN				2
#define 	SO				3
#define 	SIGCK			1


#define 	InitSignature()	SignatureDDR |= 1<<EN | 1<<SIGCK;\
							SignatureDDR &= ~(1<<SO);\
							SignaturePORT &= ~(1<<SIGCK);
#define 	Enable_High()	SignaturePORT |= 1<<EN;
#define 	Enable_Low()	SignaturePORT &= ~(1<<EN);
#define		EN_read			(SignaturePIN & (1<<EN))

#define 	SIGCK_Low()		SignaturePORT &= ~(1<<SIGCK);
#define 	SIGCK_High() 	SignaturePORT |= 1<<SIGCK;
#define 	ResetSignature() SignaturePORT &= ~(1<<SIGCK);\
							SignaturePORT |= 1<<EN;\
							asm volatile("nop");\
							SignaturePORT &= ~(1<<EN);\
							asm volatile("nop");
#define 	SO_read()		(SignaturePIN >> SO)&0x01;		

	
