#include <avr/io.h> 

//___________________USB16 Crystal definition_____________________________ 
#ifndef F_CPU
#define F_CPU 16000000
#endif

//___________________USB16 SPI definition_________________________________
#define 	RST		0
#define		MISO	3
#define 	MOSI	2
#define		SCK		1
#define		HWB		5	//Solder pin 8 of icp to HWB
						//This is TRST PB5 on USB16 V1.0
						//This is TXD_TRST PD3 on USB16 V1.1
#define		RST_input	DDRB &= ~(1<<RST);
#define		RST_output	DDRB |= 1<<RST;
#define		RST_low		PORTB &= ~(1<<RST);
#define 	RST_high	PORTB |= 1<<RST;
#define		RST_read	(PINB & (1<<RST))

#define		MISO_input	DDRB &= ~(1<<MISO);
#define		MISO_output	DDRB |= 1<<MISO;
#define		MISO_low	PORTB &= ~(1<<MISO);
#define 	MISO_high	PORTB |= 1<<MISO;
#define		MISO_read	(PINB & (1<<MISO))

#define		MOSI_input	DDRB &= ~(1<<MOSI);
#define		MOSI_output	DDRB |= 1<<MOSI;
#define		MOSI_low	PORTB &= ~(1<<MOSI);
#define 	MOSI_high	PORTB |= 1<<MOSI;
#define		MOSI_read	(PINB & (1<<MOSI))

#define		SCK_input	DDRB &= ~(1<<SCK);
#define		SCK_output	DDRB |= 1<<SCK;
#define		SCK_low		PORTB &= ~(1<<SCK);
#define 	SCK_high	PORTB |= 1<<SCK;
#define		SCK_read	(PINB & (1<<SCK))

#define 	Init_HWB	DDRB |= 1<<HWB;		//This needs to be an input 
											//to prevent contention
#define		HWB_input	DDRB &= ~(1<<HWB);
#define		HWB_output	DDRB |= 1<<HWB;
#define		HWB_low		PORTB &= ~(1<<HWB);
#define 	HWB_high	PORTB |= 1<<HWB;
#define		HWB_read	(PINB & (1<<HWB))

//___________________USB16 PGM led definition_____________________________
//

#define PGMC			1					//port bit for cathode
#define Pgm				0					//port bit for cathode
#define PGMA 			0					//port bit for anode
#define PGMPort			PORTD				//actual port for Led 
#define PGMDDR			DDRD

#define PgmInit()		PGMDDR |= 1<<PGMA | 1<<PGMC;
#define PgmOn()			PGMPort |= 1<<PGMA;\
						PGMPort &= ~(1<<PGMC);
#define PgmOff()		PGMPort &= ~(1<<PGMA);
#define PgmToggle()		PGMPort ^= 1<<PGMA;

#define HeartBeat()		PgmOn();\
						_delay_ms(150);\
						PgmOff();	\
						_delay_ms(150);\
						PgmOn();\
						_delay_ms(600);\
						PgmOff();\
						_delay_ms(150);
#define Pulse(On,Off)	PgmOn();\
						_delay_ms(On);\
						PgmOff();	\
						_delay_ms(Off);		

#define DisplayPgmOn()  DisplayLED(1,100,100,'D',Pgm,0xff,0)
#define DisplayPgmOff() DisplayLED(1,  0,100,'D',Pgm,0xff,0)
											

//___________________USB16 Power LED definition___________________________
#define Pwr 			6					//port bit for anode
#define PwrPort			PORTC				//actual port for Led 
#define PwrDDR			DDRC

#define PwrInit()		PwrDDR |= 1<<Pwr;
#define PwrOn()			PwrPort |= 1<<Pwr;
#define PwrOff()		PwrPort &= ~(1<<Pwr);
#define PwrToggle()		PwrPort ^= 1<<Pwr;

#define HeartBeatPwr()	PwrOn();\
						_delay_ms(150);\
						PwrOff();	\
						_delay_ms(150);\
						PwrOn();\
						_delay_ms(600);\
						PwrOff();\
						_delay_ms(150);
#define PulsePwr(On,Off)PwrOn();\
						_delay_ms(On);\
						PwrOff();	\
						_delay_ms(Off);	


#define DisplayPwrOn()  DisplayLED(0,100,100,'C',Pwr,0xff,0)
#define DisplayPwrOff() DisplayLED(0,  0,100,'C',Pwr,0xff,0)

//___________________Target Boot & Reset definition_______________________
#define TargetRst 		5					//bit that is toggled for rst-
#define TargetRstPort	PORTC				//rst- output port
#define TargetRstDDR	DDRC
#define TargetHWB 		3					//bit that is toggled for HWB
#define TargetHWBPort	PORTB				//HWB output port
#define TargetHWBDDR	DDRB

#define	TargetRst_input		TargetRstDDR &= ~(1<<TargetRst);
#define	TargetRst_output	TargetRstDDR |= 1<<TargetRst;
#define TargetRst_high		TargetRstPort |= 1<<TargetRst;
#define TargetRst_low		TargetRstPort &= ~(1<<TargetRst);

#define	TargetHWB_input		TargetHWBDDR &= ~(1<<TargetHWB);
#define	TargetHWB_output	TargetHWBDDR |= 1<<TargetHWB;
#define TargetHWB_high		TargetHWBPort |= 1<<TargetHWB;
#define TargetHWB_low		TargetHWBPort &= ~(1<<TargetHWB);

//___________________USB16 Photosensor definition_________________________ 
#define SensorDDR		DDRD
#define SensorPORT		PORTD
#define SensorPIN		PIND
#define sensor 			1<<PGMC

//___________________USB16 Boot Vector definition_________________________  
#define BOOT_START		FLASHEND/2-4095		//4K boot sector
#define BOOT_END		(FLASHEND/2-1)

#if(FLASH_END == 0x1FFFF)
	#define BOOT_END 	0xFFFE
#elif(FLASH_END == 0x3FFF)
	#define BOOT_END 	0x3FFE
#endif

//___________________USB16 Uart definition________________________________ 
//Define a basic set of uart regiters and map them to the actual AT90USB16
#define SerialPort		PORTD
#define SerialDDR		DDRD
#define SerialPIN		PIND
#define RXD				2
#define TXD				3
#define UBRRH			UBRR1H
#define UBRRL			UBRR1H
#define RXC				RXC1
#define TXC				TXC1
#define TXEN			TXEN1
#define RXEN			RXEN1
#define RXCIE			RXCIE1
#define UCSRA			UCSR1A 
#define UCSRB			UCSR1B
#define UDRE			UDRE1 
#define UDR				UDR1

//___________________Timer 0 definition___________________________________ 
//Define a basic set of uart regiters and map them to the actual AT90USB16

#define TIFR		TIFR1
#define TIMSK		TIMSK1
#define	TICIE1		ICIE1

//___________________JTAG definition____________________
//#define URJTAG
#ifdef URJTAG
#define 	TMS		0		//Default for URJTAG w SPI
#define		TCK		1
#define 	TDI		2
#define		TDO		3
#define		DONE	7
#define  	JPORT	PORTB
#define 	JDDR	DDRB
#define 	JPIN	PINB

#else 
#define 	TMS		0		//Default for USB16
#define		TCK		2
#define 	TDI		3
#define		TDO		4
#define		DONE	7
#define  	JPORT	PORTB
#define 	JDDR	DDRB
#define 	JPIN	PINB
#endif

#define 	TMS_0	0
#define 	TMS_1	1
#define		TMS_input	JDDR &= ~(1<<TMS);
#define		TMS_output	JDDR |= 1<<TMS;
#define		TMS_low		JPORT &= ~(1<<TMS);
#define 	TMS_high	JPORT |= 1<<TMS;
#define		TMS_read	(JPIN & (1<<TMS))

#define		TCK_input	JDDR &= ~(1<<TCK);
#define		TCK_output	JDDR |= 1<<TCK;
#define		TCK_low		JPORT &= ~(1<<TCK);
#define 	TCK_high	JPORT |= 1<<TCK;
#define		TCK_read	(JPIN & (1<<TCK))

#define		TDI_input	JDDR &= ~(1<<TDI);
#define		TDI_output	JDDR |= 1<<TDI;
#define		TDI_low		JPORT &= ~(1<<TDI);
#define 	TDI_high	JPORT |= 1<<TDI;
#define		TDI_read	(JPIN & (1<<TDI))

#define		TDO_input	JDDR &= ~(1<<TDO);
#define		TDO_output	JDDR |= 1<<TDO;
#define		TDO_low		JPORT &= ~(1<<TDO);
#define 	TDO_high	JPORT |= 1<<TDO;
#define		TDO_read	(JPIN & (1<<TDO))

#define		DONE_input	JDDR &= ~(1<<DONE);
#define		DONE_output	JDDR |= 1<<DONE;
#define		DONE_low	JPORT &= ~(1<<DONE);
#define 	DONE_high	JPORT |= 1<<DONE;
#define		DONE_read	(JPIN & (1<<DONE))

//___________________USBKeypad definition_________________________________ 
#define right_click		"\2"
#define middle_click	"\4"
#define left_click		"\1"
#define no_click		"\0"
#define Tab				"Doug\r"
#define Enter			"Greg\r"
#define On 				1
#define Off 			0

#define On_level		75
#define Off_level		150
#define max_timeout 	150

//Hardware configuration for USB_Keypad V1.1
//PC4-7 are switch inputs
//		SW1 = PC5 	mouse 	0
//		SW2 = PC7  	mouse 	1
//		SW3 = PC6 	keyboard 0
//		SW4 = PC4 	keyboard 1
//Setup all switch and config signals as inputs with pullups
#define Switch_init	DDRC &= ~(1<<DDC4 | 1<<DDC5 | 1<<DDC6 | 1<<DDC7);\
					PORTC |= (1<<DDC4 | 1<<DDC5 | 1<<DDC6 | 1<<DDC7);

#define Read_config	((PINC & (1<<PINC4 | 1<<PINC5 | 1<<PINC6 | 1<<PINC7))>>4)

#define LedInit() 	PgmInit();\
					PwrInit();
#define Led3On() 	PgmOn();
#define Led3Off() 	PgmOff();				

