//USB_analyzerV1.0.h

#include <avr/io.h> 

//___________________USB16 Crystal definition_____________________________ 
#ifndef F_CPU
#define F_CPU 16000000
#endif

//___________________Signature definition_______________________________________
#define 	RD				6		//PC6
#define 	RD_DDR			DDRC
#define 	RD_PIN			PINC
#define 	RD_PORT			PORTC

#define		RD_input		RD_DDR &= ~(1<<RD);
#define		RD_output		RD_DDR |= 1<<RD;
#define		RD_low			RD_PORT &= ~(1<<RD);
#define 	RD_high			RD_PORT |= 1<<RD;
#define		RD_read			(RD_PIN & (1<<RD))

#define		RST				3
#define 	SO				2
#define 	Signature_DDR	DDRD
#define 	Signature_PIN	PIND
#define 	Signature_PORT	PORTD

#define		SO_input		Signature_DDR &= ~(1<<SO);
#define		SO_output		Signature_DDR |= 1<<SO;
#define		SO_low			Signature_PORT &= ~(1<<SO);
#define 	SO_high			Signature_PORT |= 1<<SO;
#define		SO_read			((Signature_PIN & (1<<SO))>>SO)

#define		RST_input		Signature_DDR &= ~(1<<RST);
#define		RST_output		Signature_DDR |= 1<<RST;
#define		RST_low			Signature_PORT &= ~(1<<RST);
#define 	RST_high		Signature_PORT |= 1<<RST;
#define		RST_read		((Signature_PIN & (1<<RST))>>RST)


#define    ResetSignature() SO_input;\
							RD_output;\
							RST_output;\
							RD_low;\
							RST_high;\
							asm volatile("nop");\
							RST_low;\
							asm volatile("nop");\
							RST_high;\
							asm volatile("nop");


//___________________SPI definition_______________________________________
#define		SCK				1
#define 	MOSI			2
#define		MISO			3	
#define 	SPI_DDR			DDRB
#define 	SPI_PIN			PINB
#define 	SPI_PORT		PORTB

#define		MISO_input		SPI_DDR	 &= ~(1<<MISO);
#define		MISO_output		SPI_DDR	 |= 1<<MISO;
#define		MISO_low		SPI_PORT &= ~(1<<MISO);
#define 	MISO_high		SPI_PORT |= 1<<MISO;
#define		MISO_read		(SPI_PIN & (1<<MISO))

#define		MOSI_input		SPI_DDR	 &= ~(1<<MOSI);
#define		MOSI_output		SPI_DDR	 |= 1<<MOSI;
#define		MOSI_low		SPI_PORT &= ~(1<<MOSI);
#define 	MOSI_high		SPI_PORT |= 1<<MOSI;
#define		MOSI_read		(SPI_PIN & (1<<MOSI))

#define		SCK_input		SPI_DDR	 &= ~(1<<SCK);
#define		SCK_output		SPI_DDR	 |= 1<<SCK;
#define		SCK_low			SPI_PORT &= ~(1<<SCK);
#define 	SCK_high		SPI_PORT |= 1<<SCK;
#define		SCK_read		(SPI_PIN & (1<<SCK))

//___________________Uart definition______________________________________ 
//Define a basic set of uart regiters and map them to the processor
#define 	SerialPort		PORTD
#define 	SerialDDR		DDRD
#define 	SerialPIN		PIND
#define 	RXD				2
#define 	TXD				3
#define 	UBRRH			UBRR1H
#define 	UBRRL			UBRR1H
#define 	RXC				RXC1
#define 	TXC				TXC1
#define 	TXEN			TXEN1
#define 	RXEN			RXEN1
#define 	RXCIE			RXCIE1
#define 	UCSRA			UCSR1A 
#define 	UCSRB			UCSR1B
#define 	UDRE			UDRE1 
#define 	UDR				UDR1

//___________________JTAG definition______________________________________
#define 	TMS				5
#define		TCK				4
#define 	TDI				7
#define		TDO				6
#define  	JPORT			PORTF
#define 	JDDR			DDRF
#define 	JPIN			PINF

#define 	TMS_0			0
#define 	TMS_1			1
#define		TMS_input		DisableJTAG(); JDDR &= ~(1<<TMS);
#define		TMS_output		DisableJTAG(); JDDR |= 1<<TMS;
#define		TMS_low			DisableJTAG(); JPORT &= ~(1<<TMS);
#define 	TMS_high		DisableJTAG(); JPORT |= 1<<TMS;
#define		TMS_read		(JPIN & (1<<TMS))

#define		TCK_input		DisableJTAG(); JDDR &= ~(1<<TCK);
#define		TCK_output		DisableJTAG(); JDDR |= 1<<TCK;
#define		TCK_low			DisableJTAG(); JPORT &= ~(1<<TCK);
#define 	TCK_high		DisableJTAG(); JPORT |= 1<<TCK;
#define		TCK_read		(JPIN & (1<<TCK))

#define		TDI_input		DisableJTAG(); JDDR &= ~(1<<TDI);
#define		TDI_output		DisableJTAG(); JDDR |= 1<<TDI;
#define		TDI_low			DisableJTAG(); JPORT &= ~(1<<TDI);
#define 	TDI_high		DisableJTAG(); JPORT |= 1<<TDI;
#define		TDI_read		(JPIN & (1<<TDI))

#define		TDO_input		DisableJTAG(); JDDR &= ~(1<<TDO);
#define		TDO_output		DisableJTAG(); JDDR |= 1<<TDO;
#define		TDO_low			DisableJTAG(); JPORT &= ~(1<<TDO);
#define 	TDO_high		DisableJTAG(); JPORT |= 1<<TDO;
#define		TDO_read		(JPIN & (1<<TDO))


//___________________FPGA definition______________________________________
#define		PROG_B			1
#define  	PROG_PORT		PORTF
#define 	PROG_DDR		DDRF
#define 	PROG_PIN		PINF	


#define 	DisableJTAG()	MCUCR = 1<<JTD;\
							MCUCR = 1<<JTD;

#define		Prog_B_output	DisableJTAG();\
							PROG_DDR |= 1<<PROG_B;
#define		Prog_B_low		DisableJTAG();\
							PROG_PORT &= ~(1<<PROG_B);
#define		Prog_B_high		DisableJTAG();\
							PROG_PORT |= 1<<PROG_B;

#define 	Load_FPGA()		Prog_B_output;\
							Prog_B_low;\
							asm volatile ("nop");\
							asm volatile ("nop");\
							asm volatile ("nop");\
							asm volatile ("nop");\
							asm volatile ("nop");\
							Prog_B_high;						

//___________________PORTD definition______________________________________
#define 	SOP_INT			0		//PD0 SOP valid 
#define 	RDY				1		//PD1 FIFO has data
#define 	FRST 			3		//PD3 FPGA reset 

#define		SOP_INT_input	DDRD &= ~(1<<SOP_INT);
#define		SOP_INT_output	DDRD |= 1<<SOP_INT;
#define		SOP_INT_low		PORTD &= ~(1<<SOP_INT);
#define 	SOP_INT_high	PORTD |= 1<<SOP_INT;
#define		SOP_INT_read	(PIND & (1<<SOP_INT))

#define		RDY_input		DDRD &= ~(1<<RDY);
#define		RDY_output		DDRD |= 1<<RDY;
#define		RDY_low			PORTD &= ~(1<<RDY);
#define 	RDY_high		PORTD |= 1<<RDY;
#define		RDY_read		(PIND & (1<<RDY))

#define		FRST_input		DDRD &= ~(1<<FRST);
#define		FRST_output		DDRD |= 1<<FRST;
#define		FRST_low		PORTD &= ~(1<<FRST);
#define 	FRST_high		PORTD |= 1<<FRST;
#define		FRST_read		(PIND & (1<<FRST))

#define 	FPGA_reset()	FRST_output;\
							FRST_high;\
							asm volatile ("nop");\
							asm volatile ("nop");\
							FRST_low;\
							asm volatile ("nop");\
							asm volatile ("nop");\
							FRST_high;\
							asm volatile ("nop");\
							asm volatile ("nop");	




unsigned volatile char rdy,sop;
unsigned int waiting;

#define FIFOEMPTY 			!(RDY_read)

#define Wait_for_SOP()		while((SOP_INT_read)==0){\
								answer[0] = ReadFIFO();\
							};

#define WaitforRdy()		RDY_input;\
							temp = FIFOEMPTY;\
							while(temp){\
								temp = FIFOEMPTY;\
								waiting++;\
								if(waiting == 1000){\
									waiting = 0;\
									usb_putchar('.');\
								}\
							};

#define USB_Analyzer_Init() RDY_input;\
							SOP_INT_input;\
							DDRB = 0x00;\
							RD_output;\
							FRST_output;


						



	
