#include <avr/io.h> 

//___________________USB16 Crystal definition_____________________________ 
#ifndef F_CPU
#define F_CPU 16000000
#endif

//___________________USB16 SPI definition_________________________________
#define 	RST		0
#define		MISO	3
#define 	MOSI	2
#define		SCK		1
#define		HWB		5	//Solder pin 8 of icp to HWB
						//This is TRST PB5 on USB16 V1.0
						//This is TXD_TRST PD3 on USB16 V1.1
#define		RST_input	DDRB &= ~(1<<RST);
#define		RST_output	DDRB |= 1<<RST;
#define		RST_low		PORTB &= ~(1<<RST);
#define 	RST_high	PORTB |= 1<<RST;
#define		RST_read	(PINB & (1<<RST))

#define		MISO_input	DDRB &= ~(1<<MISO);
#define		MISO_output	DDRB |= 1<<MISO;
#define		MISO_low	PORTB &= ~(1<<MISO);
#define 	MISO_high	PORTB |= 1<<MISO;
#define		MISO_read	(PINB & (1<<MISO))

#define		MOSI_input	DDRB &= ~(1<<MOSI);
#define		MOSI_output	DDRB |= 1<<MOSI;
#define		MOSI_low	PORTB &= ~(1<<MOSI);
#define 	MOSI_high	PORTB |= 1<<MOSI;
#define		MOSI_read	(PINB & (1<<MOSI))

#define		SCK_input	DDRB &= ~(1<<SCK);
#define		SCK_output	DDRB |= 1<<SCK;
#define		SCK_low		PORTB &= ~(1<<SCK);
#define 	SCK_high	PORTB |= 1<<SCK;
#define		SCK_read	(PINB & (1<<SCK))

#define 	Init_HWB	DDRB |= 1<<HWB;		//This needs to be an input 
											//to prevent contention
#define		HWB_input	DDRB &= ~(1<<HWB);
#define		HWB_output	DDRB |= 1<<HWB;
#define		HWB_low		PORTB &= ~(1<<HWB);
#define 	HWB_high	PORTB |= 1<<HWB;
#define		HWB_read	(PINB & (1<<HWB))

//___________________USB16 PGM led definition_____________________________
//
//Tidy this so that it only has the PGM definition
//

#define LEDC			1					//port bit for cathode
#define LEDA 			0					//port bit for anode
#define LedPort			PORTD				//actual port for Led 
#define LEDDDR			DDRD

#define LedInit()		LEDDDR |= 1<<LEDA | 1<<LEDC;
#define LedOn()			LedPort |= 1<<LEDA;\
						LedPort &= ~(1<<LEDC);
#define LedOff()		LedPort &= ~(1<<LEDA);
#define LedToggle()		LedPort ^= 1<<LEDA;

#define Led				0

#define Pgm 			1					//port bit for anode
#define PgmPort			PORTD				//actual port for Led 
#define PgmDDR			DDRD
#define PgmInit()		PgmDDR |= 1<<Pgm;
#define PgmOn()			PgmPort |= 1<<Pgm;
#define PgmOff()		PgmPort &= ~(1<<Pgm);
#define PgmToggle()		PgmPort ^= 1<<Pgm;

#define HeartBeat()		LedOn();\
						_delay_ms(150);\
						LedOff();	\
						_delay_ms(150);\
						LedOn();\
						_delay_ms(600);\
						LedOff();\
						_delay_ms(150);
#define Pulse(On,Off)	LedOn();\
						_delay_ms(On);\
						LedOff();	\
						_delay_ms(Off);		
											

//___________________USB16 Power LED definition___________________________
#ifdef  V1_0
	#define Pwr 		6					//port bit for anode
	#define PwrPort		PORTC				//actual port for Led 
	#define PwrDDR		DDRC
#else 										//V1_1 or V1_2
	#define Pwr 		4					//port bit for anode
	#define PwrPort		PORTD				//actual port for Led 
	#define PwrDDR		DDRD
#endif

#define PwrInit()		PwrDDR |= 1<<Pwr;
#define PwrOn()			PwrPort |= 1<<Pwr;
#define PwrOff()		PwrPort &= ~(1<<Pwr);
#define PwrToggle()		PwrPort ^= 1<<Pwr;

#define HeartBeatPwr()	PwrOn();\
						_delay_ms(150);\
						PwrOff();	\
						_delay_ms(150);\
						PwrOn();\
						_delay_ms(600);\
						PwrOff();\
						_delay_ms(150);
#define PulsePwr(On,Off)PwrOn();\
						_delay_ms(On);\
						PwrOff();	\
						_delay_ms(Off);	

//___________________USB16 Photosensor definition_________________________ 
#define SensorDDR		DDRD
#define SensorPORT		PORTD
#define SensorPIN		PIND
//___________________USB16 Boot Vector definition_________________________  
#define BOOT_START		FLASHEND/2-4095		//4K boot sector
#define BOOT_END		(FLASHEND/2-1)

#if(FLASH_END == 0x1FFFF)
	#define BOOT_END 	0xFFFE
#elif(FLASH_END == 0x3FFF)
	#define BOOT_END 	0x3FFE
#endif

//___________________USB16 Uart definition________________________________ 
//Define a basic set of uart regiters and map them to the actual AT90USB16
#define SerialPort		PORTD
#define SerialDDR		DDRD
#define SerialPIN		PIND
#define RXD				2
#define TXD				3
#define UBRRH			UBRR1H
#define UBRRL			UBRR1H
#define RXC				RXC1
#define TXC				TXC1
#define TXEN			TXEN1
#define RXEN			RXEN1
#define RXCIE			RXCIE1
#define UCSRA			UCSR1A 
#define UCSRB			UCSR1B
#define UDRE			UDRE1 
#define UDR				UDR1

//___________________Timer 0 definition___________________________________ 
//Define a basic set of uart regiters and map them to the actual AT90USB16

#define TIFR		TIFR1

#define TIMSK		TIMSK1
#define	TICIE1		ICIE1



//___________________USB16 Debug definition________________________________ 
#ifdef  V1_0
#define debugWire()		DDRB &= ~(1<<DDB0);	//RST- as input for debugWire 
#endif

//___________________Logic Analyzer JTAG definition____________________
#ifdef LA16
#define 	TMS		4
#define		TCK		5
//#define 	TDI		7		//This is the reverse of LA16 V1.0
//#define		TDO		6	//schematic because of mods
#define 	TDI		6		
#define		TDO		7		
#define		DONE	2
#define  	JPORT	PORTC
#define 	JDDR	DDRC
#define 	JPIN	PINC
#endif
#define URJTAG
#ifdef URJTAG
#define 	TMS		0		//Default for URJTAG w SPI
#define		TCK		1
#define 	TDI		2
#define		TDO		3
#define		DONE	7
#define  	JPORT	PORTB
#define 	JDDR	DDRB
#define 	JPIN	PINB

#else 
#define 	TMS		0		//Default for USB16
#define		TCK		2
#define 	TDI		3
#define		TDO		4
#define		DONE	7
#define  	JPORT	PORTB
#define 	JDDR	DDRB
#define 	JPIN	PINB
#endif

#define 	TMS_0	0
#define 	TMS_1	1
#define		TMS_input	JDDR &= ~(1<<TMS);
#define		TMS_output	JDDR |= 1<<TMS;
#define		TMS_low		JPORT &= ~(1<<TMS);
#define 	TMS_high	JPORT |= 1<<TMS;
#define		TMS_read	(JPIN & (1<<TMS))

#define		TCK_input	JDDR &= ~(1<<TCK);
#define		TCK_output	JDDR |= 1<<TCK;
#define		TCK_low		JPORT &= ~(1<<TCK);
#define 	TCK_high	JPORT |= 1<<TCK;
#define		TCK_read	(JPIN & (1<<TCK))

#define		TDI_input	JDDR &= ~(1<<TDI);
#define		TDI_output	JDDR |= 1<<TDI;
#define		TDI_low		JPORT &= ~(1<<TDI);
#define 	TDI_high	JPORT |= 1<<TDI;
#define		TDI_read	(JPIN & (1<<TDI))

#define		TDO_input	JDDR &= ~(1<<TDO);
#define		TDO_output	JDDR |= 1<<TDO;
#define		TDO_low		JPORT &= ~(1<<TDO);
#define 	TDO_high	JPORT |= 1<<TDO;
#define		TDO_read	(JPIN & (1<<TDO))

#define		DONE_input	JDDR &= ~(1<<DONE);
#define		DONE_output	JDDR |= 1<<DONE;
#define		DONE_low	JPORT &= ~(1<<DONE);
#define 	DONE_high	JPORT |= 1<<DONE;
#define		DONE_read	(JPIN & (1<<DONE))

