#ifndef _USB_API_H_
#define _USB_API_H_
//_____ I N C L U D E S ____________________________________________________
#include "typedefs.h"    		// Type definition
#include <stdio.h>
#include <avr/pgmspace.h>

u08 Enable_CDC_flush;

//________________________Usb FIFO init_____________________________________
typedef struct{
	u08 valid;
	u08 ConfigOK;
	u08 cfg0;
	u08 cfg1;  
	u08 interrupt;
} S_usb_endpoint;
#define 					RX_INT 0
#define 					TX_INT 1
#define 					NO_INT 2

//I need to define the EP number based on the processor 7 or 5
#if defined __AVR_AT90USB1287__ | __AVR_ATmega32U4__
#define possible_eps		7
#else
#define possible_eps		5
#endif
S_usb_endpoint	ep_init[possible_eps];

//Note that because interrupts occur at any time, an ISR might change the 
//value of the endpoint after a "Select_Endpoint" operation. To prevent this
//all ISRs should save and restore the EP number with these defines.
#define SaveEP()				EP_temp = UENUM
#define RestoreEP()				UENUM = EP_temp

//For stream handlers
//
#define USB_STREAM_TIMEOUT_MS  5
#define Stream_OK        	0 	//Command completed successfully, no error.
#define	Stream_Stalled    	1 	//The endpoint was stalled during the 
								//stream transfer by the host or device.
#define	Stream_Disconnected 1 	//Device was disconnected from the host 
								//during the transfer.
#define	Stream_Timeout      2 	//The host failed to accept or send the 
								//next packet within the software timeout 
								//period set by the USB_STREAM_TIMEOUT_MS macro.


// Define the action to associate to each USB event
// The usb state machine is time dependant so it is important that you don't
// block or delay them with your user functions

#define Usb_sof_action()		OneMsTimer();
#define Usb_wake_up_action()
#define Usb_resume_action()
#define Usb_suspend_action();
#define Usb_reset_action()
#define Usb_vbus_on_action()
#define Usb_vbus_off_action()
#define Usb_set_configuration_action()
#define Usb_id_action();

//_____ D E F I N I T I O N S ______________________________________________
#define usb_task_time		    5;		//ms
#define usb_flush_time		   	10;		//ms

volatile u08 CPU, EP_temp;
volatile u16 Vid, Pid, timer1,timer2;
u08 rx_counter, RX_EP, TX_EP;

//Public functions
void usb_init(u08 xtal, u16 Vid, u16 Pid);
void usb_user_endpoint_init(u08 conf_nb);
void usb_OUT_Callback(void *fct);
void usb_IN_Callback(void *fct);
void Vendor_Callback(void *fct);
void *Usb_Vendor_Callback;
void Trace_Interrupt(void);

//Private functions
void OneMsTimer(void);
void usb_init_ep(u08 num, u08 type, u08 dir, u08 nyet, u08 size, u08 bank, u08 interrupt);
void usb_user_endpoint_init(u08 conf_nb);
void usb_config_ep(u08 config0, u08 config1);
void usb_init_device (void);
void SetupEndpoints(void);
u08 usb_configured(void);
void bootloader_start(void);

u08 Endpoint_WaitUntilReady(void);
u08 Endpoint_RW_Stream(const void* Buffer, u16 Length);
u08 Endpoint_Write_Control_Stream(const void* Buffer, u16 Length);
u08 Endpoint_Read_Control_Stream(void* Buffer, u16 Length);




#endif  //_USB_API_H_
