//
// usb_debug.h
//

#ifndef _USB_DEBUG_H_
#define _USB_DEBUG_H_
#include "typedefs.h"			//debug definition

//Use the following options to enable the debug level required. The
//DEBUG flag must always be set to enbale debugging; after that any
//combination of flags can be set by un-commenting the following defines.
//Note that because these options generate alot of text meassages the 
//code size of your application will be increased substantially.
//Also not that the DEBUG_EP flag when selected will generate messages 
//every 1ms of 1000/sec. Consider yourself warned!
#ifdef DEBUG
#define DEBUG_INIT				//Trace the device hardware init. 
//#define DEBUG_ENUM			//Trace the enumeration process
//#define DEBUG_EP				//Trace the EP IN responses
//#define DEBUG_HID				//Trace the HID enumeration and report
//#define DEBUG_CDC				//Trace the CDC and line settings
#endif

// Function declarations
void debug_CDC_line_code(void);
void debug_to_uart(void);
void debug_HexDump(char* buf,int size);
void debug_ep_init(unsigned char cfg0,unsigned char cfg1);
void debug_dev_interupt(void);
void debug_ep_interupt(void);

#ifdef DEBUG
#define debug1(string,...);	fprintf_P(stdout,PSTR(string),__VA_ARGS__)
#else
#define debug1(string,...);	//this compiles to no code for all optimizations
#endif

#ifdef DEBUG_INIT
#define debug_init(string,...);	fprintf_P(stdout,PSTR(string),__VA_ARGS__)
#else
#define debug_init(string,...);	//this compiles to no code for all optimizations
#endif

#ifdef DEBUG_ENUM
#define debug_enum(string,...);	fprintf_P(stdout,PSTR(string),__VA_ARGS__)
#else
#define debug_enum(string,...);	//this compiles to no code for all optimizations
#endif

#ifdef DEBUG_EP
#define debug_ep_int(x)			debug_ep_int(x)
#define debug_ep(string,a)		printf_P(PSTR("\nEP%d "),UENUM);\
								printf_P(PSTR("T=%d "),Usb_data_toggle());\
								printf_P(PSTR(string),a);
#else
#define debug_ep_int(x)	
#define debug_ep(string,a)		//this compiles to no code for all optimizations
#endif

#ifdef DEBUG_CDC
#define debug_line(string,...);	fprintf_P(stdout,PSTR(string),__VA_ARGS__)
#else
#define debug_line(string,...);	//this compiles to no code for all optimizations
#endif

#ifdef DEBUG_HID
#define debug_hid(string,...);	fprintf_P(stdout,PSTR(string),__VA_ARGS__)
#else
#define debug_hid(string,...);	//this compiles to no code for all optimizations
#endif

#endif /* _USB_DEBUG_H_ */

