//../usb/library/include/usb_descriptors.h
//
// This file contains the usb descriptor structures
//

#ifndef _USB_DESCRIPTORS_H_
#define _USB_DESCRIPTORS_H_

//_____ I N C L U D E S ____________________________________________________
#include "usb_api.h"

//_____ M A C R O S ________________________________________________________
#define Usb_get_dev_desc_pointer	&(usb_dev_desc.bLength)
#define Usb_get_dev_desc_length		sizeof (usb_dev_desc)

//_____ U S B	D E F I N E _______________________________________________
#define CONF_ATTRIBUTES				USB_BUSPOWERED
//USB Device Descriptor
#define NB_CONFIGURATION			1
#ifdef CDC
#define EP_CONTROL_LENGTH			32				//64 for full speed USB2.0
#else
#define EP_CONTROL_LENGTH			64				//64 for full speed USB2.0
//#define EP_CONTROL_LENGTH			8				//64 for full speed USB2.0
#endif

#define LANG_ID 					0
#define MAN_INDEX					1
#define PROD_INDEX					2
#define SN_INDEX					3
#define Language_ID					0x0409			//English

#define DEVICE_STATUS				0x00 			// TBD
#define INTERFACE_STATUS			0x00 			// TBD

#define EP0	0										//Control
#define EP1	1										//CDC Tx
#define EP2	2										//CDC Rx
#define EP3	3										//CDC Int
#define EP4	4
#define EP5	5
#define EP6	6
#define EP7	7
//________________________Usb Request_______________________________________
typedef struct{
	u08 bmRequestType; 		u08 bRequest; 				u16 wValue;	
	u16 wIndex; 			u16 wLength;	
}	S_UsbRequest;

//________________________Usb Device Descriptor_____________________________
typedef struct {
	u08 bLength; 			u08 bDescriptorType;		u16 bscUSB;
	u08 bDeviceClass;		u08 bDeviceSubClass;		u08 bDeviceProtocol;
	u08 bMaxPacketSize0;	u16 idVendor;				u16 idProduct;
	u16 bcdDevice;			u08 iManufacturer;			u08 iProduct;
	u08 iSerialNumber;		u08 bNumConfigurations; 
}	S_usb_device_descriptor;

//________________________Usb Configuration Descriptor______________________
typedef struct {
	u08 bLength;			u08 bDescriptorType;		u16 wTotalLength;
	u08 bNumInterfaces;		u08 bConfigurationValue;	u08 iConfiguration;
	u08 bmAttibutes;		u08 MaxPower;
}	S_usb_configuration_descriptor;

//___________________Usb Interface Associated Descriptor____________________
typedef struct {
	u08 bLength;			u08 bDescriptorType;		u08 bFirstInterface;
	u08 bInterfaceCount;	u08 bFunctionClass;			u08 FunctionSubClass;
	u08 bFunctionProtocol; 	u08 iFunction;
}	S_usb_Interface_Association_descriptor;

//________________________Usb Interface Descriptor__________________________
typedef struct {
	u08 bLength;			u08 bDescriptorType;		u08 bInterfaceNumber;
	u08 bAlternateSetting;	u08 bNumEndpoints;			u08 bInterfaceClass;
	u08 bInterfaceSubClass;	u08 bInterfaceProtocol;		u08 iInterface;
}	S_usb_interface_descriptor;

//________________________Usb Endpoint Descriptor___________________________
typedef struct {
	u08 bLength;			u08 bDescriptorType;		u08 bEndpointAddress;
	u08 bmAttributes;		u16 wMaxPacketSize;			u08 bInterval;
} S_usb_endpoint_descriptor;

//________________________Usb Device Qualifier Descriptor___________________
typedef struct {
	u08 bLength;			u08 bDescriptorType;		u16 bscUSB;
	u08 bDeviceClass;		u08 bDeviceSubClass;		u08 bDeviceProtocol; 
	u08 bMaxPacketSize0;	u08 bNumConfigurations; 	u08 bReserved;
}	S_usb_device_qualifier_descriptor;

//______________________ Usb Language Descriptor____________________________
typedef struct {
	u08 bLength;			u08 bDescriptorType;		u16 wlangid;
}	S_usb_language_descriptor;

//_________________________ Usb User Descriptor _____________________________
typedef struct {
	S_usb_configuration_descriptor	cfg;
	S_usb_interface_descriptor		ifc0;
	S_usb_endpoint_descriptor		ep1;
	S_usb_endpoint_descriptor		ep2;
} S_usb_user_configuration_descriptor;

S_usb_device_descriptor usb_dev_desc;
S_usb_device_qualifier_descriptor usb_qual_desc;
S_usb_language_descriptor usb_user_language_id;

//__________________________ CDC Functional Descriptors _____________________
//See CDC Specification 1.1 for details 

//__________________________ Header Functional Descriptor ___________________
typedef struct {
	u08 bLength;			u08 bDescriptorType;		u08 bDscSubType;
	u16 bcdCDC;
} S_usb__CDC_header_descriptor;

//__________________________ Abstract Control Management Descriptor _________
typedef struct {
	u08 bLength;			u08 bDescriptorType;		u08 bDscSubType;
	u08 bmCapabilities;
} S_usb_CDC_ACM_descriptor;

//__________________________ Union Functional Descriptor_____________________
typedef struct {
	u08 bLength;			u08 bDescriptorType;		u08 bDscSubType;
	u08 bMasterIntf; 		u08 bSaveIntf0;
} S_usb_CDC_union_descriptor;

//__________________________ Call Management Functional Descriptor __________
typedef struct {
	u08 bLength;			u08 bDescriptorType;		u08 bDscSubType;
	u08 bmCapabilities;		u08 bDataInterface;
} S_usb_CDC_call_management_descriptor;

//_________________________ Usb CDC Descriptor ______________________________
typedef struct {
	S_usb_configuration_descriptor 				cfg;
	S_usb_interface_descriptor					ifc0;
	S_usb__CDC_header_descriptor				cdca;
	S_usb_CDC_call_management_descriptor 		cdcb;
	S_usb_CDC_ACM_descriptor					cdcc;
	S_usb_CDC_union_descriptor					cdcd;
	S_usb_endpoint_descriptor					ep3;
	S_usb_interface_descriptor					ifc1;
	S_usb_endpoint_descriptor					ep1;
	S_usb_endpoint_descriptor					ep2;
} S_usb_CDC_descriptor;

//_________________________ Usb CDC IA Descriptor ____________________________
typedef struct {
	S_usb_configuration_descriptor 				cfg;
	S_usb_Interface_Association_descriptor		iad1;
	S_usb_interface_descriptor					ifc0;
	S_usb__CDC_header_descriptor				cdca;
	S_usb_CDC_call_management_descriptor 		cdcb;
	S_usb_CDC_ACM_descriptor					cdcc;
	S_usb_CDC_union_descriptor					cdcd;
	S_usb_endpoint_descriptor					ep3;
	S_usb_interface_descriptor					ifc1;
	S_usb_endpoint_descriptor					ep1;
	S_usb_endpoint_descriptor					ep2;
} S_usb_CDCIA_descriptor;

//_________________________ Usb CDC IAD with Simpleport Descriptor ___________
typedef struct {
	S_usb_configuration_descriptor 				cfg;
	S_usb_Interface_Association_descriptor		iad1;
	S_usb_interface_descriptor					ifc0;
	S_usb__CDC_header_descriptor				cdca;
	S_usb_CDC_call_management_descriptor 		cdcb;
	S_usb_CDC_ACM_descriptor					cdcc;
	S_usb_CDC_union_descriptor					cdcd;
	S_usb_endpoint_descriptor					ep6;
	S_usb_interface_descriptor					ifc1;
	S_usb_endpoint_descriptor					ep4;
	S_usb_endpoint_descriptor					ep5;
	S_usb_interface_descriptor					ifc3;
	S_usb_endpoint_descriptor					ep2;
	S_usb_endpoint_descriptor					ep3;
} S_usb_CDC_simpleport_descriptor;

//_________________________ Usb app with CDC Descriptor ______________________
typedef struct {
	S_usb_configuration_descriptor 				cfg;
	S_usb_Interface_Association_descriptor		iad1;
	S_usb_interface_descriptor					ifc0;
	S_usb__CDC_header_descriptor				cdca;
	S_usb_CDC_call_management_descriptor 		cdcb;
	S_usb_CDC_ACM_descriptor					cdcc;
	S_usb_CDC_union_descriptor					cdcd;
	S_usb_endpoint_descriptor					ep6;
	S_usb_interface_descriptor					ifc1;
	S_usb_endpoint_descriptor					ep4;
	S_usb_endpoint_descriptor					ep5;
	S_usb_interface_descriptor					ifc2;
	S_usb_endpoint_descriptor					ep2;
	S_usb_endpoint_descriptor					ep3;
} S_usb_app_with_CDC_descriptor;

//_________________________ Usb CDC with UrJTAG Descriptor ___________________
typedef struct {
	S_usb_configuration_descriptor 				cfg;
	S_usb_Interface_Association_descriptor		iad1;
	S_usb_interface_descriptor					ifc1;
	S_usb__CDC_header_descriptor				cdca;
	S_usb_CDC_call_management_descriptor 		cdcb;
	S_usb_CDC_ACM_descriptor					cdcc;
	S_usb_CDC_union_descriptor					cdcd;
	S_usb_endpoint_descriptor					ep3;
	S_usb_interface_descriptor					ifc2;
	S_usb_endpoint_descriptor					ep1;
	S_usb_endpoint_descriptor					ep2;
	S_usb_interface_descriptor					ifc0;
	S_usb_endpoint_descriptor					ep4;
	S_usb_endpoint_descriptor					ep5;
} S_usb_libUSB_CDC_descriptor;

//_____ U S B	H I D	D E S C R I P T O R __________________________________
typedef struct {
		u08 bLength;								// Size of this descriptor in bytes
		u08 bDescriptorType;						// HID descriptor type
		u16 bscHID;									// Binay Coded Decimal Spec. release
		u08 bCountryCode;							// Hardware target country
		u08 bNumDescriptors;						// Number of HID class descriptors to follow
		u08 bRDescriptorType;						// Report descriptor type
		u16 wDescriptorLength;						// Total length of Report descriptor
} S_usb_hid_descriptor;

typedef	struct {
	int 	size;									// Size of report in bytes
	u08* 	report_ptr;								// pointer to start of HID_report
} S_hid_report;

//extern S_hid_report HID_report[2];
extern S_hid_report HID_report[3];

typedef struct
{
	S_usb_configuration_descriptor 				cfg_kbd;
	S_usb_interface_descriptor					ifc_kbd;
	S_usb_hid_descriptor						hid_kbd;
	S_usb_endpoint_descriptor					ep1_kbd;
	S_usb_interface_descriptor					ifc1;
	S_usb__CDC_header_descriptor				cdca;
	S_usb_CDC_call_management_descriptor 		cdcb;
	S_usb_CDC_ACM_descriptor					cdcc;
	S_usb_CDC_union_descriptor					cdcd;
	S_usb_endpoint_descriptor					ep2;
	S_usb_interface_descriptor					ifc2;
	S_usb_endpoint_descriptor					ep3;
	S_usb_endpoint_descriptor					ep4;
} S_usb_HID_KBD_CDC_descriptor;


typedef struct
{
	S_usb_configuration_descriptor 				cfg_kbd;
	S_usb_interface_descriptor					ifc_kbd;
	S_usb_hid_descriptor						hid_kbd;
	S_usb_endpoint_descriptor					ep1_kbd;
} S_usb_HID_keyboard_descriptor;

typedef struct
{
	S_usb_configuration_descriptor 				cfg_mouse;
	S_usb_interface_descriptor					ifc_mouse;
	S_usb_hid_descriptor						hid_mouse;
	S_usb_endpoint_descriptor					ep1_mouse;
} S_usb_HID_mouse_descriptor;

typedef struct
{
	S_usb_configuration_descriptor 				cfg_mouse;
	S_usb_interface_descriptor					ifc_mouse;
	S_usb_hid_descriptor						hid_mouse;
	S_usb_endpoint_descriptor					ep1_mouse;
	S_usb_interface_descriptor					ifc_kbd;
	S_usb_hid_descriptor						hid_kbd;
	S_usb_endpoint_descriptor					ep2_kbd;
} S_usb_HID_composite_descriptor;

typedef struct
{
	S_usb_configuration_descriptor 				cfg_mouse;
	S_usb_interface_descriptor					ifc_mouse;
	S_usb_hid_descriptor						hid_mouse;
	S_usb_endpoint_descriptor					ep1_mouse;
	S_usb_interface_descriptor					ifc_kbd;
	S_usb_hid_descriptor						hid_kbd;
	S_usb_endpoint_descriptor					ep2_kbd;
	S_usb_interface_descriptor					ifc_listen;
	S_usb_hid_descriptor						hid_listen;	
	S_usb_endpoint_descriptor					ep3_listen;
	S_usb_endpoint_descriptor					ep4_listen;	
} S_usb_HID_composite_listen_descriptor;

/*{
	S_usb_configuration_descriptor 				cfg_mouse;
	S_usb_interface_descriptor					ifc_mouse;
	S_usb_hid_descriptor						hid_mouse;
	S_usb_endpoint_descriptor					ep1_mouse;
	S_usb_interface_descriptor					ifc_kbd;
	S_usb_hid_descriptor						hid_kbd;
	S_usb_endpoint_descriptor					ep2_kbd;
	S_usb_configuration_descriptor				cfg_listen;
	S_usb_interface_descriptor					ifc_listen;
	S_usb_endpoint_descriptor					ep3_listen;
	S_usb_endpoint_descriptor					ep4_listen;	
} S_usb_HID_composite_listen_descriptor;*/

typedef struct
{
	S_usb_configuration_descriptor				cfg_mouse;
	S_usb_interface_descriptor					ifc_mouse;
	S_usb_hid_descriptor						hid_mouse;
	S_usb_endpoint_descriptor					ep1_mouse;
	S_usb_interface_descriptor					ifc_kbd;
	S_usb_hid_descriptor						hid_kbd;
	S_usb_endpoint_descriptor					ep2_kbd;
	S_usb_interface_descriptor					ifc_control;
	S_usb_endpoint_descriptor					ep3_Tx;
	S_usb_endpoint_descriptor					ep4_Rx;
} S_super_composite_descriptor;

//This structure allows us to define which conf_desc to use while
//maintaining a constant size structure as required by the library use.

typedef struct
{
	int 	size;			//size of configuration descriptor in bytes
	u08*	conf_desc_ptr;	//pointer to configuration descriptor
}S_conf_desc;


extern S_conf_desc conf_desc[1];

typedef struct
{
	u08*	string;
	u08 	size;
}S_string_desc;

extern S_string_desc string_desc[3];

typedef struct
{
	u08		BitInterrupt:2,
			BitBank:2,
			BitEndpoint:4;
}S_EP_definition;


extern S_EP_definition EP_definition[7];

#endif	//_USB_DESCRIPTORS_H_
