//usb_drv.h
#ifndef _USB_DRV_H_
#define _USB_DRV_H_

#define Usb_data_toggle()		((UESTA0X&MSK_DTSEQ) >> 2)
#define MSK_DTSEQ				0x0C

//_____ MACROs ______________________________________________
//keep this order (set StallRq/clear RxSetup) or an
//OUT request following the SETUP may be acknowledged
#define Usb_stall_and_ack_setup();\
      	Endpoint_enable_stall();\
      	Endpoint_ack_setup_receive();\
      	return;
		
#define clock_prescale_set(x) \
{ \
        uint8_t tmp = _BV(CLKPCE); \
        __asm__ __volatile__ ( \
                "in __tmp_reg__,__SREG__" "\n\t" \
                "cli" "\n\t" \
                "sts %1, %0" "\n\t" \
                "sts %1, %2" "\n\t" \
                "out __SREG__, __tmp_reg__" \
                : /* no outputs */ \
                : "d" (tmp), \
                  "M" (_SFR_MEM_ADDR(CLKPR)), \
                  "d" (x) \
                : "r0"); \
}		

//USB EndPoint
#define EP_CONTROL   				0
#define MSK_EP_DIR    				0x7F
#define MSK_UADD        			0x7F
//Parameters for endpoint configuration
#define Control            			0<<6
#define Isochronous		       	 	1<<6
#define Bulk   		            	2<<6
#define Interrupt		        	3<<6

#define OUT            				0
#define IN             				1

#define SIZE_8                 	 	0<<4
#define SIZE_16                 	1<<4
#define SIZE_32                 	2<<4
#define SIZE_64                 	3<<4
#define SIZE_128                	4<<4
#define SIZE_256                	5<<4
#define SIZE_512                	6<<4
#define SIZE_1024               	7<<4

#define BANK_1                 		0<<2
#define BANK_2               		1<<2

#define NYET_ENABLED            	0<<1
#define NYET_DISABLED           	1<<1

#define usb_configure_endpoint(num, type, dir, size, bank, nyet)\
		( Usb_select_endpoint(num),\
                  usb_config_ep(((type) |(nyet) |(dir)),((size) |(bank))))


//General USB management
#define Usb_enable_regulator()  			(UHWCON |= (1<<UVREGE))
#define Usb_disable_uid_pin()       		(UHWCON &= ~(1<<UIDE))
#define Usb_force_device_mode()     		(Usb_disable_uid_pin(), UHWCON |= (1<<UIMOD))

#define Usb_enable()                		(USBCON |= (1<<USBE))
#define Usb_disable()   					USBCON &= ~(1<<USBE)
#define Usb_select_device()         		(USBCON  &= ~(1<<HOST))
#define Usb_freeze_clock()          		(USBCON  |=  (1<<FRZCLK))
#define Usb_unfreeze_clock()        		(USBCON  &= ~(1<<FRZCLK))
#define Usb_enable_vbus_interrupt() 		(USBCON  |=  (1<<VBUSTE))

#define Usb_enable_vbus_pad()         		(USBCON |= (1<<OTGPADE))
#define Usb_disable_vbus_pad()        		(USBCON &= ~(1<<OTGPADE))

#define Usb_enable_reset_interrupt()		(UDIEN   |=  (1<<EORSTE))

#define Is_usb_vbus_high()          		(USBSTA &   (1<<VBUS))
#define Is_usb_id_device()          		(USBSTA &   (1<<ID))

#define Usb_ack_id_transition()     		(USBINT  = ~(1<<IDTI))
#define Usb_ack_vbus_transition()   		(USBINT  = ~(1<<VBUSTI))
#define Is_usb_id_transition()      		(USBINT &   (1<<IDTI))
#define Is_usb_vbus_transition()    		(USBINT &   (1<<VBUSTI))

// USB Device management
#define Usb_attach()                 		(UDCON   &= ~(1<<DETACH))
#define Usb_detach()    					UDCON  |= 1<<DETACH
#define Usb_ack_resume()                    (UDINT   = ~(1<<EORSMI))
#define Usb_ack_wake_up()                  	(UDINT   = ~(1<<WAKEUPI))
#define Usb_ack_reset()                     (UDINT   = ~(1<<EORSTI))
#define Usb_ack_sof()                       (UDINT   = ~(1<<SOFI))
#define Usb_ack_suspend()                   (UDINT   = ~(1<<SUSPI))
#define Is_usb_wake_up()                    (UDINT &   (1<<WAKEUPI))
#define Usb_enable_address()                (UDADDR  |=  (1<<ADDEN))
#define Usb_configure_address(addr)         (UDADDR  =   (UDADDR & (1<<ADDEN)) | ((u08)addr & MSK_UADD))


// General endpoint management
#define Endpoint_select_endpoint(ep)		(UENUM = (u08)ep )
#define Endpoint_reset_endpoint(ep)			(UERST   =   1 << (u08)ep, UERST  =  0)

#define Endpoint_enable_endpoint()			(UECONX  |=  (1<<EPEN))
#define Endpoint_disable_endpoint()			(UECONX  &=  ~(1<<EPEN))
#define Endpoint_enable_stall()				(UECONX  |=  (1<<STALLRQ))
#define Endpoint_reset_data_toggle()		(UECONX  |=  (1<<RSTDT))
#define Endpoint_clear_stall()				(UECONX  |=  (1<<STALLRQC))
#define Is_usb_endpoint_enabled()			(UECONX & (1<<EPEN))

#define Endpoint_allocate_memory()			(UECFG1X |=  (1<<ALLOC))
#define Endpoint_deallocate_memory()		(UECFG1X &=  ~(1<<ALLOC))
#define ConfigOK()							(UESTA0X &  (1<<CFGOK))
#define IN_Busy()							(UESTA0X &  (1<<NBUSYBK1 | 1<<NBUSYBK0))

#define Usb_FIFO_available()				(UEINTX&(1<<RWAL))
#define Usb_FIFO_full()						!(UEINTX&(1<<RWAL))
#define Is_usb_read_control_enabled()		(UEINTX&(1<<TXINI))
#define Endpoint_setup_received_int()		(UEINTX&(1<<RXSTPI))
#define Endpoint_OUT_received_int()			(UEINTX&(1<<RXOUTI))
#define Is_usb_in_ready()					(UEINTX&(1<<TXINI))

#define Endpoint_ack_setup_receive()		(UEINTX &= ~(1<<RXSTPI));
#define Usb_ack_receive_out()   			UEINTX &= ~(1<<RXOUTI);\
											UEINTX &= ~(1<<FIFOCON);
#define Usb_ack_in_ready()					UEINTX &= ~(1<<TXINI);\
											UEINTX &= ~(1<<FIFOCON);
#define Usb_send_in()						(UEINTX &= ~(1<<FIFOCON));
#define Usb_send_control_in()				(UEINTX &= ~(1<<TXINI));

#define Usb_read_u08()						(UEDATX)
#define Usb_write_byte(byte)				(UEDATX  =   (u08)byte)
#define Usb_write_u16(u16)					(UEDATX  =   (u08)&u16[0]),\
											(UEDATX  =   (u08)&u16[1])
#define Usb_write_u32(u32)					(UEDATX  =   (u08)&u32[0]),\
											(UEDATX  =   (u08)&u32[1]),\
											(UEDATX  =   (u08)&u32[2]),\
											(UEDATX  =   (u08)&u32[3])

#define Usb128_byte_counter()				(((u16)UEBCHX) << 8) | (UEBCLX)
#define Usb16_byte_counter()				((u08)UEBCLX)

#define Usb_interrupt_flags()				(UEINT != 0x00)
#endif  // _USB_DRV_H_



