// usb_standard_request.h
//
// This file contains the USB endpoint 0 management routines corresponding to
// the standard enumeration process (refer to chapter 9 of the USB
// specification). The enumeration parameters (descriptor tables) are 
// contained in the usb_descriptors.c file.

#ifndef _USB_STANDARD_REQUEST_H_
#define _USB_STANDARD_REQUEST_H_

//_____ I N C L U D E S ____________________________________________________
#include "usb_descriptors.h"

//_____ S T A N D A R D    D E F I N I T I O N S ___________________________
//Device descriptor types for bmRequestType
// format of bmRequestType byte
#define	bmRT_DIR_MASK						(0x1<<7)
#define	bmRT_DIR_IN							(1<<7)
#define	bmRT_DIR_OUT						(0<<7)
#define StdRequest							0x00
#define ClassRequest						0x20
#define VendorRequest						0x40

#define CRTL								0x00
#define ISOC								0x01
#define BULK								0x02
#define INTR								0x03

//Standard Requests
#define GET_STATUS							0x00
#define CLEAR_FEATURE						0x01		//see FEATURES below
#define SET_FEATURE							0x03		//see FEATURES below
#define SET_ADDRESS							0x05
#define GET_DESCRIPTOR						0x06
#define SET_DESCRIPTOR						0x07
#define GET_CONFIGURATION					0x08
#define SET_CONFIGURATION					0x09
#define GET_INTERFACE						0x0A
#define SET_INTERFACE						0x0B
#define SYNCH_FRAME							0x0C

#define REQUEST_DEVICE_STATUS				0x80
#define REQUEST_INTERFACE_STATUS			0x81
#define REQUEST_ENDPOINT_STATUS				0x82
#define ZERO_TYPE							0x00
#define INTERFACE_TYPE						0x01
#define ENDPOINT_TYPE						0x02

//Descriptor Types
#define DEVICE_DESCRIPTOR					0x01
#define CONFIGURATION_DESCRIPTOR			0x02
#define STRING_DESCRIPTOR					0x03
#define INTERFACE_DESCRIPTOR				0x04
#define ENDPOINT_DESCRIPTOR					0x05
#define DEVICE_QUALIFIER					0x06
#define IAD_DESCRIPTOR						0x0B

//Standard Features
//#define FEATURE_DEVICE_REMOTE_WAKEUP		0x01
#define FEATURE_ENDPOINT_HALT				0x00

//Device Status
#define USB_BUSPOWERED						0x80
//#define USB_SELFPOWERED					0xC0
//#define USB_REMOTEWAKEUP					0xA0

//_____ D E C L A R A T I O N ______________________________________________
Bool 	usb_user_read_request(u08 type, u08 request);
u08 	bmRequestType, data_to_transfer, bmRequest;
u16 	wValue,wIndex,wLength;
volatile u08 	*pbuffer;
u08 stateep1,stateep2;

//_____ Public functions  __________________________________________________
void	handle_get_request(void);
void	usb_process_request( void);
#endif  // _USB_STANDARD_REQUEST_H_

