//stdio_driver.c
//_____ I N C L U D E S ____________________________________________________
#include "include/stdio_driver.h"
#include "include/usb16.h"

void uart_init(u32 baud)
{
	SerialDDR   |= 1<<TXD;							//allow txd to be an output
	UBRR1H = (u08)((((u32)F_CPU)/((u32)baud*8)-1)>>8);
	UBRR1L = (u08)( ((u32)F_CPU)/((u32)baud*8)-1);
	UCSRA = 1<<U2X1;								//double rate allows a lower
	UCSRB = ((1<<RXEN) | (1<<TXEN));				//bit error at 115200
}

u08 uart_rx_ready(void)
{
	return(UCSRA & (1<<RXC));
}

u08 uart_getchar(void)
{
	while ( !(UCSRA & (1<<RXC)) ){}
	return(UDR);
}

void uart_putchar(u08 data)
{
  	while ( !(UCSRA & (1<<UDRE)) ){}
	UDR1 = data;									// Start transmition 
}

//_______________The following two functions provide stdio__________________
//
// Send usb character to host. Note that the data is only queued to be sent.
// The actual data transfer occurs when either the tx buffer is full or when
// the flush timer times out. 
//  
int uart_std_putchar(char c, FILE *stream)
{
	uart_putchar(c);
	if (c == '\n'){
    	uart_putchar('\r');}
  	return 0;
}

/*
 * Receive a character from the uart port.
 *
 * This features a simple line-editor that allows for delete to
 * edit the characters entered, until either CR or NL is entered.
 * Printable characters entered will be echoed using uart_std_putchar().
 *
 * Editing characters:
 *
 * \b (BS) or \177 (DEL) delete the previous character
 * \t (tab) will be replaced by a single space
 *
 * All other control characters will be ignored.
 *
 * The internal line buffer is RX_BUFSIZE characters long, which
 * includes the terminating \n (but no terminating \0).  If the buffer
 * is full (i. e., at RX_BUFSIZE-1 characters in order to keep space for
 * the trailing \n), any further input attempts will send a \a (BEL character)
 * to uart_putchar() , although line editing is still allowed.
 *
 * Successive calls to uart_getchar() will be satisfied from the
 * internal buffer until that buffer is emptied again.
 */
#define RX_BUFSIZE 	20
 
u08 *uart_rxp = 0;
int uart_std_getchar(FILE *stream)
{
	u08 RxChar, *CharPtr, TextBuffer[RX_BUFSIZE];

  	if (uart_rxp == 0)
    		for (CharPtr = TextBuffer;;){
			RxChar = uart_getchar();
			if (RxChar == CR){						// behaviour similar to Unix stty ICRNL 
	  			RxChar = new_line;
			}
			if (RxChar == new_line){				//CR to newline
	    		*CharPtr = RxChar;
	    		uart_std_putchar(RxChar, stream);
	    		uart_rxp = TextBuffer;
	    		break;
	  		}else if(RxChar == tab){				//tab to space
	  			RxChar = ' ';
			};

			if ((RxChar >= (u08)' ' && RxChar <= (u08)'\x7e') ||RxChar >= (u08)'\xa0'){
	    		if (CharPtr == TextBuffer + RX_BUFSIZE - 1){
	      			uart_std_putchar(bell, stream);	//no room in buffer
	    		}else{
					*CharPtr++ = RxChar;			//add ch to buffer
					uart_std_putchar(RxChar, stream);//echo character
	      		}
	    		continue;
	  		}
			if((RxChar == '\b') | (RxChar == '\x7f')){//backspace or del
	    		if (CharPtr > TextBuffer){
					uart_std_putchar(backspace, stream);
					uart_std_putchar(space, stream);
					uart_std_putchar(backspace, stream);
					CharPtr--;
	      		}
	    	}
   		}
  		RxChar = *uart_rxp++;						//last char in buffer
  		if (RxChar == new_line){
    		uart_rxp = 0;
		}
  	return RxChar;
}


