
//_____  I N C L U D E S ___________________________________________________ 
#include <avr/io.h>  
#include <avr/interrupt.h>
#include "include/CDC_api.h"
#include "include/typedefs.h"
#include "include/usb_api.h"
#include "include/usb_task.h"
#include "include/usb_drv.h"
#include "include/usb_descriptors.h"
#include "include/usb_standard_request.h"
#include <avr/eeprom.h>
#include <avr/power.h>				//clock prescaler
#include <avr/pgmspace.h>
#include <stdio.h>
#include "include/stdio_driver.h"
#include "include/usb_debug.h"
#define USB_STDIO

u08 Endpoint_WaitUntilReady(void)
{
	u16 TimeoutMSRem = USB_STREAM_TIMEOUT_MS;
	Usb_ack_sof();
	while (!(Usb_FIFO_available())){
		if (!(usb_configured())){
			return Stream_Disconnected;
//usb_api.c:356: warning: implicit declaration of function 'Endpoint_IsStalled'			
//		}else if (Endpoint_IsStalled()){
//			return Stream_Stalled;
		}
		if (UDINT & (1<< SOFI)){
			Usb_ack_sof();
			if (!(TimeoutMSRem--))
				return Stream_Timeout;
		}
	}
	return Stream_OK;
}



/* Read/Write N little endian bytes from the buffer to the endpoint,
*  sending full packets to the host when MSB is set. The last packet is
*  not automatically sent(Wr)/discarded(Rd); the user is responsible for 
*  manually sending/discarding the last packet via the Usb_send_in() macro. 
*
*  Note 	This routine should not be used on CONTROL type endpoints.
*  			For Reads MSB = 0
*
*  Buffer   Pointer to the source data read buffer.
*  Length   Number of bytes to read for the currently selected endpoint.
*
*  return 	Zero for success or a value from the endpoint errors defined
*  in the api.h file
*/
u08 Endpoint_RW_Stream(const void* Buffer, u16 Length)
{
	u08* DataStream   = (u08*)Buffer;
	u08  ErrorCode;
	
	if ((ErrorCode = Endpoint_WaitUntilReady()))
	  return ErrorCode;

	while (Length--)
	{
		if (!(Usb_FIFO_available()))
		{
			Usb_send_in();
			if ((ErrorCode = Endpoint_WaitUntilReady()))
			  return ErrorCode;
		}
		if(Length &0x8000){
			Usb_write_byte(*(DataStream++));
		}else{
			*(DataStream++) = Usb_read_u08();
		}
	}
	return Stream_OK;
}

/** Writes N little endian bytes to the CONTROL type endpoint from the buffer,
*  sending full packets to the host as needed. The host OUT acknowledgement 
*  is not automatically cleared in both failure and success states; the user 
*  is responsible for manually clearing the setup OUT to finalize the 
*  transfer via the Usb_send_control_in() macro.
*
*  Note 	This routine should only be used on CONTROL type endpoints.
*
*  Warning 	Unlike the standard stream read/write commands, the control
*           stream commands cannot be chained together; i.e. the entire 
*           stream data must be read or written at the one time.
*
*  Buffer  	Pointer to the source data buffer to read from.
*  Length  	Number of bytes to read for the currently selected endpoint
*           into the buffer.
*
*  return 	Zero for success or a value from the endpoint errors defined 
*           in the api.h file
*/
u08 Endpoint_Write_Control_Stream(const void* Buffer, u16 Length)
{
	u08* DataStream = (u08*)Buffer;
	u08     ZLP=1;
	
	while (Length && !(Endpoint_OUT_received_int())){
		while (!(Is_usb_in_ready()));
		while (Length && (Usb_byte_counter() < EP_CONTROL_LENGTH)){
			Usb_write_byte(*(DataStream++));	
			Length--;
		}
		ZLP = (Usb_byte_counter() == EP_CONTROL_LENGTH);
		Usb_send_control_in();
	}
	
	if (Endpoint_OUT_received_int())
//		return ENDPOINT_RWCSTREAM_ERROR_HostAborted;
	
	if (ZLP){
		while (!(Is_usb_in_ready()));
		Usb_send_control_in();
	}
	while (!(Endpoint_OUT_received_int()));
	return Stream_OK;
}


/** Reads N little endian bytes from the CONTROL endpoint and saves them in
*  a buffer, discarding fully read packets from the host as needed. The 
*  device IN acknowledgement is not automatically sent after success or 
*  failure states; the user is responsible for manually sending the
*  setup IN to finalize the transfer via the Usb_send_control_in() macro.
*
*  Note 	This routine should only be used on CONTROL type endpoints.
*
*  Warning 	Unlike the standard stream read/write commands, the control 
*           stream commands cannot be chained together; i.e. the entire
*           stream data must be read or written at the one time.
*
*  Buffer  	Pointer to the destination data buffer to write to.
*  Length  	Number of bytes to send via the currently selected endpoint.
*
*  return 	Zero for success or a value from the endpoint errors defined 
*           in the api.h file
*/
u08 Endpoint_Read_Control_Stream(void* Buffer, u16 Length)
{
	u08* DataStream = (u08*)Buffer;
	
	while (Length){
		while (!(Endpoint_OUT_received_int()));
		while (Length && Usb_byte_counter()){
			*(DataStream++) = Usb_read_u08();
			Length--;
		}
		Usb_ack_receive_out();
	}
	while (!(Is_usb_in_ready()));
	return Stream_OK;
}


