//
//../usb_library/usb_CDC_descriptors.c
//This file contains the usb parameters that uniquely identify the
//usbCDC16 application through descriptor tables.
//_____ I N C L U D E S ____________________________________________________

#include "../Gecko_library/include/usb_api.h"
#include "../Gecko_library/include/usb_drv.h"
#include "../Gecko_library/include/usb_descriptors.h"
#include "../Gecko_library/include/usb_standard_request.h"
#include <avr/pgmspace.h>

#define Manufacturer 	"USB16"
#define Product			"Serial CDC demo"
#define Serial 			"GPL/GPL2"

// usb device descriptor
S_usb_device_descriptor PROGMEM usb_dev_desc =
{
	sizeof(usb_dev_desc),					// Size of descriptor in bytes
	DEVICE_DESCRIPTOR,					 	// DEVICE descriptor type
	0x0200,									// BCD - USB Spec Release Number 
	0x02,									// Class Code
	0,										// Subclass code
	0,										// Protocol code
	EP_CONTROL_LENGTH,						// Max size EP0,usb_descriptors.h
	0x03eb,									// Vendor ID
	0x2020,									// Product ID 
	0x1000,									// BCD - Device release number
	MAN_INDEX,								// Manufacturer string index
	PROD_INDEX,								// Product string index
	SN_INDEX,								// Serial number string index
	1										// Number of configurations
};

// usb configuration descriptor 			
S_usb_CDC_descriptor PROGMEM usb_conf_desc = {
 {  sizeof(S_usb_configuration_descriptor),	// Size of descriptor in bytes
	CONFIGURATION_DESCRIPTOR, 				// CONFIGURATION descriptor type
	sizeof(usb_conf_desc),	  				// Total length of data for cfg
	0x02,					  				// Number of interfaces in cfg
	0x01,					  				// Index value of  configuration
	0,					  					// Configuration string index
	CONF_ATTRIBUTES,						// Attributes, usb_descriptors.h
	100,					 				// Max power consumption (2X mA)
 },
 //Interface 0 Descriptor
 {  	sizeof(S_usb_interface_descriptor),	// Size of descriptor in bytes
	INTERFACE_DESCRIPTOR,	 				// INTERFACE descriptor type
	0,					  					// Interface Number
	0,					  					// Alternate Setting Number
	1,					  					// Number of endpoints in intf
	0x02,			  						// Class code where CDC ACM=0x02
	0x02, 									// Subclass: 0x02=communication
	0x01,				 					// Protocol: 0x01=AT commands
	0,
 },
 // CDC Class-Specific Descriptors 
// see Table 25 in USB CDC Specification 1.1 
 {  sizeof(S_usb__CDC_header_descriptor),	//
	0x24,									//CS_INTERFACE
	0x00,									//DSC_FN_HEADER
	0x0110									//
 },
 { 	sizeof(S_usb_CDC_call_management_descriptor),//
 	0x24,									//CS_INTERFACE
	0x01,									//DSC_FN_CALL_MGT
	0x03,									//
	0x01									//CDC_DATA_INTF_ID
 },
 { 	sizeof(S_usb_CDC_ACM_descriptor),		//
 	0x24,						//CS_INTERFACE
	0x02,						//DSC_FN_ACM- Abstract Control Management
	0x06						//
 },
 { 	sizeof(S_usb_CDC_union_descriptor),		//
 	0x24,									//CS_INTERFACE
	0x06,									//DSC_FN_UNION
	0x00,									//CDC_COMM_INTF_ID
	0x01									//CDC_DATA_INTF_ID
 },
 //Usb Endpoint 3 Descriptor
 { 	sizeof(S_usb_endpoint_descriptor),		//
  	ENDPOINT_DESCRIPTOR,					//
  	(0x80|EP3),								//Endpoint number
  	INTR,									//Transfer type BULK,INTR,ISOC
 	32,										//EP fifo size in bytes
  	0xFF,									//Interval ms 
 },
 //Interface 1 Descriptor
 { 	sizeof(S_usb_interface_descriptor),		// Size of descriptor in bytes
  	INTERFACE_DESCRIPTOR,					// INTERFACE descriptor type
	1,										// Interface Number
	0,										// Alternate Setting Number
	2,										// Number of endpoints in intf
	0x0A,									// Class code:0x0A=CDC ACM i/f
	0,										// Subclass code
	0,										// Protocol code
	0,						  				// Interface string index
 },
//Usb Endpoint 1 Descriptor
 { 	sizeof(S_usb_endpoint_descriptor),		//Size of endpoint in bytes
  	ENDPOINT_DESCRIPTOR,					//
  	(0x80|EP1),								//Endpoint number
  	BULK,									//EP transfer type attribute
  	32,										//EP fifo size in bytes
  	0x00,									//Endpoint interval
 },
 //Usb Endpoint 2 Descriptor
 { 	sizeof(S_usb_endpoint_descriptor),		//Size of endpoint in bytes
  	ENDPOINT_DESCRIPTOR,					//
  	EP2,									//Endpoint number
  	BULK,									//EP transfer type attribute
	32,										//EP fifo size in bytes
	0x00									//Endpoint interval
 }
};

S_conf_desc	conf_desc[]={
	{sizeof(usb_conf_desc),
	(u08*)&usb_conf_desc}
};

// usb_user_language_id
S_usb_language_descriptor PROGMEM usb_user_language_id = {
	sizeof(usb_user_language_id)
, 	STRING_DESCRIPTOR
, 	Language_ID								//LANGUAGE_ID = English
};

u08 Manufacturer_string[] PROGMEM = {Manufacturer};
u08 Product_string[] PROGMEM = {Product};
u08 Serial_string[] PROGMEM = {Serial};

S_string_desc 	string_desc[] = {
	{Manufacturer_string,sizeof(Manufacturer)-1},
	{Product_string,sizeof(Product)-1},
	{Serial_string,sizeof(Serial)-1}
};

S_EP_definition EP_definition[] = {
	{NO_INT,BANK_1,EP0},
	{NO_INT,BANK_1,EP1},
	{NO_INT,BANK_1,EP2},
	{NO_INT,BANK_1,EP3}
};


