//usb_HID_descriptors.c
//This file contains the usb parameters that uniquely identify the
//USBHID16 application through descriptor tables.

//_____ I N C L U D E S ____________________________________________________
#include "../usblibrary/include/usb_descriptors.h"
#include "../usblibrary/include/usb_standard_request.h"
#include <avr/pgmspace.h>
#include "../usblibrary/include/HID_report.h"

#define Manufacturer 	"USB16"
#define Product			"HID demo"
#define Serial 			"GPL/GPL2"

//_____ D E F I N I T I O N ________________________________________________
//This is the keyboard HID with a 2 byte report
//The first byte defines the key modifier while the second byte defines the 
//actual key code. 
//The key modifiers include shift, alt and ctrl
//The key code itself has a value of 0-101 which defines the 101 keyboard
//Report: (modifier byte, keycode byte)
u08 PROGMEM Usb_hid_report[0x25] = {
	Usage_page (Generic_desktop),		//Start report
	Usage 0x06,          				// Usage (Keyboard)
	Collection Application,         	//
		Usage_page (Keyboard),      	//
		Usage_minimum (224),			//
		Usage_maximum (231),			//
		Logical_minimum (0),			//
		Logical_maximum (1),			//
		Report_size (1),				//1 byte report
		Report_count (8),				//
		Input 0x02,          			//Input (Data, Variable, Absolute)
		Usage_minimum (0),         		//
		Usage_maximum (101),        	//
		Logical_minimum (0),        	//
		Logical_maximum (101),      	//
		Report_size (8),          		//8 bit byte
		Report_count (1),				//one return code
		Input 0x00,          			//Input (Data, Array)
	End_collection              		//End Collection
};

S_hid_report HID_report[]= {
	{sizeof(Usb_hid_report),
	Usb_hid_report}
};
// usb_user_device_descriptor
S_usb_device_descriptor PROGMEM usb_dev_desc =
{
    sizeof(usb_dev_desc),   				// Descriptor size in bytes
    DEVICE_DESCRIPTOR,      				// DEVICE descriptor type
    0x0200,									// USB BCD Release Number
    0,      	       						// Class Code
    0,                   					// Subclass code
    0,                   					// Protocol code
    64,          							// Max packet size for EP0
    0x03EB,       							// Vendor ID
	0x1020,      							// Product ID: HID Demo
    0x1000,  								// BCD Device release number
    MAN_INDEX,                   			// Manufacturer string index
    PROD_INDEX,                   			// Product string index
    SN_INDEX,                   			// Serial number string index
    1                    					// Number of configurations
};

// usb configuration descriptor 			// HID CONFIGURATION
S_usb_HID_configuration_descriptor  PROGMEM usb_conf_desc = 
{
 {  sizeof(S_usb_configuration_descriptor),// Descriptor size in bytes
    CONFIGURATION_DESCRIPTOR,          	 	// Descriptor type
  	sizeof(usb_conf_desc),				    // Length of data for this cfg
    1,                      				// Number of interfaces in cfg
    1,                      				// Index for this configuration
    0,                      				// Configuration string index
    CONF_ATTRIBUTES,               			// Attributes
    50,                     				// Max power consumption (2X mA)
 },
 //Interface 0 Descriptor
 { 	sizeof(S_usb_interface_descriptor), 	// Descriptor size in bytes
    INTERFACE_DESCRIPTOR,               	// Descriptor type
    0,                      				// Interface Number
    0,                      				// Alternate Setting Number
    1,                      				// Number of endpoints in intf
    0x03,              						// Class code where HID =0x03
    0x02, 									// Subclass code - 0x02=mouse
    0x02,                 					// Protocol code - 0x02=mouse
    0,                      				// Interface string index
 },
 //Usb HID Descriptor
 { 	sizeof(S_usb_hid_descriptor),			//Descriptor size in bytes
  	0x21,									//HID descriptor
  	0x1001,									//HID_BCD
 	0,										//Country code
 	1,										//Class descriptor number
  	0x22,									//Descriptor type
	sizeof(Usb_hid_report)
 },

 //Usb Endpoint Descriptor
 { 	sizeof(S_usb_endpoint_descriptor),		//Descriptor size in bytes
  	ENDPOINT_DESCRIPTOR,					//Descriptor type
  	(EP1 | 0x80),							//ENDPOINT_NB
  	INTR,									//EP_ATTRIBUTES
 	8,										//EP size in bytes
  	0x02,									//EP_INTERVAL
 }
};
S_conf_desc	conf_desc[]={
	{sizeof(usb_conf_desc),
	(u08*)&usb_conf_desc}
};

// usb_user_language_id
S_usb_language_descriptor PROGMEM usb_user_language_id = {
	sizeof(usb_user_language_id),
 	STRING_DESCRIPTOR,
	0x0409
};

u08 Manufacturer_string[] PROGMEM = {Manufacturer};
u08 Product_string[] PROGMEM = {Product};
u08 Serial_string[] PROGMEM = {Serial};

S_string_desc 	string_desc[] = {
	{Manufacturer_string,sizeof(Manufacturer)-1},
	{Product_string,sizeof(Product)-1},
	{Serial_string,sizeof(Serial)-1}
};



