//To Do
//1) Get rid of option to enable interrupts. Always enable with null handler
//2) Shorten device structures if possible
//3) Make all of the library DEBUG code work without stdio. Smaller library
//4) Migrate to a better general I/O structure for both uart and usb.
//
//_____  I N C L U D E S ___________________________________________________ 
#include <avr/io.h>  
#include <avr/interrupt.h>
#include "include/CDC_api.h"
#include "include/typedefs.h"
#include "include/usb_api.h"
#include "include/usb_task.h"
#include "include/usb_drv.h"
#include "include/usb_descriptors.h"
#include "include/usb_standard_request.h"
#include <avr/eeprom.h>
#include <avr/power.h>				//clock prescaler
#include <avr/pgmspace.h>
#include <stdio.h>
#include "include/stdio_driver.h"
#include "include/usb_debug.h"
#define USB_STDIO

#if defined __AVR_AT90USB1287__ | __AVR_ATmega32U4__
void (*avrupdate_jump_to_boot)( void ) = (void *) 0x1e000;	//0x0E000
#elif defined __AVR_AT90USB162__
void (*avrupdate_jump_to_boot)( void ) = (void *) 0x3000;	//0x03000
#elif defined __AVR_ATmega32U4__
void (*avrupdate_jump_to_boot)( void ) = (void *) 0x6000;	//0x06000
#endif

u08 ee_bootloader EEMEM = 1;
u16 debug_timer=0;

//
//Define this function in your main application code if you want the 
//usb_OUT_Callback = Rx fifo data to be processed.
//usb_IN_Callback = Tx fifo data to be processed.
//Vendor_Callback = to process vendor specific setup requests.
//
void *OUT_Callback;
void *IN_Callback;
void *Usb_Vendor_Callback;

void usb_OUT_Callback(void *fct)
{
  	OUT_Callback = fct;
}
void usb_IN_Callback(void *fct)
{
  	IN_Callback = fct;
}
void Vendor_Callback(void *fct)
{
  	Usb_Vendor_Callback = fct;
}
u08 No_user_handler(void)
{
	return FALSE;
}

//This is driven by the 1ms start of frame requests from the host
void OneMsTimer(void)
{
u08 EP_temp;
	EP_temp = UENUM;							//SaveEP();
	if(timer1 > 0) {--timer1;}					//usb_task
	if(timer1 == 0) {usb_task(); timer1 = usb_task_time;}
	#ifdef CDC
	if(Enable_CDC_flush){
		if(timer2 > 0) {--timer2;}				//usb_task
		if(timer2 == 0) {usb_flush(); timer2 = usb_flush_time;}
	}
	#endif
	debug_timer++;
	UENUM = EP_temp;							//RestoreEP();
}

void usb_init(u08 xtal, u16 Vendorid, u16 Productid)
{
//Manage watchdog timer after the bootloader finishes
	asm ("WDR");								//stop watchdog timer
	MCUSR= !(1<<WDRF);							//Wdt_clear_flag
	WDTCSR |= (1<<WDCE) | (1<<WDE);				//Wdt_change_enable
	WDTCSR = 0x00;								//Wdt_stop

//Setup master clock
	CPU = xtal;
	if(CPU == 16){
  		clock_prescale_set(0);					//prescaler divides by 1
	}else if(CPU == 8){
   		clock_prescale_set(1);   				//prescaler divides by 2
	}
	GTCCR = 0;									//enable prescaler
	
	Enable_CDC_flush = 0;
	#ifdef CDC
	Enable_CDC_flush = 1;
	#endif
	
	usb_user_request_callback(No_user_handler);
//	usb_OUT_Callback(No_user_handler);
//	usb_IN_Callback(No_user_handler);
//	Vendor_Callback(No_user_handler);	

	debug_to_uart();							//Setup debug if requested
	SetupEndpoints();							//get the ep's from descriptor
	Vid = Vendorid;								//Setup USB VID and PID	
	Pid = Productid;
	timer1 = usb_task_time;						//Setup usb task timers
	timer2 = usb_flush_time;
//Initiallize usb software 
	usb_task_init();							//Note: this must come 1st.
												//since it enables interrupts
	UDIEN |= (1<<SOFE);							//Enable 1ms sof interrupt from host
}

u08 usb_configured(void)
{
	if(usb_configuration_nb != 0){
		return TRUE;
	}else{
		return FALSE;
	}
}	

void usb_init_ep(u08 num, u08 type, u08 dir, u08 nyet, u08 size, u08 bank, u08 interrupt)
{
	if(num < possible_eps){
		ep_init[num].valid = TRUE;
		ep_init[num].cfg0 = ((type) | (nyet) | (dir));
		ep_init[num].cfg1 = ((size) | (bank<<2));	
		ep_init[num].interrupt = interrupt;
		debug_init("\nusb_init_ep[%d]",num);
		debug_ep_init(ep_init[num].cfg0,ep_init[num].cfg1);
		debug_init("\ninterrupt = %d ",interrupt);
		debug_init("  Config OK = %d\n",ConfigOK());
	}
}

u08 Desc_size_toEP_size(u08 size)
{
u16 volatile root,i;
u08 power;
	root = size/8;
	power = 0;
	i=1;
	while(i != root){
		power = power+1;
		i = i*2;
	}
	size = power<<4;
	return size;
}

//This routine reads the descriptor table and determines how to configure
//the physical hardware. This is done through three function calls as 
//illustrated.
//1) SetupEndpoints is called to read the descriptor tables and extract 
//   the ep information
//2) usb_init_ep is called by 1 above to fill in a data stucture that 
//   holds the actual data to be written to the hardware. This is done 
//   so that the hardware can be written from the lowest to highest ep 
//   number in sequence as required by the hardware.
//3) usb_user_endpoint_init actually writes the endpoint data to the 
//   hardware. It also saves the result status in the configuration table.
void SetupEndpoints(void)
{
u08 ep,type,dir,size,bank,intr;
u08 volatile length;
u08 volatile * structure_ptr;
u08	volatile * end_of_structure;
	length = (int)(conf_desc[0].size); 
	structure_ptr = (u08*)conf_desc[0].conf_desc_ptr;
	end_of_structure = structure_ptr+length;
	while(structure_ptr < end_of_structure){
		structure_ptr += pgm_read_byte(structure_ptr);
		if(pgm_read_byte(structure_ptr+1) == ENDPOINT_DESCRIPTOR){
			ep = pgm_read_byte(structure_ptr+2)&0x0f;
			dir = pgm_read_byte(structure_ptr+2)>>7	;
			type = pgm_read_byte(structure_ptr+3)<<6;
			size = pgm_read_byte(structure_ptr+4);
			
			size = Desc_size_toEP_size(size);

			bank = (int)(EP_definition[ep].BitBank);
			intr = (int)(EP_definition[ep].BitInterrupt);
			usb_init_ep(ep, type, dir, NYET_ENABLED, size, bank, intr);

			if((type == BULK<<6) && (dir == OUT )){
				RX_EP = (int)(EP_definition[ep].BitEndpoint);};//CDC
			if((type == INTR<<6) && (dir == IN )){
				TX_EP = (int)(EP_definition[ep].BitEndpoint);};//HID
			if((type == BULK<<6) && (dir == IN )){
				TX_EP = (int)(EP_definition[ep].BitEndpoint);};//CDC
		};
	};
}

//  This function configures an endpoint with the selected type.
// Note that NYET should be enabled for USB16 to work

void usb_config_ep(u08 config0, u08 config1)
{
	Endpoint_enable_endpoint();
	UECFG0X = config0;
	UECFG1X = config1;
	Endpoint_allocate_memory();
}

// This function initializes the USB device controller and
// configures the Control Endpoint 0.
//
void usb_init_device (void)
{
u08 size;
	Endpoint_select_endpoint(EP_CONTROL);
    if(!Is_usb_endpoint_enabled()){					//only set if unallocated
		size = pgm_read_byte(&usb_dev_desc.bMaxPacketSize0);
		size = Desc_size_toEP_size(size);
		usb_config_ep((Control|OUT|NYET_ENABLED),(size|(BANK_1<<2)));
	}
}

// This function is called when the host requests that the device is 
// configured. The data to write to the hardware is stored in the 
// ep_init structure which has been filled in by the SetupEndpoints 
// call. Note that the hardware ep's are written from lowest to 
// highest as required by the AT90USB devices. 
//
// To do: Note that I need a common endpoint handler for each interrupt.
//
void usb_user_endpoint_init(u08 conf_nb)
{  	
int ep;
	for(ep = 1; ep < possible_eps; ep++){				//
		if(ep_init[ep].valid == TRUE){					//only fill actual endpoints
			Endpoint_select_endpoint(ep);
			usb_config_ep(ep_init[ep].cfg0,ep_init[ep].cfg1);
			
			debug_init("\nusb_user_endpoint_init[%d]",ep);
			debug_ep_init(ep_init[ep].cfg0,ep_init[ep].cfg1);
			debug_init("  Config OK = %d\n",ConfigOK());
			
			ep_init[ep].ConfigOK = ConfigOK();			//flag for config status
			Endpoint_reset_endpoint(ep);
			if(ep_init[ep].interrupt == RX_INT){
				UEIENX |= 1<<RXOUTE;					//enable RXOUTI for RX_EP
			}
			if(ep_init[ep].interrupt == TX_INT){
				UEIENX |= 1<<TXINE;						//enable TXINI for TX_EP
			}
		}
	}
}

//
//Note that this is not as fast as it could be. Because we don't free the 
//Rx fifo until after the packet is processed we effectively run the USB
//transfer in series with the application rather than in parallel. We could
//use two buffers to fix this or re-work the design. For now its good enough.
//
SIGNAL(USB_COM_vect)
{	
u08 EP_temp;
	EP_temp = UENUM;										//SaveEP();
  	void (*ptr)();											//
    Endpoint_select_endpoint(RX_EP);						//Select RX FIFO endpoint
    if (Endpoint_OUT_received_int() & (UEIENX & (1<<RXOUTE))){	//wait for host data
		UEINTX &= ~(1<<RXOUTI);	
		ptr = OUT_Callback;									//call application with buffer
		(*ptr)();											//data to be processed 
	}														//or (*ptr)(&buf);	

#ifndef CDC		
	Endpoint_select_endpoint(TX_EP);
	if (Is_usb_in_ready() & (UEIENX & (1<<TXINE))){			//wait for host IN request
		ptr = IN_Callback;									//call application with buffer
		(*ptr)();											//data to be processed
	};
#endif
	UENUM = EP_temp;										//RestoreEP();	
}

void bootloader_start(void)
{
	eeprom_write_byte(&ee_bootloader,0xFF); 				//force bootloader to start
  	eeprom_busy_wait ();									//
	Usb_disable();
	Usb_detach();
	avrupdate_jump_to_boot();								//Jump to bootloader sector
}

