/*
 * JTAG test library
 */
//#define TRACE	//define this is you need jtag tracing
#include "../include/LA16_V1.1.h"	//USB16 hardware definition
#include "../include/typedefs.h"	//USB16 hardware definition
#include "../include/stdio_driver.h"//console i/o
#include "../include/usb_descriptors.h"//
#include <util/delay.h>

//#define EXCLUDE


enum state
{	Test_Logic_Reset,
	Run_Test_Idle,
	Select_DR_Scan,
	Capture_DR,
	Shift_DR,
	Exit1_DR,
	Pause_DR,
	Exit2_DR,
	Update_DR,
	Select_IR_Scan,
	Capture_IR,
	Shift_IR,
	Exit1_IR,
	Pause_IR,
	Exit2_IR,
	Update_IR
};
enum state volatile JTAG_state;

#ifndef EXCLUDE
void Print_JTAG(void)
{
#ifdef LA16
	if(DDRC & (1<<TMS)){
		printf_P(PSTR("\n  TMS = out"));
	}else{
		printf_P(PSTR("\n  TMS = in "));
	};
	if(DDRC & (1<<TCK)){
		printf_P(PSTR(" TCK = out"));
	}else{
		printf_P(PSTR(" TCK = in "));
	};
	if(DDRC & (1<<TDI)){
		printf_P(PSTR(" TDI = out"));
	}else{
		printf_P(PSTR(" TDI = in "));
	};
	if(DDRC & (1<<TDO)){
		printf_P(PSTR(" TDO = out\n"));
	}else{
		printf_P(PSTR(" TDO= in\n"));
	};
#endif
	printf_P(PSTR("  TMS = %d  "),TMS_read>>TMS);
	printf_P(PSTR("  TCK = %d  "),TCK_read>>TCK);
	printf_P(PSTR("  TDI = %d  "),TDI_read>>TDI);
	printf_P(PSTR("  TDO = %d  "),TDO_read>>TDO);
	printf_P(PSTR("\n"));
}

//Test bit is a general purpose routine that tests for stuck at bits.
//This routine has only been tested with eth AT90USB162 but should
//work with other members of the AVR architecture. Check to make sure
//the SFR special function register is correct for the target hardware.
//The SFR for the PORT is defined, while the PIN SFR is two less than the
//PORT and the DDR is one less than the PORT
/*void TestBit(u08 port, u08 bit, char* string)
{
u08 PORT, save_PORT, save_DDR;
	switch(port){
	case 'B':
		PORT = 0x05;						//SFR address
		break;
	case 'C':
		PORT = 0x08;
		break;
	case 'D':
		PORT = 0x0B;
		break;
	default:
		PORT = 0x05;
		break;
	}
	save_PORT = _SFR_IO8(PORT);
	save_DDR = _SFR_IO8(PORT-1);
	_SFR_IO8(PORT-1) |= 1<<bit;		//DDR bits output;	
	printf_P(PSTR("%s high test"),string);
	_SFR_IO8(PORT) |= 1<<bit;		//PORT bits high;
	if(_SFR_IO8(PORT-2)& (1<<bit)){ //test PIN state
		printf_P(PSTR(" passed."));
	}else{
		printf_P(PSTR(" failed to go high."));
	}
	Print_JTAG();
	printf_P(PSTR("%s low test"),string);
	_SFR_IO8(PORT) &= ~(1<<bit);	//bit low
	if(!((_SFR_IO8(PORT-2)) & (1<<bit))){
		printf_P(PSTR(" passed."));
	}else{
		printf_P(PSTR(" failed to go low."));
	}
	Print_JTAG();
	_SFR_IO8(PORT) = save_PORT;
	_SFR_IO8(PORT-1) = save_DDR;
}*/
#endif
//Setup the JTAG tap pins. TDO is the output of the chain
//while TMS,TCK,TDI are inputs. Notice that we provide a 
//pullup on the TDO input.
void JTAG_Init(void)
{
	TDO_input;				//This is the tap input
	TDO_high;				//with pullups
	TDI_low;				//TDI,TMS and TCK are
	TCK_low;				//outputs
	TMS_low;				//
	TDI_output;
	TCK_output;
	TMS_output;
}

void JTAG_Off(void)			//all pins are inputs
{
	TDI_high;				//pullups
	TDO_high;
	TCK_high;
	TMS_high;
	TDI_input;				//all pins input
	TDO_input;
	TCK_input;
	TMS_input;
}

void PrintCR(void)
{
	printf_P(PSTR("\n"));
}

void PrintTDO(void)
{
	printf_P(PSTR(" TDO = %d "),(TDO_read>>TDO));
}

void PrintTDI(void)
{
	printf_P(PSTR(" TDI = %d "),(TDI_read>>TDI));
}

void PrintTMS(void)
{
	printf_P(PSTR(" TMS = %d "),(TMS_read>>TMS));
}

void PrintTCK(void)
{
	printf_P(PSTR(" TCK = %d "),(TCK_read>>TCK));
}

//This routine reports the JTAG state by printing it to the console.
//This routine needs to be called each time TCK is toggled in order 
//to keep track of the state. The variable TMS defines which state 
//the JTAG tap is supposed to transition to based on the JTAG state
//machine.
void PrintState(u08 tms)
{
	switch(JTAG_state){
	case Test_Logic_Reset:
		if(tms == 1){
			JTAG_state = Test_Logic_Reset;
			printf_P(PSTR("Test_Logic_Reset\n"));
		}else{
			JTAG_state = Run_Test_Idle;
			printf_P(PSTR("Run_Test_Idle\n"));
		}
		break;
	case Run_Test_Idle:
		if(tms == 1){
			JTAG_state = Select_DR_Scan;
			printf_P(PSTR("Select_DR_Scan\n"));
		}else{
			JTAG_state = Run_Test_Idle;
			printf_P(PSTR("Run_Test_Idle\n"));
		}
		break;
	case Select_DR_Scan:
		if(tms == 1){
			JTAG_state = Select_IR_Scan;
			printf_P(PSTR("Select_IR_Scan\n"));
		}else{
			JTAG_state = Capture_DR;
			printf_P(PSTR("Capture_DR\n"));
		}
		break;	
	case Capture_DR:
		if(tms == 1){
			JTAG_state = Exit1_DR;
			printf_P(PSTR("Exit1_DR\n"));
		}else{
			JTAG_state = Shift_DR;
			printf_P(PSTR("Shift_DR\n"));
		}
		break;	
	case Shift_DR:
		if(tms == 1){
			JTAG_state = Exit1_DR;
			printf_P(PSTR("Exit1_DR\n"));
		}else{
			JTAG_state = Shift_DR;
			printf_P(PSTR("Shift_DR\n"));
		}
		break;	
	case Exit1_DR:
		if(tms == 1){
			JTAG_state = Update_DR;
			printf_P(PSTR("Update_DR\n"));
		}else{
			JTAG_state = Pause_DR;
			printf_P(PSTR("Pause_DR\n"));
		}
		break;	
	case Pause_DR:
		if(tms == 1){
			JTAG_state = Exit2_DR;
			printf_P(PSTR("Exit2_DR\n"));
		}else{
			JTAG_state = Pause_DR;
			printf_P(PSTR("Pause_DR\n"));
		}
		break;	
	case Exit2_DR:
		if(tms == 1){
			JTAG_state = Update_DR;
			printf_P(PSTR("Update_DR\n"));
		}else{
			JTAG_state = Shift_DR;
			printf_P(PSTR("Shift_DR\n"));
		}
		break;	
	case Update_DR:
		if(tms == 1){
			JTAG_state = Select_DR_Scan;
		}else{
			JTAG_state = Run_Test_Idle;
		}
		break;	
	case Select_IR_Scan:
		printf_P(PSTR("Select_IR_Scan\n"));
		if(tms == 1){
			JTAG_state = Test_Logic_Reset;
			printf_P(PSTR("Test_Logic_Reset\n"));
		}else{
			JTAG_state = Capture_IR;
		}
		break;	
	case Capture_IR:
		if(tms == 1){
			JTAG_state = Exit1_IR;
			printf_P(PSTR("Exit1_IR\n"));
		}else{
			JTAG_state = Shift_IR;
			printf_P(PSTR("Shift_IR\n"));
		}
		break;	
	case Shift_IR:
		if(tms == 1){
			JTAG_state = Exit1_IR;
			printf_P(PSTR("Exit1_IR\n"));
		}else{
			JTAG_state = Shift_IR;
			printf_P(PSTR("Shift_IR\n"));
		}
		break;	
	case Exit1_IR:
		if(tms == 1){
			JTAG_state = Update_IR;
			printf_P(PSTR("Update_IR\n"));
		}else{
			JTAG_state = Pause_IR;
			printf_P(PSTR("Pause_IR\n"));
		}
		break;	
	case Pause_IR:
		if(tms == 1){
			JTAG_state = Exit2_IR;
			printf_P(PSTR("Exit2_IR\n"));
		}else{
			JTAG_state = Pause_IR;
			printf_P(PSTR("Pause_IR\n"));
		}
		break;	
	case Exit2_IR:
		if(TMS == 1){
			JTAG_state = Update_IR;
			printf_P(PSTR("Update_IR\n"));
		}else{
			JTAG_state = Shift_IR;
			printf_P(PSTR("Shift_IR\n"));
		}
		break;
	case Update_IR:
		if(tms == 1){
			JTAG_state = Select_DR_Scan;
			printf_P(PSTR("Select_DR_Scan\n"));
		}else{
			JTAG_state = Run_Test_Idle;
			printf_P(PSTR("Run_Test_Idle\n"));
		}
		break;
	};
}


//This function clocks the TCK and sets the TDI and TMS
//bits as requested. The state of TMS allows the tap macro 
//to move to the next state on each TCK. The value of the TDO
//input is returned to the caller. TDI is the data input
//to the tap chain and is clocked on the rising edge of TCK.
u08 JTAG_clock(u08 tdi, u08 tms)
{
u08 volatile result;
#ifdef TRACE
	PrintState(tms);
#endif
	if(tdi){				//set data out
		TDI_high;
	}else{
		TDI_low;
	};
	if(tms){				//set tap control state
		TMS_high;
	}else{
		TMS_low;
	};			
	TCK_high;				//clock TDI,TMS data out
	asm volatile("nop");
	asm volatile("nop");
	TCK_low;
	asm volatile("nop");
	asm volatile("nop");
	result = ((TDO_read >>TDO)&0x01);
	return result;//read TDO bit into bit 0;falling edge
}

//Clocking TCK 5 times when TMS is set high forces the tap
//logic into its reset state.
void JTAG_reset_tap(void)
{
u08 i;
	for(i=0;i<5;i++){
		JTAG_clock(0,TMS_1);
	}
}

//Read a 32 bit pattern from the JTAG TDO pin
uint32_t JTAG_read32(void)
{
u08 volatile i,temp;
uint32_t volatile result = 0;
uint32_t volatile shift=1;
	for(i=0;i<32;i++){
		temp = JTAG_clock(0,0);
		printf("%d",temp);
		if(temp){
			result = (uint32_t)result+shift;
		}
		shift *= 2;
	}
	printf_P(PSTR("\n"));
	return result;
}
#ifndef EXCLUDE
//This routine initiallizes the tap to the reset state;
//then it walks thru the states to the "shift_IR" instruction 
//register state. In this state each TCK that occurs with 
//TMS low will clock TDI into the instruction register and TDO
//out of the instruction register.
void JTAG_shift_IR(void)
{
	JTAG_reset_tap();		//reset-state
	JTAG_clock(0,0);		//run-test idle
	JTAG_clock(0,TMS_1);	//select DR-scan
	JTAG_clock(0,TMS_1);	//select IR-scan
	JTAG_clock(0,0);		//capture-IR
//Don't actually enter the shift state since this 
//will cause data to be transfered into the chain.
//	JTAG_clock(0,0);		//shift-IR
}

//This routine initiallizes the tap to the reset state;
//then it walks thru the states to the "shift_DR" data
//register state. In this state each TCK that occurs with 
//TMS low will clock TDI into the data register and TDO
//out of the data register.
void JTAG_shift_DR(void)
{
	JTAG_reset_tap();		//reset-state
	JTAG_clock(0,0);		//run-test idle
	JTAG_clock(0,TMS_1);	//select DR-scan
	JTAG_clock(0,0);		//capture-DR
//Don't actually enter the shift state since this 
//will cause data to be transfered into the chain.
//	JTAG_clock(0,0);		//shift-DR
}

//Detect the number of devices in the JTAG chain.
u08 JTAG_detect_length(void)
{
u16 i;
u08 temp;
		JTAG_Init();
		JTAG_shift_IR();
		//Bypass the DR by filling the instruction register with 1's
		for(i=0;i<100;i++){
#ifdef TRACE
			printf_P(PSTR("TDI = %d TDO = %d "),1,JTAG_clock(1,0));
#else
			JTAG_clock(1,0);
#endif
		}//BYPASS data register
#ifdef TRACE
		printf_P(PSTR("TDI = %d TDO = %d "),1,JTAG_clock(1,1));	//TMS must be high when writing last bit
		printf_P(PSTR("TDI = %d TDO = %d "),1,JTAG_clock(1,1));	//update IR
		printf_P(PSTR("TDI = %d TDO = %d "),1,JTAG_clock(1,1)); //select DR scan
		printf_P(PSTR("TDI = %d TDO = %d "),1,JTAG_clock(1,0));	//capture DR
#else
		JTAG_clock(1,1);	//TMS must be high when writing last bit
		JTAG_clock(1,1);	//update IR
		JTAG_clock(1,1); 	//select DR scan
		JTAG_clock(1,0);	//capture DR

#endif
		for(i=0;i<100;i++){					//flush DR register with 0
#ifdef TRACE			
			printf_P(PSTR("TDI = %d TDO = %d "),0,JTAG_clock(0,0));
#else
			JTAG_clock(0,0);
#endif
		}
		for(i=1;i<100;i++){
			temp = JTAG_clock(1,0);
#ifdef TRACE
			printf_P(PSTR("TDI = %d TDO = %d "),1,temp);
#endif
			if(temp){break;}
		};
		JTAG_reset_tap();
	return i;
}

void JTAG_report_IDCODE(void)
{
u32	result;
	printf_P(PSTR("\nIDCODE  "));
	result = JTAG_read32();
	printf_P(PSTR("Manufacturer ID = 0x%x\n"),(u16)((result)&0x02FF));
	if(((result)&0x02FF) == 0x3f){printf_P(PSTR("Atmel\n"));}
	if(((result)&0x02FF) == 0x93){printf_P(PSTR("Xilinx\n"));}
	printf_P(PSTR("Part Number     = 0x%x\n"),(u16)((result>>12)&0xffff));
	printf_P(PSTR("Version         = 0x%x\n"),(u08)(result>>28)&0x0f);
}
#endif





