/*
 * Debug routines
 */
#include "../include/typedefs.h"			//USB16 hardware definition
#include "../include/usb_debug.h"
#include "../include/usb_descriptors.h"		//
#include "../include/usb_standard_request.h"//line_coding
#include "../include/usb_drv.h"
#include "../include/CDC_api.h"


#ifdef DEBUG_EP
void debug_ep_int(u08 ep)
{
	Usb_select_endpoint(ep);
	printf_P(PSTR("Endpoint %d UEINTX 0x%x "),ep,UEINTX);
	if(UEINTX&1<<FIFOCON)	{printf_P(PSTR(" FIFOCON     "));};
	if(UEINTX&1<<NAKINI)	{printf_P(PSTR(" Nak IN      "));};
	if(UEINTX&1<<RWAL)		{printf_P(PSTR(" R/W allowed "));};
	if(UEINTX&1<<NAKOUTI)	{printf_P(PSTR(" Rx Nak out  "));};
	if(UEINTX&1<<RXSTPI)	{printf_P(PSTR(" Rx stop     "));};
	if(UEINTX&1<<RXOUTI)	{printf_P(PSTR(" Rx OUT      "));};
	if(UEINTX&1<<STALLEDI)	{printf_P(PSTR(" Stalled     "));};
	if(UEINTX&1<<TXINI)	{printf_P(PSTR(" Tx IN       "));};
	UEINTX &= ~(1<<NAKINI | 1<<NAKOUTI);		//clear NAK for before next print
	printf_P(PSTR("\n"));
}
#endif

#ifdef DEBUG_EP
void debug_dev_int(void)
{
	printf_P(PSTR("UERST 0x%x\n"),UERST);
	printf_P(PSTR("UDINT ="));
	if(UDINT&1<<UPRSMI)	{printf_P(PSTR(" Upstrean resume "));};
	if(UDINT&1<<EORSMI)	{printf_P(PSTR(" End of resume   "));};
	if(UDINT&1<<WAKEUPI)	{printf_P(PSTR(" Wakeup          "));};
	if(UDINT&1<<EORSTI)	{printf_P(PSTR(" End of reset    "));};
	if(UDINT&1<<SOFI)		{printf_P(PSTR(" Start of frame  "));};
	if(UDINT&1<<SUSPI)		{printf_P(PSTR(" Suspend         "));};
	printf_P(PSTR("\n"));
}
#else
#define debug_dev_int()
#endif

void print_line_code(void)
{
	printf_P(PSTR("     Baud: %lu bps "),(u32)line_coding.dwDTERate);
	switch(line_coding.bCharFormat){
	case 0:
		printf_P(PSTR(" 0"));
		break;
	case 1:
		printf_P(PSTR(" 1.5"));
		break;
	case 2:
		printf_P(PSTR(" 2"));
		break;
	};
	printf_P(PSTR(" stop bits Parity: "));
	switch(line_coding.bParityType){
	case 0:
		printf_P(PSTR("none"));
		break;
	case 1:
		printf_P(PSTR("odd"));
		break;
	case 2:
		printf_P(PSTR("even"));
		break;
	case 3:
		printf_P(PSTR("mark"));
		break;
	case 4:
		printf_P(PSTR("space"));
		break;
	};
	printf(" data bits: %d\n",line_coding.bDataBits);
}

//Test bit is a general purpose routine that tests for stuck at bits.
//This routine has only been tested with eth AT90USB162 but should
//work with other members of the AVR architecture. Check to make sure
//the SFR special function register is correct for the target hardware.
//The SFR for the PORT is defined, while the PIN SFR is two less than the
//PORT and the DDR is one less than the PORT
void TestBit(u08 port, u08 bit, char* string)
{
u08 PORT, save_PORT, save_DDR;
	switch(port){
	case 'B':
		PORT = 0x05;						//SFR address
		break;
	case 'C':
		PORT = 0x08;
		break;
	case 'D':
		PORT = 0x0B;
		break;
	default:
		PORT = 0x05;
		break;
	}
	save_PORT = _SFR_IO8(PORT);
	save_DDR = _SFR_IO8(PORT-1);
	_SFR_IO8(PORT-1) |= 1<<bit;		//DDR bits output;	
	printf_P(PSTR("%s high test"),string);
	_SFR_IO8(PORT) |= 1<<bit;		//PORT bits high;
	if(_SFR_IO8(PORT-2)& (1<<bit)){ //test PIN state
		printf_P(PSTR(" passed."));
	}else{
		printf_P(PSTR(" failed to go high."));
	}
	printf_P(PSTR("%s low test"),string);
	_SFR_IO8(PORT) &= ~(1<<bit);	//bit low
	if(!((_SFR_IO8(PORT-2)) & (1<<bit))){
		printf_P(PSTR(" passed."));
	}else{
		printf_P(PSTR(" failed to go low."));
	}
	_SFR_IO8(PORT) = save_PORT;
	_SFR_IO8(PORT-1) = save_DDR;
}
