/*
 * ECE 4760 Final Project
 *
 * RFID Checkout Lane
 * Kevin Yang (ky238) and Jigar Shah (jjs367)
 *
 */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lcd_lib.h"

#include <util/delay.h>

#include "uart.h"

#define begin {
#define end }
#define SERIAL_BUF_SIZE 21
#define LIST_LENGTH 11
#define CUSTOMER_LENGTH 4

//task subroutines
void initialize(void); 						//all the usual mcu stuff 
int find(char[20]);							//declares the find function
int findCustomer(char[10]);
int scanned(int);

volatile unsigned int time1;				// 1ms time base
volatile unsigned int i = 0; 				// buffer put cursor
char serialBuffer[SERIAL_BUF_SIZE];			// serial buffer

const int8_t LCD_on[] PROGMEM = 		"RFID on!\0";
const int8_t LCD_welcome[] PROGMEM =	"ECE476 Checkout Lane\0";
const int8_t LCD_instruct[] PROGMEM =	"   Please scan ID.  \0";
const int8_t LCD_hello[] PROGMEM = 		"Hello,\0";
const int8_t LCD_up[] PROGMEM = 		"^\0";
const int8_t LCD_down[] PROGMEM = 		"v\0";
const int8_t LCD_price[] PROGMEM = 		"Total: \0";
const int8_t LCD_confirm[] PROGMEM =	"Confirm Transaction?\0";
const int8_t LCD_thanks[] PROGMEM =		"Thanks for shopping!\0";
int8_t lcd_buffer[20];						// LCD buffer
int8_t lcd_buffer2[20];						// LCD buffer

int bit;									// individual bits received
int tagBegin = 0;							// has the tag begun?
int nextFlag = 0;							// indicates a new flag

float total = 0.0;							// total price
int stage = 0;								// indicates stage

int selectorIdx = 1;
int currIdx = 0;
int ready = 0;								// used for debouncing

// list of recognized customer RFID tags
char customerRFID[CUSTOMER_LENGTH][20] = {	{'1','4','0','1','E','0','0','5','4','0','0','0','0','1','1','3','3','0','5','0'},
								{'1','4','0','1','6','0','0','5','0','0','0','0','0','0','0','1','2','8','4','1'},
								{'1','4','0','1','E','0','0','5','4','0','0','0','0','1','1','3','3','5','6','9'},
								{'1','4','0','1','E','0','0','5','4','0','0','0','0','1','1','3','3','1','7','C'}};


// list of customers' names
char customers[CUSTOMER_LENGTH][10] = {	{'K','e','v','i','n',' ','Y','a','n','g'},
										{'J','i','g','a','r',' ','S','h','a','h'},
										{'B','r','u','c','e',' ','L','a','n','d'},
										{'J','a','n','e',' ','D','o','e',' ',' '}};

// list of recognized inventory tags		
char inventory[LIST_LENGTH][20] = {	{'1','4','0','2','0','1','0','0','0','0','0','0','4','8','7','1','E','6','5','7'},
									{'1','4','0','2','0','1','0','0','0','0','0','0','4','8','7','1','E','6','5','8'},
									{'1','4','0','2','0','1','0','0','0','0','0','0','4','8','7','1','E','6','3','B'},
									{'1','4','0','2','0','1','0','0','0','0','0','0','4','8','7','1','E','6','3','C'},
									{'1','4','0','2','0','1','0','0','0','0','0','0','4','8','7','1','E','6','3','D'},
									{'1','4','0','1','E','0','0','4','0','1','0','0','0','2','2','C','6','8','A','3'},
									{'1','4','0','1','E','0','0','4','0','1','0','0','0','2','2','C','2','B','3','6'},
									{'1','4','0','1','E','0','0','4','0','1','0','0','0','2','2','C','6','6','0','B'},
									{'1','4','0','1','E','0','0','4','0','1','0','0','0','2','2','C','2','C','D','8'},
									{'1','4','0','1','E','0','0','4','0','1','0','0','0','2','2','C','7','C','5','8'},
									{'1','4','0','1','E','0','0','4','0','1','0','0','0','2','2','C','7','1','6','0'}};

// list of items (in same order as corresponding tags)
char items[LIST_LENGTH][20] = {	{'S','m','a','r','t','i','e','s',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '},
								{'S','m','a','r','t','i','e','s',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '},
								{'G','a','r','l','i','c',' ','C','h','i','p','s',' ',' ',' ',' ',' ',' ',' ',' '},
								{'C','h','o','c','o','l','a','t','e',' ','C','a','n','d','y',' ','B','a','r',' '},
								{'M','&','M','s',' ','C','a','n','d','y',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '},
								{'S','o','d','a',' ','P','o','p',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '},
								{'I','t','e','m','2',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '},
								{'I','t','e','m','3',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '},
								{'I','t','e','m','4',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '},
								{'I','t','e','m','5',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '},
								{'I','t','e','m','7',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '},
								};

// stores the previous item for display
char prevItem[20];

// list of prices (in same order as corresponding tags)
float prices[LIST_LENGTH] = {1.0,
							1.0,
							1.0,
							1.0,
							1.0,
							1.0,
							1.0,
							1.0,
							1.0,
							1.0,
							1.0
							};

// keeps track of scanned items (no double scans)
int itemScanned[20];
int itemIndex = 0;

FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);


// UART character ready ISR
// triggers when there is a change to UART input
ISR (USART0_RX_vect) {
	// length is all stuff between <lf> tags - <cr>
	// for <cr> it exceeds serial buffer, causes nextFlag to be set
	bit = UDR0;
	if (i < SERIAL_BUF_SIZE && bit !='\r')	// '\r' will only be encountered in a response to a command, never reached in normal operation
	begin
		
		if (bit == '\n')					// '\n' indicates beginning of new tag
		begin
			if (tagBegin == 0)
				tagBegin = 1;
			else if (tagBegin == 1)
				tagBegin = 0;
		end
		if (tagBegin == 1 && bit != '\n') // prevents '\n' from being written into buffer
		begin
			serialBuffer[i] = bit;
			i++;
		end	
	end
	else
	begin
		// new tag is available
		nextFlag = 1;
		i = 0;
	end
}


/*
 *Entry point and task scheduler loop
 */
int main(void)
begin  
	initialize();    
	
	// sends signal
	fprintf(stdout, "\r0B1400\r");
	LCDclr();

	while(1)
	begin
		
		// gets message from chip indicating RFID reading is off
		if (serialBuffer[0] == '9' && serialBuffer[1] == 'C')
		begin
			fprintf(stdout, "\r0B1400\r");	// turns reading on
			nextFlag = 0;
		end

		// gets message from chip indicating RFID reading is on
		if (serialBuffer[0] == '1' && serialBuffer[1] == 'C')
		begin
			CopyStringtoLCD(LCD_on, 0,0);
			nextFlag = 0;					// does nothing, resets the nextFlag
		end
		
	
		switch(stage)
		begin
			case 0:								// scans customer loyalty card
			begin
				CopyStringtoLCD(LCD_welcome, 0, 1);		// displays title
				CopyStringtoLCD(LCD_instruct, 0, 2);	// displays instructions
				if (nextFlag == 1)				// get the next RFID
				begin			
					LCDclr();
					// looks for the customer
					int customerNo = findCustomer(serialBuffer);
					if (customerNo != 99)		// is customer valid?
					begin
						// writes the customer's name to the screen
						for (int idx = 0; idx < 10; idx++)
						begin
							sprintf(lcd_buffer, "%c", customers[customerNo][idx]);
							LCDGotoXY(idx+7,0);
							LCDstring(lcd_buffer, strlen(lcd_buffer));
							CopyStringtoLCD(LCD_hello, 0, 0);
						end
						stage = 1;	// goes to next stage
					end
					nextFlag = 0;
				end
				break;
			end
			case 1:
			begin								// item scanning stage
				if (nextFlag == 1)				// is there a new RFID tag?
				begin
					// looks for item
					int answer = find(serialBuffer);
					if (scanned(answer) == 1)		// if the item has already been scanned, prevent double-counting
					begin
						answer = 99;
					end
					if (answer != 99)				// is item new and valid?
					begin
						// prints item to screen
						// allows the past two items to be displayed to the screen
						for (int idx = 0; idx < 20; idx++)
						begin
							sprintf(lcd_buffer, "%c", items[answer][idx]);
							sprintf(lcd_buffer2, "%c", prevItem[idx]);
							prevItem[idx] = items[answer][idx];
							if (itemIndex == 1)
							begin
								LCDGotoXY(idx+2,1);
								LCDstring(lcd_buffer, strlen(lcd_buffer));
							end
							else
							begin
								LCDGotoXY(idx+2,1);
								LCDstring(lcd_buffer2, strlen(lcd_buffer));
								LCDGotoXY(idx+2,2);
								LCDstring(lcd_buffer, strlen(lcd_buffer));
								CopyStringtoLCD(LCD_up, 0, 1);
								CopyStringtoLCD(LCD_down, 0, 2);
							end
						end
						total += prices[answer];	// add the price
						// display the price
						CopyStringtoLCD(LCD_price, 0, 3);
						sprintf(lcd_buffer, "$%.2f", total);
						LCDGotoXY(7,3);
						LCDstring(lcd_buffer, strlen(lcd_buffer));
					end
					currIdx = itemIndex;
					nextFlag = 0;
				end

				// button pressing/debouncing
				if ((PINA == 0b00000111) & ready)	// OK button lifted, accepted
				begin
					ready = 0;
					stage = 2;
					LCDclr();
				end	
				if (PINA == 0b00000011)				// OK button pressed
					ready = 1;
				break;
			end
			case 2:											// Confirmation of transaction
			begin								
				// draws options
				CopyStringtoLCD(LCD_confirm, 0, 0);					// Confirm transaction?
				sprintf(lcd_buffer, "Confirm");						// Confirm
				LCDGotoXY(5,1);
				LCDstring(lcd_buffer, strlen(lcd_buffer));
				sprintf(lcd_buffer, "Cancel");						// Cancel
				LCDGotoXY(5,2);
				LCDstring(lcd_buffer, strlen(lcd_buffer));

				
				sprintf(lcd_buffer, ">");							// draws selector cursor
				LCDGotoXY(4,selectorIdx);
				LCDstring(lcd_buffer, strlen(lcd_buffer));

				if ((PINA == 0b00000111) & ready)					// code that accepts OK key (once the key has been lifted)
				begin
					ready = 0;
					if (selectorIdx == 1)
						stage = 3;
					if (selectorIdx == 2)
						stage = 0;
					LCDclr();					
				end

				if (PINA == 0b00000101)								// down button is pressed
				begin
					sprintf(lcd_buffer, " ");						// clears selector cursor's previous position
					LCDGotoXY(4,selectorIdx);
					LCDstring(lcd_buffer, strlen(lcd_buffer));
					selectorIdx++;
					if (selectorIdx == 3) selectorIdx = 2;			// prevents overrun
				end
				else if (PINA == 0b00000110)						// up button is pressed
				begin
					sprintf(lcd_buffer, " ");						// clears selector cursor's previous position
					LCDGotoXY(4,selectorIdx);
					LCDstring(lcd_buffer, strlen(lcd_buffer));
					selectorIdx--;
					if (selectorIdx == 0) selectorIdx = 1;			// prevents overrun
				end
				else if (PINA == 0b00000011)						// OK button is pressed, waits for button lift before accepting
				begin
					ready = 1;
				end
				break;
			end
			case 3:													// Thank you for your purchase!
			begin	
				CopyStringtoLCD(LCD_thanks, 0, 1);
				if ((PINA == 0b00000111) & ready)					// button lift detected
				begin
					ready = 0;
					stage = 0;										// goes to beginning stage again
					LCDclr();
				end
				
				if (PINA == 0b00000011)								// OK button pressed, waits for button lift before moving on
				begin
					ready = 1;
				end
				break;
			end
		end	// end switch statement
		nextFlag = 0;

		
	end // end while
end  // end main

/*
 * checks if scanned before, adds to list
 * Returns 1 if done before, else returns 0
 *
 */
int scanned(int itemNo)
begin
	for (int j = 0; j < 20; j++)
	begin
		if (itemNo == itemScanned[j])
		begin
			return 1;	// if found
		end
	end
	itemScanned[itemIndex] = itemNo;	// not found, new item
	itemIndex++;
	return 0;
end

/*
 * find customers' RFID
 */
int findCustomer(char item[10])
begin
	for (int j = 0; j < CUSTOMER_LENGTH; j++)
	begin
		for (int i = 0; i < 20; i++)
		begin
			if (item[i] != customerRFID[j][i])
				i = 20;
			if (i == 19)
				return j;
		end
	end
	return 99;
end


/*
 * find RFID
 */
int find(char item[20])
begin
	for (int j = 0; j < LIST_LENGTH; j++)
	begin
		for (int i = 0; i < 20; i++)
		begin
			if (item[i] != inventory[j][i])
				i = 20;
			if (i == 19)
				return j;
		end
	end
	return 99;
end


/*
 * initializes LCD 
 */
void init_lcd(void)
begin
	LCDinit();				//initialize the display
	LCDcursorOFF();
	LCDclr();				//clear the display
end
 
/*
 *Initialize
 */
void initialize(void)
begin
	//init the uart, in uart.c
	uart_init();
	stdout = stdin = stderr = &uart_str;

	DDRA = 0b11111000;

	//enable RX char interrupt to asyncrhonously receive
	UCSR0B |= (1<<RXCIE0);

	init_lcd();
	LCDclr(); 

	//crank up the ISRs
	sei();
end
