/* Virtual alarm
   Custodio Gomes Barcellos 
   ATMEGA8 11/03/2010    
   clock 16Mhz
   compiler CodeVision 2.03
 */

#include <mega8.h> 
#include <delay.h>   

#define xtal 16000000   // define cristal frequency
#define SENSOR 25       // sensibility adjust

// work variables                                                                     
bit C1, C2, C3, C4, cfg1, cfg2, cfg3, cfg4; // CURSOR FLAGS

 // hold variables horz, vert and cursor level     
unsigned int h, v, ch1,ch2, ch3, ch4, cv1, cv2, cv3, cv4, vc1, vc2, vc3, vc4, flag;


void v_pos(unsigned int t) // delay for position on line
{
int i;
t = t/4;     
i=0;
do{
        i++;
}while(i != t);
} 

interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
TCNT0 = 0;
            PORTB.2 = ~PORTB.2;   // Audio output     
}             

interrupt [ADC_INT] void adc_isr(void)
{
     // activated by cursors
               if(C1){ // SENSOR adjust A/D sensibility
                      if(ADCW > (vc1 + SENSOR) || ADCW < (vc1 - SENSOR)){
                                PORTB.0 = 1; // audio and led output
                                vc1 = ADCW; // hold video level on this cursor
                        }
                }         
                
                if(C2){ // SENSOR adjust A/D sensibility            
                      if(ADCW > (vc2 + SENSOR) || ADCW < (vc2 - SENSOR)){
                                PORTD.7 = 1;   // audio and led output
                                vc2 = ADCW; // hold video level on this cursor
                        }
                }
                if(C3){ // SENSOR adjust A/D sensibility
                      if(ADCW > (vc3 + SENSOR) || ADCW < (vc3 - SENSOR)){
                                PORTD.6 = 1;   // audio and led output
                                vc3 = ADCW; // hold video level on this cursor
                        }
                }
                if(C4){ // SENSOR adjust A/D sensibility                                        
                      if(ADCW > (vc4 + SENSOR) || ADCW < (vc4 - SENSOR)){                
                                PORTD.5 = 1;   // audio and led output
                                vc4 = ADCW;  // hold video level on this cursor
                        }
                } 
}

interrupt [EXT_INT0] void ext_int0_isr(void)
{     
       if(h == ch1){ // select active line
                v_pos(cv1);      // position horizontal on the line
                PORTD.3 = 1;      // on pin. output pulse pin for sample hold and video point
                PORTD.3 = 1;      // repeat for more visible point                
                PORTD.3 = 0;          // off pin
                if(v == 1){     // sample only one cursor each vertical
                        ADCSRA = ADCSRA | 0x40; // init a/d conversion on pulse position
                        C1 = 1; //     FLAG FOR INDICATION ACTIVE CURSOR
                        C2 = 0;
                        C3 = 0;
                        C4 = 0;  
                }
      }
      
      if(h == ch2){                             // THE SAME COMMENTS ABOVE
                v_pos(cv2);    
                PORTD.3 = 1;   
                PORTD.3 = 1;                   
                PORTD.3 = 0;   
                if(v == 2){    
                        ADCSRA = ADCSRA | 0x40; 
                        C1 = 0; 
                        C2 = 1;
                        C3 = 0;
                        C4 = 0;                
                }
      } 

      if(h == ch3){ 
                v_pos(cv3);     
                PORTD.3 = 1;    
                PORTD.3 = 1;                    
                PORTD.3 = 0;    
                if(v == 3){     
                        ADCSRA = ADCSRA | 0x40; 
                        C1 = 0; 
                        C2 = 0;
                        C3 = 1;
                        C4 = 0;
                }
      } 
      
      if(h == ch4){ 
                v_pos(cv4); 
                PORTD.3 = 1; 
                PORTD.3 = 1;                 
                PORTD.3 = 0; 
                if(v == 4){  
                        ADCSRA = ADCSRA | 0x40; 
                        C1 = 0; 
                        C2 = 0;
                        C3 = 0;
                        C4 = 1;                
                }
      }                           
h++;    // horizontal line counter
}                        

void main(void)
{  
                          
DDRD = 0b11101011; // Vertical and horizontal inputs
DDRB = 0b0000101; // b0 for ADC. b2 for audio.
DDRC = 0b0000000; // active pull ups port C
PORTC = 0xfe;   // active pull ups. A/D in pull up off
PORTB = 0b0000010;   // pull up on PORTB for reset button.

ADMUX = 0xC0;   // channel 0 A/D select for ADC0 and vref intern
ADCSRA = 0xCf;   // ADC on, 128, no masked and init.

TCCR0 = 0x03;  // for audio sound
TCNT0 = 0x00;
TIMSK = 0x00;

GICR = 0x40;
MCUCR = 0x02;

//initial adjusts for output pins
PORTB.0 = 0;
PORTB.2 = 0;  
h = 0;  
flag = 1;

           
// inicial position for cursors
ch1 = 132;
ch2 = 142;
ch3 = 152;
ch4 = 162;
cv1 = 6;
cv2 = 16;
cv3 = 26;
cv4 = 36;     

// flags indicate which cursors are under adjust 
cfg1 = 1;
cfg2 = 0;
cfg3 = 0;
cfg4 = 0;         

// global enable interrupts
#asm
    sei
#endasm

for(;;)
   {    
       // Detect alarm
    if(PORTD.7 == 1 || PORTB.0 == 1 || PORTD.6 == 1 || PORTD.5 == 1){       // AD trigger and set active led
        TIMSK = 0x01; // interrupt overflow timer0 on (audio generator)     // indicate which was activated
        while(PINB.1);    // reset button. timer0 and led are activated. reset deactivate
        PORTD.7 = 0;  // all
        PORTB.0 = 0;  // leds
        PORTD.6 = 0;  // off here. 
        PORTD.5 = 0;  
        TIMSK = 0x00; // timer0 overflow off. audio off.
    }
    
      if(!(PIND.4)){  // vertical reset. all below run once a field
            h = 1;      // first horizontal line
            C1 = 0; // clear all variables on first line
            C2 = 0;
            C3 = 0;
            C4 = 0;                
            PORTB.0 = 0;
            PORTD.5 = 0;
            PORTD.6 = 0;
            PORTD.7 = 0;   
            PORTB.2 = 0;
            v++;        // increments vertical for
            if(v == 5){ // coordinate A/D conversion. only one sample for field
                v = 1;     // A/D conversion is very slow. one sample per line is the limit
            }           
            delay_ms(1); // delay blank vertical timing
                   
                   ////////// below rotines for cursor 1 position adjust

            if(!PINC.1 && cfg1){
                ch1++;  
                if(ch1 == ch2 || ch1 == ch3 || ch1 == ch4) ch1++;                
                if(ch1 > 240) ch1 = 240;  // limit horiz.               
            }
            if(!PINC.2 && cfg1){
                ch1--;                  
                if(ch1 == ch3 || ch1 == ch2 || ch1 == ch4) ch1--;                                
                if(ch1 < 45) ch1 = 45; // limit horiz.
            }   
            if(!PINC.3 && cfg1){
                cv1++;
                if(cv1 == cv3 || cv1 == cv2 || cv1 == cv4) cv1 += 3;                                                            
                if(cv1 > 300) cv1 = 300; // limit vertical
            }
            if(!PINC.4 && cfg1){     
                cv1--;
                if(cv1 == cv3 || cv1 == cv2 || cv1 == cv4) cv1 -= 3;                                                                            
                if(cv1 < 5) cv1 = 5;  // limit vertical
            }          

                   ////////// below rotines for cursor 2 position adjust

            if(!PINC.1 && cfg2){
                ch2++;  
                if(ch2 == ch1 || ch2 == ch3 || ch2 == ch4) ch2++;                                
                if(ch2 > 240) ch2 = 240;  // limit horiz.               
            }
            if(!PINC.2 && cfg2){
                ch2--;      
                if(ch2 == ch1 || ch2 == ch3 || ch2 == ch4) ch2--;                                                            
                if(ch2 < 45) ch2 = 45; // limit horiz.
            }   
            if(!PINC.3 && cfg2){
                cv2++;
                if(cv2 == cv3 || cv2 == cv1 || cv2 == cv4) cv2 += 3;                                                                            
                if(cv2 > 300) cv2 = 300; // limit vertical
            }
            if(!PINC.4 && cfg2){     
                cv2--;
                if(cv2 == cv3 || cv2 == cv1 || cv2 == cv4) cv2 -= 3;                                                                                            
                if(cv2 < 5) cv2 = 5;  // limit vertical
            }         
        
                    ////////// below rotines for cursor 3 position adjust

            if(!PINC.1 && cfg3){
                ch3++;  
                if(ch3 == ch1 || ch3 == ch2 || ch3 == ch4) ch3++;
                if(ch3 > 240) ch3 = 240;  // limit horiz.               
            }
            if(!PINC.2 && cfg3){
                ch3--;                                           
                if(ch3 == ch1 || ch3 == ch2 || ch3 == ch4) ch3--;                
                if(ch3 < 45) ch3 = 45; // limit horiz.
            }   
            if(!PINC.3 && cfg3){
                cv3++;
                if(cv3 == cv1 || cv3 == cv2 || cv3 == cv4) cv3 += 3;                                            
                if(cv3 > 300) cv3 = 300; // limit vertical
            }
            if(!PINC.4 && cfg3){     
                cv3--;
                if(cv3 == cv1 || cv3 == cv2 || cv3 == cv4) cv3 -= 3;                                                            
                if(cv3 < 5) cv3 = 5;  // limit vertical
            }          
                               
                    ////////// below rotines for cursor 4 position adjust

            if(!PINC.1 && cfg4){
                ch4++;                                          
                if(ch4 == ch1 || ch4 == ch2 || ch4 == ch3) ch4++;                
                if(ch4 > 240) ch4 = 240;  // limit horiz.               
            }
            if(!PINC.2 && cfg4){
                ch4--;      
                if(ch4 == ch1 || ch4 == ch2 || ch4 == ch3) ch4--;                                            
                if(ch4 < 45) ch4 = 45; // limit horiz.
            }   
            if(!PINC.3 && cfg4){
                cv4++;      
                if(cv4 == cv1 || cv4 == cv2 || cv4 == cv3) cv4 += 3;                                                            
                if(cv4 > 300) cv4 = 300; // limit vertical
            }
            if(!PINC.4 && cfg4){     
                cv4--;
                if(cv4 == cv1 || cv4 == cv2 || cv4 == cv3) cv4 -= 3;                                                                            
                if(cv4 < 5) cv4 = 5;  // limit vertical
            }          
         

        
            ///////////////// below rotines set adjust flags
                                             
            if(!PINC.5){       // read function button      
        
                flag++;
                if(flag > 4) flag = 1;
                while(!PINC.5); // debounce
                
                switch(flag)
                {
                      
                case 1 :       // cursors 1, 2, 3, 4
                        cfg1 = 1;
                        cfg2 = 0;
                        cfg3 = 0;
                        cfg4 = 0;         
                        PORTB.0 = 1;
                        TIMSK = 0x01; // audio on
                        delay_ms(300); // time for see led on and sound
                        TIMSK = 0x00; // audio off                       
                        break;
                case 2 :
                        cfg1 = 0;
                        cfg2 = 1;
                        cfg3 = 0;
                        cfg4 = 0;         
                        PORTD.7 = 1;
                        TIMSK = 0x01;
                        delay_ms(300);
                        TIMSK = 0x00;
                        break;                        
                case 3 :
                        cfg1 = 0;
                        cfg2 = 0;
                        cfg3 = 1;
                        cfg4 = 0;         
                        PORTD.6 = 1;
                        TIMSK = 0x01;
                        delay_ms(300);
                        TIMSK = 0x00;
                        break;                        
                case 4 :
                        cfg1 = 0;
                        cfg2 = 0;
                        cfg3 = 0;
                        cfg4 = 1;         
                        PORTD.5 = 1;
                        TIMSK = 0x01;
                        delay_ms(300);
                        TIMSK = 0x00;
                        break;                        
                }
            }
      }            
   }
}
